function varargout = Euler(varargin)
%      
% This is a GUI for displaying Euler rotations 
% R(a,b,g) = exp(-i*alpha*Iz)*exp(-i*beta*Iy)*exp(-i*gamma*Iz)
% 
% User specifies (alpha,beta,gamma) (in degrees) for two
% rotations, R1 and R2. The program computes 
% the final orientation of a set of axes that are
% rotated by R1(or R2) (Z = black, x = red, blue = y]
% It also show the individual steps of each rotation:
% first about Z by gamma, then about Y by beta,
% finally about Z by alpha;
%
% The program also computes single sets of angles (a12,b12,g12)
% and (a21,b21,g21) that correspond to R12 = R1 followed by R2
% and R21 = R2 followed by R1, respectively.

% Begin initialization code - DO NOT EDIT
gui_Singleton = 1;
gui_State = struct('gui_Name',       mfilename, ...
                   'gui_Singleton',  gui_Singleton, ...
                   'gui_OpeningFcn', @Euler_OpeningFcn, ...
                   'gui_OutputFcn',  @Euler_OutputFcn, ...
                   'gui_LayoutFcn',  [] , ...
                   'gui_Callback',   []);
if nargin && ischar(varargin{1})
    gui_State.gui_Callback = str2func(varargin{1});
end

if nargout
    [varargout{1:nargout}] = gui_mainfcn(gui_State, varargin{:});
else
    gui_mainfcn(gui_State, varargin{:});
end

% EULER M-file for Euler.fig
% End initialization code - DO NOT EDIT


% --- Executes just before Euler is made visible.
function Euler_OpeningFcn(hObject, eventdata, handles, varargin)
% This function has no output args, see OutputFcn.
% hObject    handle to figure
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
% varargin   command line arguments to Euler (see VARARGIN)

% Choose default command line output for Euler
handles.output = hObject;


createAxesButton('alfa1','\alpha ',18,'bold',[0.9,0.5]);
createAxesButton('beta1','\beta ',18,'bold',[0.9,0.5]);
createAxesButton('gamma1','\gamma ',18,'bold',[0.9,0.5]);
helptext = cell(5,1);
helptext{1} = '  ';
helptext{2} = 'This program computes and displays the single rotations  ';
helptext{3} = 'equivalent to the products of rotations R1*R2 and R2*R1,';
helptext{4} = 'where R(\alpha) = exp(-i*\alpha*I_{Z})*exp(-i*\beta*I_{Y})*exp(-i*\gamma*I_{Z}).       ';
helptext{5}=  'R12 means R1 followed by R2                          ';
createAxesButton('axes7',helptext,12,'normal',[0.9,0.8]);
param = struct('R1',eye(3),...% Cartesian rotation matrix for R1
               'R2',eye(3),...% CArtesian rotation matrix for R2
               'hfig',[],...  % handle for display figure
               'ht',[],...      % handle(s) for tensor object to be rotated
               'xzxdata',[],...
               'xzydata',[],...
               'xzzdata',[],...
               'yzxdata',[],...
               'yzydata',[],...
               'yzzdata',[],...
               'zzxdata',[],...
               'zzydata',[],...
               'zzzdata',[]);

hfig = figure(5); % draw the starting figure
v = [2 0 0]' ; z = [0 0 0]';z2 = [0 0];
hax = plot3(v,z,z,'k--',-v,z,z,'k--') ; hold on ;
hay = plot3(z,v,z,'b--',z,-v,z,'b--');
haz = plot3(z,z,v,'r--',z,z,-v,'r--');
htx = plot3([1 0],z2,z2,'k','Linewidth',3);
hty = plot3(z2,[1 0],z2,'b','Linewidth',3);
htz = plot3(z2,z2,[1 0],'r','Linewidth',3);

xzxdata = get(htx,'Xdata') ; xzydata = get(htx,'Ydata'); xzzdata = get(htx,'Zdata');
yzxdata = get(hty,'Xdata') ; yzydata = get(hty,'Ydata'); yzzdata = get(hty,'Zdata');
zzxdata = get(htz,'Xdata') ; zzydata = get(htz,'Ydata'); zzzdata = get(htz,'Zdata');
param.xzxdata = xzxdata;
param.xzydata = xzydata;
param.xzzdata = xzzdata;
param.yzxdata = yzxdata;
param.yzydata = yzydata;
param.yzzdata = yzzdata;
param.zzxdata = zzxdata;
param.zzydata = zzydata;
param.zzzdata = zzzdata;
hx = text(2.15,0.1,-0.05,'X','Fontsize',16,'Color','black');
hy = text(0,2.3,0,'Y','Fontsize',16,'Color','blue');
hz = text(-0.1,0,2.3,'Z','Fontsize',16,'Color','red');
hold off
view(-37.5,15)% makes a pleasing perspective
axis square
ht = [htx ; hty ;htz]; % handle(s) for tensor object
set(ht,'erasemode','background') ; % this speeds up the drawing
param.ht = ht;
param.hfig = hfig;
handles.param = param;
% Update handles structure
guidata(hObject, handles);
% UIWAIT makes Euler wait for user response (see UIRESUME)
% uiwait(handles.figure1);


% --- Outputs from this function are returned to the command line.
function varargout = Euler_OutputFcn(hObject, eventdata, handles) 
% varargout  cell array for returning output args (see VARARGOUT);
% hObject    handle to figure
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Get default command line output from handles structure
varargout{1} = handles.output;

function a1_Callback(hObject, eventdata, handles)
% hObject    handle to a1 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: get(hObject,'String') returns contents of a1 as text
%        str2double(get(hObject,'String')) returns contents of a1 as a double
a = eval(get(hObject,'String')); 
b = eval(get(findobj('tag','b1'),'string'));
g = eval(get(findobj('tag','g1'),'string'));
R = rmat(a,b,g);
param = handles.param;
param.R1 = R;
% Update handles structure
handles.param = param;
guidata(hObject, handles);


% --- Executes during object creation, after setting all properties.
function a1_CreateFcn(hObject, eventdata, handles)
% hObject    handle to a1 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end



function b1_Callback(hObject, eventdata, handles)
% hObject    handle to b1 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: get(hObject,'String') returns contents of b1 as text
%        str2double(get(hObject,'String')) returns contents of b1 as a double
b = eval(get(hObject,'String')); 
a= eval(get(findobj('tag','a1'),'string'));
g = eval(get(findobj('tag','g1'),'string'));
R = rmat(a,b,g);
param = handles.param;
param.R1 = R;
% Update handles structure
handles.param = param;
guidata(hObject, handles);


% --- Executes during object creation, after setting all properties.
function b1_CreateFcn(hObject, eventdata, handles)
% hObject    handle to b1 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end



function g1_Callback(hObject, eventdata, handles)
% hObject    handle to g1 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: get(hObject,'String') returns contents of g1 as text
%        str2double(get(hObject,'String')) returns contents of g1 as a double
g = eval(get(hObject,'String')); 
b = eval(get(findobj('tag','b1'),'string'));
a = eval(get(findobj('tag','a1'),'string'));
R = rmat(a,b,g);
param = handles.param;
param.R1 = R;
% Update handles structure
handles.param = param;
guidata(hObject, handles);


% --- Executes during object creation, after setting all properties.
function g1_CreateFcn(hObject, eventdata, handles)
% hObject    handle to g1 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end



function a2_Callback(hObject, eventdata, handles)
% hObject    handle to a2 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: get(hObject,'String') returns contents of a2 as text
%        str2double(get(hObject,'String')) returns contents of a2 as a double
a = eval(get(hObject,'String')); 
b = eval(get(findobj('tag','b2'),'string'));
g = eval(get(findobj('tag','g2'),'string'));
R = rmat(a,b,g);
param = handles.param;
param.R2 = R;
% Update handles structure
handles.param = param;
guidata(hObject, handles);


% --- Executes during object creation, after setting all properties.
function a2_CreateFcn(hObject, eventdata, handles)
% hObject    handle to a2 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end



function b2_Callback(hObject, eventdata, handles)
% hObject    handle to b2 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: get(hObject,'String') returns contents of b2 as text
%        str2double(get(hObject,'String')) returns contents of b2 as a double
b = eval(get(hObject,'String')); 
a = eval(get(findobj('tag','a2'),'string'));
g = eval(get(findobj('tag','g2'),'string'));
R = rmat(a,b,g);
param = handles.param;
param.R2 = R;
% Update handles structure
handles.param = param;
guidata(hObject, handles);


% --- Executes during object creation, after setting all properties.
function b2_CreateFcn(hObject, eventdata, handles)
% hObject    handle to b2 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end



function g2_Callback(hObject, eventdata, handles)
% hObject    handle to g2 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: get(hObject,'String') returns contents of g2 as text
%        str2double(get(hObject,'String')) returns contents of g2 as a double
g = eval(get(hObject,'String')); 
b = eval(get(findobj('tag','b2'),'string'));
a = eval(get(findobj('tag','a2'),'string'));
R = rmat(a,b,g);
param = handles.param;
param.R2 = R;
% Update handles structure
handles.param = param;
guidata(hObject, handles);


% --- Executes during object creation, after setting all properties.
function g2_CreateFcn(hObject, eventdata, handles)
% hObject    handle to g2 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end


% --- Executes on button press in close_button.
function close_button_Callback(hObject, eventdata, handles)
% hObject    handle to close_button (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
closereq

% --- Executes on button press in reset_button.
function reset_button_Callback(hObject, eventdata, handles)
% hObject    handle to reset_button (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
hr = [findobj('tag','a1') findobj('tag','b1') findobj('tag','g1')...
    findobj('tag','a2') findobj('tag','b2') findobj('tag','g2')...
    findobj('tag','a12') findobj('tag','b12') findobj('tag','g12')...
    findobj('tag','a21') findobj('tag','b21') findobj('tag','g21')];
set(hr,'string','0.00');
param = handles.param;
param.R1 = eye(3);
param.R2 = eye(3);
hfig = param.hfig;
figure(hfig);
v = [2 0 0]' ; z = [0 0 0]';z2 = [0 0];
hax = plot3(v,z,z,'k--',-v,z,z,'k--') ; hold on ;
hay = plot3(z,v,z,'b--',z,-v,z,'b--');
haz = plot3(z,z,v,'r--',z,z,-v,'r--');
htx = plot3([1 0],z2,z2,'k','Linewidth',3);
hty = plot3(z2,[1 0],z2,'b','Linewidth',3);
htz = plot3(z2,z2,[1 0],'r','Linewidth',3);
hx = text(2.15,0.1,-0.05,'X','Fontsize',16,'Color','black');
hy = text(0,2.3,0,'Y','Fontsize',16,'Color','blue');
hz = text(-0.1,0,2.3,'Z','Fontsize',16,'Color','red');
hold off
view(-37.5,15)% makes a pleasing perspective
axis square
ht = [htx ; hty ;htz]; % handle(s) for tensor object
set(ht,'erasemode','background') ; % this speeds up the drawing
param.ht = ht;
param.hfig = hfig;
handles.param = param;
% Update handles structure
guidata(hObject, handles);

% --- Executes on button press in show1_button.
function show1_button_Callback(hObject, eventdata, handles)
% hObject    handle to show1_button (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
param = handles.param;
figure(param.hfig) ; ht = param.ht;
set(ht,'erasemode','xor') ; % this avoids overdrawing the dotted axes
a = eval(get(findobj('tag','a1'),'String')); 
b = eval(get(findobj('tag','b1'),'string'));
g = eval(get(findobj('tag','g1'),'string'));

showsteps(handles,a,b,g,'initial')
set(ht,'erasemode','background') ; % this  overdraws the dotted axes

% --- Executes on button press in show2_button.
function show2_button_Callback(hObject, eventdata, handles)
% hObject    handle to show2_button (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
param = handles.param;
figure(param.hfig) ; ht = param.ht;
set(ht,'erasemode','xor') ; % this avoids overdrawing the dotted axes
a = eval(get(findobj('tag','a2'),'String')); 
b = eval(get(findobj('tag','b2'),'string'));
g = eval(get(findobj('tag','g2'),'string'));

showsteps(handles,a,b,g,'initial')
set(ht,'erasemode','background') ; % this  overdraws the dotted axes

% --- Executes on button press in equiv21.
function equiv21_Callback(hObject, eventdata, handles)
% hObject    handle to equiv21 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
param = handles.param;
R = param.R1*param.R2;
[a,b,g] = equiv_angles(param.R2,param.R1);

showsteps(handles,a,b,g,'initial')
set(param.ht,'erasemode','background') ; % this  overdraws the dotted axes
set(findobj('tag','a21'),'string',sprintf('%7.2f',a));
set(findobj('tag','b21'),'string',sprintf('%7.2f',b));
set(findobj('tag','g21'),'string',sprintf('%7.2f',g));

% --- Executes on button press in equiv12.
function equiv12_Callback(hObject, eventdata, handles)
% hObject    handle to equiv12 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
param = handles.param;
R = param.R2*param.R1;
[a,b,g] = equiv_angles(param.R1,param.R2);
showsteps(handles,a,b,g,'initial');
set(param.ht,'erasemode','background') ; % this  overdraws the dotted axes
set(findobj('tag','a12'),'string',sprintf('%7.2f',a));
set(findobj('tag','b12'),'string',sprintf('%7.2f',b));
set(findobj('tag','g12'),'string',sprintf('%7.2f',g));

% --- Executes on button press in show12_button.
function show12_button_Callback(hObject, eventdata, handles)
% hObject    handle to show12_button (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
param = handles.param;
R = param.R2*param.R1;
set(param.ht,'erasemode','xor') ; % this avoids overdrawing the dotted axes
a2 = eval(get(findobj('tag','a2'),'String')); 
b2 = eval(get(findobj('tag','b2'),'string'));
g2 = eval(get(findobj('tag','g2'),'string'));
a1 = eval(get(findobj('tag','a1'),'String')); 
b1 = eval(get(findobj('tag','b1'),'string'));
g1 = eval(get(findobj('tag','g1'),'string'));
[a,b,g] = equiv_angles(param.R1,param.R2);
showsteps(handles,a1,b1,g1,'initial');
pause(.5)
showsteps(handles,a2,b2,g2,'proceed');
set(findobj('tag','a12'),'string',sprintf('%7.2f',a));
set(findobj('tag','b12'),'string',sprintf('%7.2f',b));
set(findobj('tag','g12'),'string',sprintf('%7.2f',g));

% --- Executes on button press in show21_buttton.
function show21_buttton_Callback(hObject, eventdata, handles)
% hObject    handle to show21_buttton (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
param = handles.param;
R = param.R1*param.R2;
set(param.ht,'erasemode','xor') ; % this avoids overdrawing the dotted axes
a2 = eval(get(findobj('tag','a2'),'String')); 
b2 = eval(get(findobj('tag','b2'),'string'));
g2 = eval(get(findobj('tag','g2'),'string'));
a1 = eval(get(findobj('tag','a1'),'String')); 
b1 = eval(get(findobj('tag','b1'),'string'));
g1 = eval(get(findobj('tag','g1'),'string'));
[a,b,g] = equiv_angles(param.R2,param.R1);
showsteps(handles,a2,b2,g2,'initial');
pause(.5)
showsteps(handles,a1,b1,g1,'proceed');
set(findobj('tag','a21'),'string',sprintf('%7.2f',a));
set(findobj('tag','b21'),'string',sprintf('%7.2f',b));
set(findobj('tag','g21'),'string',sprintf('%7.2f',g));

% --- Executes during object creation, after setting all properties.
function a12_CreateFcn(hObject, eventdata, handles)
% hObject    handle to a12 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called


% --- Executes during object creation, after setting all properties.
function b12_CreateFcn(hObject, eventdata, handles)
% hObject    handle to b12 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called


% --- Executes during object creation, after setting all properties.
function g12_CreateFcn(hObject, eventdata, handles)
% hObject    handle to g12 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called


% --- Executes during object creation, after setting all properties.
function a21_CreateFcn(hObject, eventdata, handles)
% hObject    handle to a21 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called


% --- Executes during object creation, after setting all properties.
function b21_CreateFcn(hObject, eventdata, handles)
% hObject    handle to b21 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called


% --- Executes during object creation, after setting all properties.
function g21_CreateFcn(hObject, eventdata, handles)
% hObject    handle to g21 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

%
% UTILITY FUNCTIONS
%

function createAxesButton(tag,string,fontsize,fontweight,pos)
text_obj = text('Parent',findobj('tag',tag),...
    'Position',pos,...
    'String',string,...
    'Interpreter','tex',...
    'HorizontalAlignment','right',...
    'VerticalAlignment','middle',...
    'FontName','Times',...
    'FontSize',18,...
    'FontAngle','normal',...
    'FontWeight','bold',...
    'Color',[0 0 1]);


function R = rmat(a,b,g)
% computes 3x3 rotation matrix for Euler rotation (a,b,g) (in DEGREES!)
% uses standard Rose conventions: 
%   R(z,alfa)             -- counterclockwise about z, a degrees
%   next, R(y1,beta)      -- counterclockwise about y1, b degrees
%   finally, R(z2, gamma) -- counterclockwise about z2, g degrees
%
%   NOTE: this is implemented as the equivalent rotation
%         R = R(z,gamma)R(y,beta)R(z,alfa)
%
cg = cosd(g); sg = sind(g);
cb = cosd(b); sb = sind(b);
ca = cosd(a); sa = sind(a);
Rg=[ cg -sg 0;
    sg cg 0;
    0  0 1];
Rb=[cb   0 sb;
    0    1   0;
    -sb  0 cb];
Ra=[ ca -sa 0;
    sa ca 0;
    0   0 1];
R=Ra*Rb*Rg; 
function [a,b,g] = equiv_angles(R1,R2)
%
% [a,b,g] = equiv_angles(R1,R2)
%
% returns Euler angles [alfa beta gamma] 
% that define a single rotation R12 = R2*R1, equivalent to R1[alfa1 beta1 gamma1]
% FOLLOWED by R2[alfa2 beta2 gamma2]
%
% angles in radians, NOT degrees
%
% a=[phi, theta, psi] aka [alfa,beta,gamma] as defined by ROSE
% the rotation is psi around z, then theta around ORIGINAL y, then phi around ORIGINAL z
% this is the same as alfa around z, then beta around INTERMEDIATE y, then
% gamma aound INTERMEDIATE z as discussed for example, by Brink & Satchler
% p20
%
% by RLV


R=R2*R1;  

beta=acosd(R(3,3)); % 0 <= beta <= 180 degrees

if R(3,3)~=1 && R(3,3)~=-1 % the general case....
    % 0 <= alfa <= 2*pi
    ca=R(1,3)/sind(beta);
    sa=R(2,3)/sind(beta);
    if abs(ca) < 1.e-5
        ca = 0;
    end
    if abs(sa) < 1.e-5
        sa = 0;
    end
    if ca>=0 && sa>=0 % [0,pi/2)
        alfa=asind(sa);
    elseif ca<0 && sa>=0 %[pi/2 pi)
        alfa=acosd(ca);
    elseif ca<=0 && sa<0 %[pi 3*pi/2)
        alfa=360-acosd(ca);
    elseif sa<0 && ca >=0 %[3*pi/2 2pi)
        alfa=360+asind(sa);
    end
    % 0 <= gamma <= 2*pi
    cg=-R(3,1)/sind(beta);
    sg=R(3,2)/sind(beta);
    if abs(cg) < 1.e-5
        cg = 0;
    end
    %if abs(sg) < 1.e-5
       % sg = 0.0;
%end
    if cg>=0 && sg>=0 % [0,pi/2)
        gam=asind(sg);
    elseif cg<0 && sg>=0 %[pi/2 pi)
        gam=acosd(cg);
    elseif cg<0 && sg<=0 %[pi 3*pi/2)
        gam=360-acosd(cg);
    elseif cg>=0 && sg <0 %[3*pi/2 2pi)
        gam=360+asind(sg);
    end
elseif R(3,3)==1 % beta=0 then choose gamma = 0
    gam=0;
    sa=-R(1,2);
    ca=R(1,1);
    if ca>=0 && sa>=0 % [0,pi/2)
        alfa=asind(sa);
    elseif sa>0 && ca<=0 %[pi/2 pi)
        alfa=acosd(ca);
    elseif ca<=0 && sa<0 %[pi 3*pi/2)
        alfa=360-acosd(ca);
    elseif sa<0 && ca >=0 %[3*pi/2 2pi)
        alfa=360+asind(sa);
    end

else          %theta=pi then alfa = 0
    alfa=0;
    sg= R(2,1);
    cg=R(2,2);
    if cg>=0 && sg>=0 % [0,pi/2)
        gam=asind(sg);
    elseif sg>0 && cg<=0 %[pi/2 pi)
        gam=acosd(cg);
    elseif cg<=0 && sg<0 %[pi 3*pi/2)
        gam=360-acosd(cg);
    elseif sg<0 && cg >=0 %[3*pi/2 2pi)
        gam=360+asind(sg);
    end
end
% replace angles very close to 2*pi with zero, for cosmetic reasons
if abs(alfa-360) <= 0.00001
    alfa = 0;
end
if abs(gam-360) <= 0.00001
    gam = 0;
end
a = alfa ; b = beta ; g = gam;

function showsteps(handles,a,b,g,mode)
param = handles.param;
figure(param.hfig) ; ht = param.ht;
set(ht,'erasemode','xor') ; % this avoids overdrawing the dotted axes
switch mode
    case 'initial'
set(param.ht(1),'Xdata',param.xzxdata,'Ydata',param.xzydata,'Zdata',param.xzzdata);
set(param.ht(2),'Xdata',param.yzxdata,'Ydata',param.yzydata,'Zdata',param.yzzdata);
set(param.ht(3),'Xdata',param.zzxdata,'Ydata',param.zzydata,'Zdata',param.zzzdata);
    otherwise
end
v = [2 0 0]' ; z = [0 0 0]';
roz = [0 0 0]; ray = [0 1 0]; 
raz = [0 0 1];
for jj = 1:30
    rotate(ht,raz,g/30,roz);
    drawnow 
end
hold on ;
plot3(v,z,z,'k--',-v,z,z,'k--') ;
plot3(z,v,z,'b--',z,-v,z,'b--');
plot3(z,z,v,'r--',z,z,-v,'r--');
hold off
pause(.3)
for jj = 1:30
    rotate(ht,ray,b/30,roz);
    drawnow 
end
hold on ;
plot3(v,z,z,'k--',-v,z,z,'k--') ;
plot3(z,v,z,'b--',z,-v,z,'b--');
plot3(z,z,v,'r--',z,z,-v,'r--');
hold off
pause(0.3)
for jj = 1:30
    rotate(ht,raz,a/30,roz);
    drawnow 
end
hold on ;
plot3(v,z,z,'k--',-v,z,z,'k--') ;
plot3(z,v,z,'b--',z,-v,z,'b--');
plot3(z,z,v,'r--',z,z,-v,'r--');
hold off
set(ht,'erasemode','background') ; % this  overdraws the dotted axes

