function varargout = express(varargin)
% EXPRESS 
%      MATLAB Program to simulate effects of jump motion on NMR Line shapes
%
%      Interactions:   1) 1st Order quadrupole 
%                      2) Chemical shifts and chemical shift anisotropy
%                      3) 2nd Order Quadrupole
%
%      Experiments:    1) Static quadrupole echoes (I = 1)
%                      2) Static T1Z anisotropy    (I = 1)
%                      3) Static T1Q anisotropy    (I = 1)
%                      4) One-pulse MAS/OMAS       (I = 1 and I > 1)
%                      5) MAS/OMAS T1Z anisotropy  (I = 1)
%
%      Usage:  type "express" at the MATLAB command window and follow
%      on-screen instructions
%      

% Last Modified by GUIDE v2.5 24-Mar-2008 09:26:35

% Begin initialization code - DO NOT EDIT
gui_Singleton = 1;
gui_State = struct('gui_Name',       mfilename, ...
                   'gui_Singleton',  gui_Singleton, ...
                   'gui_OpeningFcn', @express_OpeningFcn, ...
                   'gui_OutputFcn',  @express_OutputFcn, ...
                   'gui_LayoutFcn',  [] , ...
                   'gui_Callback',   []);
if nargin && ischar(varargin{1})
    gui_State.gui_Callback = str2func(varargin{1});
end

if nargout
    [varargout{1:nargout}] = gui_mainfcn(gui_State, varargin{:});
else
gui_mainfcn(gui_State, varargin{:});
end
% End initialization code - DO NOT EDIT

% --- Executes just before express is made visible.
function express_OpeningFcn(hObject, eventdata, handles, varargin)
% This function has no output args, see OutputFcn.
% It initializes all the EXPRESS controls and defines default parameters

set(findobj('tag','delays'),'userdata',[1 2 5 10 20 50 100 2000]');
set(findobj('tag','delays'),'string',num2str([1 2 5 10 20 50 100 2000]));
% Choose default command line output for express
handles.output = hObject;
% define data structures and default parameters
param = struct('nucleus','2H',...% User-specified Nucleus
    'title','default',...      % descriptive title for the simulation 
    'B0',17.6157,...           % User-specified field strength (Tesla) (not yet implemented)
    'w0',46.06,...             % User-specified Larmor frequency
    'spin',[],...               % Spin quantum number, set by choice of nucleus (not yet implemented)
    'swh',500,...              % Spectral window (Full window, -1/(2*dw) to +[1/(2*dw)]*(1-(1/2*n)) KHz
    'nfid',256,...             % points in FID
    'zfpts',256,...            % points in FID after zero fill
    'nppe',256,...             % points per echo for QCPMG expt
    'nechoes',1,...            % number of echoes for QCPMG expt
    'n',80,...                 % input number for powder increment.
    'nact',89,...              % actual number of powder increments,eg for ZCW this is the next Fibonacci # after n
    'tiletype','zcw',...       % user-selectable tiling algorithm for powder average
    'invers',1,...             % Inversion Efficiency (not currently used)
    'sr',25.0,...              % spin rate, KHz
    'pairs',10,...             % Number of sideband pairs
    'beta_r',0.0000,...        % rotor angle, degrees OFFSET from exact magic angle
    't90',1.6,...              % 90 degree pulse width (not currently used)
    't180',3.2,...             % 180 degree pulse width (not currently used)
    'pulse',40.0,...           % Pulse Spacing
    'lb',0,...                 % Lorenztian broadening factor, Hertz
    'gb',0,...                 % Gaussian broadening factor, Hertz
    'cq',155,...               % Site-dependent quadupole coupling constants, KHz
    'eta',0.03,...             % asymmetry parameter, Hertz
    'diso',0,...               % Site-dependent isotropic chemical shifts, ppm
    'dsigcsa',0,...            % Site-dependent csa anisotropy, ppm
    'etacsa',0,...             % Site-dependent csa asymmetry, a la Haeberlen
    'wts',1,...                % populations for each site
    'time',2,...               % dwell time, microseconds
    'runTime',0,...            % elapsed CPU time for simulation
    'left_shift',0,...         % number of points for left shift
    'fid',0,...                % powder average signals accumulated here
    'buf',[],...               % buffer to hold processed signals
    'spec1',[],...             % to hold one, real-valued, fully processed spectrum
    'experiment','2H QE',...   % Experiment Type
    'currentframe',1,...       % Current frame being edited
    'sites_total',12,...       % total number of equivalent 1-frame sites
    'onek',[],...              % equivalent 1-frame K-matrix goes here
    'custom_onek',[],...       % user-edited onek goes here
    'oneangle',[],...          % equivalent 1-frame CQ site angles go here
    'oneanglecsa',[],...       % equivalent 1-frame CSA site angles go here
    'deunits','msec',...       % units for delays: usec = microsec| msec = millisec | sec
    'taulist',[],...           % relaxation delay list in seconds
    'fit_taus',[],...          % mask for which tau-values to include in T1 data fitting
    'M_list',[],...            % array of points for fitting programs: M_list(1:npts,ntau)
    'ptslist',[],...           % point numbers of entries in M_list
    'fit_label',[],...         % header for list of points to fit
    'fits_table',[],...        % table of best fit T1 Values
    'table_image',[],...       % ascii image of fits_table for window display
    'ph0',[],...               % zero order (constant) phase correction, degrees
    'ph1',[],...               % first order (linear) phase correction, degrees
    'exp_region',[],...        % [xleft xright] point limits for expanded scale plots
    'nframes',2,...            % Number of jump frames
    'mastags',[],...           % tags for MAS parameters
    'cqtags',[],...            % tags for EFG parameters
    'csatags',[],...           % tags for CSA controls
    'delaytags',[],...         % tags for MAS controls
    'frames',[],...            % reserved for frame-specific parameters (see below)
    'runflags',[0 0 0]);       % run-tme flags (see Start_run_button_Callback)
param.delays = [1     2     5    10    20    50   100  2000];% default list of relaxation delays, in msec
param.taulist = 0.001*param.delays;
set(findobj('tag','radiobutton2'),'value',0);
set(findobj('tag','radiobutton3'),'value',1);
% default includes two frames, one with three sites and one with 4
frames{1} = struct('sites',3,...                        % number of jump sites in this frame
    'template',[0 1 1;1 0 1; 1 1 0],...  % off-diagonal element (i,j) implies jumps from (col) j to (row)
    'cq_phi',[0 0 0],...                 % alpha euler angles for cq
    'cq_theta',[74 74 74],...            % beta euler angles for cq
    'cq_psi',[0 120 240],...             % gamma euler angles for cq
    'csa_phi',[0 0 0],...                % alpha euler angles for csa
    'csa_theta',[0 0 0],...              % beta euler angles for csa
    'csa_psi',[0 0 0],...                % gamma euler angles for csa
    'rate',5.e9,...                      % jump rate, s^(-1)
    'model','all');                      % model = 'all' | 'cha' | 'lin' | 'cir'
frames{2} = struct('sites',4,...                        % number of jump siyes
    'template',[0 1 1;1 0 1; 1 1 0],...  % off-diagonal element (i,j) implies jumps from (col) j to (row) i
    'cq_phi',  [0 0 0 0],...         % alpha euler angle for cq
    'cq_theta',[0 10 10 10],...          % beta euler angle for cq
    'cq_psi',[0 0 120 240],...           % gamma euler angles for cq
    'csa_phi',[0 0 0 0],...              % alpha euler angle for csa
    'csa_theta',[0 0 0 0],...            % beta euler angle for csa
    'csa_psi',[0 0 0 0],...              % gamma euler angle for csa
    'rate',1.e5,...                      % jump rate, s^(-1)
    'model','all');                      % model = 'all' | 'cha' | 'lin' | 'cir'
%
% collect tags for some parameter groups
%
mastags = [findobj('tag','edit_spin_rate') findobj('tag','text19') findobj('tag','text57') ...
    findobj('tag','pairs') findobj('tag','text21') findobj('tag','text60') ...
    findobj('tag','OMAS_offset') findobj('tag','text20') findobj('tag','text58') ...
    findobj('tag','text59')];
csatags = [findobj('tag','edit_delta_iso') findobj('tag','uipanel9') findobj('tag','text170') ...
    findobj('tag','text_span') findobj('tag','text_skew') findobj('tag','edit_span') ...
    findobj('tag','edit_skew')];
efgtags = [findobj('tag','edit_CQ') findobj('tag','uipanel7') findobj('tag','text168') ...
    findobj('tag','edit_eta_Q') findobj('tag','text169')];
delaytags = [findobj('tag','delays') findobj('tag','units') findobj('tag','text171')];
labeltags = [efgtags(3) efgtags(5) csatags(3) csatags(4) csatags(5)];
grey = [0.702 0.702 0.702]; % kludge to keep axes from messing up text labels
set(labeltags,'visible','off','xcolor',grey,'ycolor',grey);
ss = get(findobj('tag','exp_popup'),'string');
% supply descriptive name(s) for your routine(s) below
% by replacing the strings in the following three lines
% with names of your choice.
ss{11} = 'T1Zspikelet';
ss{12} = '(user routine 2)';
ss{13} = '(user routine 3)';
set(findobj('tag','exp_popup'),'string',ss);
set(findobj('tag','expt_select'),'string',ss);
param.mastags = mastags;
param.csatags = csatags;
param.efgtags = efgtags;
param.delaytags = delaytags;
set(findobj('tag','units'),'SelectionChangeFcn',@selcbk) % MATLABese for "activate radio button group" ; controls delay units
param.frames = frames; % preserve frame-specific parameters for possible use outside EXPRESS
handles.param = param;
handles.frames = frames;
% Update handles structure
guidata(hObject, handles);

% Set initial state to show "other" panel
setState('other');
showdata(handles,'other');
updatecurrentframe(handles);
fid = fopen('startup.txt', 'rt');
str = '';
while feof(fid) == 0
    tline = fgets(fid);
    str = strvcat(str,tline);
end
fclose(fid);
str = str2mat(str);
set(findobj('tag','main_text'),'string',str);
% MATLAB KLUDGE TO GET GREEK SYMBOLS INTO TEXT LABELS
createAxesButton('test_axes','\alpha :');
createAxesButton('test_axes_beta','\beta :');
createAxesButton('test_axes_gamma','\gamma :');
createAxesButton('csa_axes_alpha','\alpha :');
createAxesButton('csa_axes_beta','\beta :');
createAxesButton('csa_axes_gamma','\gamma :');
createAxesButton('inv_sec_axes','(s^{-1})');
createAxesButton('text170','\delta_{iso} (ppm)');
createAxesButton('text_span','\Delta\sigma (ppm)');
createAxesButton('text_skew','\eta_{csa}');
createAxesButton('text169','\eta_Q');
createAxesButton('text168','C_Q (kHz)');
set(findobj('tag','one_frame_panel'),'Parent',findobj('tag','site_panel'));
axis(findobj('tag','test_axes') ); set(gca,'vis','off')
% UIWAIT makes express wait for user response (see UIRESUME)
% uiwait(handles.figure1);

function createAxesButton(tag,string)
text_obj = text('Parent',findobj('tag',tag),...
    'Position',[0.9,0.5],...
    'String',string,...
    'Interpreter','tex',...
    'HorizontalAlignment','right',...
    'VerticalAlignment','middle',...
    'FontName','Helvetica',...
    'FontSize',14,...
    'FontAngle','normal',...
    'FontWeight','bold',...
    'Color',[0 0 1]);

% --- Outputs from this function are returned to the command line.
function varargout = express_OutputFcn(hObject, eventdata, handles) 
varargout{1} = handles.output;

% --- Executes on button press in help_button.
function help_button_Callback(hObject, eventdata, handles)
% takes no action; other routines examine the
% state of the help button before execution. If its "in"
% (value = 1 in matlab-ese) EXPRESS displays a help message
% instead of executing the called-for operations.

% --- Executes on button press in exit_button.
function exit_button_Callback(hObject, eventdata, handles)
% does what it says, saving no parameters
if get(findobj('tag','help_button'),'value') == 1
    printhelp('exit');
    return
end
close all

% --- Executes on button press in save_button.
function save_button_Callback(hObject, eventdata, handles)
% saves complete image of EXPRESS as a re-runnable figure
% and also saves the param data structure, both in a user-selectable folder
if get(findobj('tag','help_button'),'value') == 1
    printhelp('save')
    return
end
[filename,pathname] = uiputfile('*.fig','save as ');
set(findobj('tag','save_button'),'Value',0);
if isequal(filename,0) || isequal(pathname,0)
    warndlg('data not saved');
else
    param = handles.param;
    param.ph0 = str2num(get(findobj('tag','edit_ph0'),'string'));
    param.ph1 = str2num(get(findobj('tag','edit_ph1'),'string'));
    guidata(hObject,handles);
    set(gcbo,'userdata', param);
    hgsave(fullfile(pathname, filename));
    save( fullfile(pathname,strcat(filename,'_param.mat')), 'param');
end

% --- Executes on button press in load_button.
function load_button_Callback(hObject, eventdata, handles)
% restores a previously SAVEed EXPRESS figure
% parameters in place before the LOAD are lost
if get(findobj('tag','help_button'),'value') == 1
    printhelp('load')
    return
end
[filename, pathname] = uigetfile('figure', 'Select a processed data file');
if isequal(filename,0) || isequal(pathname,0)
   warndlg('file not found');
else
   close(findobj('Type','figure'));
   open(fullfile(pathname, filename));
end

% --- Executes on button press in run_button.
function run_button_Callback(hObject, eventdata, handles)
% activates "RUN" panel and updates one-frame parameters
% but does not actually start the simulation
if get(findobj('tag','help_button'),'value') == 1
    printhelp('run')
    return
end
%generate equivalent 1-frame angles from current parameters
param = handles.param;
frames = handles.frames;
nframes = param.nframes;
if isempty(param.onek)
    errordlg('must first define one-frame parameters')
    return
end
for n=1:nframes
    Nsite(n) = frames{n}.sites;
    Nmodel(n,:) = frames{n}.model;
    Nrate(n) = frames{n}.rate;
    Ntemplate{n} = frames{n}.template;
end
Fangle = zeros(sum(Nsite),3);
Fanglecsa = Fangle;
sitenum=1;
for n=1:nframes
    param.frames{n}.cq_phi = frames{n}.cq_phi;
    param.frames{n}.cq_theta = frames{n}.cq_theta;
    param.frames{n}.cq_psi = frames{n}.cq_psi;
    param.frames{n}.csa_phi = frames{n}.csa_phi;
    param.frames{n}.csa_theta = frames{n}.csa_theta;
    param.frames{n}.csa_psi = frames{n}.csa_psi;
    for siteadd=1:Nsite(n)
        angle1 = [pi*frames{n}.csa_phi(siteadd)/180 pi*frames{n}.csa_theta(siteadd)/180 pi*frames{n}.csa_psi(siteadd)/180];
        angle2 = [pi*frames{n}.cq_phi(siteadd)/180 pi*frames{n}.cq_theta(siteadd)/180 pi*frames{n}.cq_psi(siteadd)/180];
        Fanglecsa(siteadd+sitenum-1,:) = angle1;
        Fangle(siteadd+sitenum-1,:) = angle2;
    end
    sitenum = sitenum + Nsite(n);
end
[oneangle Tsite]=oneframeangle(Nsite,Fangle);
[oneanglecsa Tsite]=oneframeangle(Nsite,Fanglecsa);
[onek, kmatrxcell]=oneframekmatrx(Nsite,Nmodel, Nrate, Ntemplate); % re-build one-frame kmatrix
siten = size(onek,1);
for jj = 1:siten
    tmp = 0;
    for kk = 1:siten
        if kk ~= jj
            tmp = tmp -2*param.wts(kk)*onek(jj,kk)/(param.wts(jj)+param.wts(kk));
        end
    end
    onek(jj,jj) = tmp;
end
for jj = 1:siten
    for kk = 1:siten
        onek(jj,kk) = 2*onek(jj,kk)*sqrt(param.wts(jj)*param.wts(kk))/(param.wts(jj)+param.wts(kk));
    end
end   
% if isequal(get(findobj('tag','editrates'),'value'),0) % use EDITED one-frame k-Matrix
%     param.onek = param.custom_onek;
% else % use UNEDITED one-frame k-matrix
    param.onek = onek;
% end
    [vec,val] = eig(param.onek); % diagonalize K-matrix for use in spin lattice calculations
    param.onekvec = vec;
    param.onekval = diag(val);
    clear vec val;
    param.oneangle = oneangle;
    param.oneanglecsa = oneanglecsa;
    param.kmatrxcell = kmatrxcell;
    ll = size(param.oneangle,1);
    param.total_sites = ll;
    if length(param.cq) == 1
        param.cq = param.cq*ones(1,ll);
    end
    if length(param.eta) == 1
        param.eta = param.eta*ones(1,ll);
    end
    if length(param.diso) == 1
        param.diso = param.diso*ones(1,ll);
    end
    if length(param.dsigcsa) == 1
        param.dsigcsa = param.dsigcsa*ones(1,ll);
    end
    if length(param.etacsa) == 1
        param.etacsa = param.etacsa*ones(1,ll);
    end
    if length(param.wts) == 1
        param.wts = 1/param.wts*ones(1,ll)/ll;
    end
handles.param = param;
guidata(hObject, handles);
set(findobj('tag','runTime_text'),'visible','off');
showdata(handles,'site');
setState('run');
showdata(handles,'run');

% --- Executes on button press in view_output_button.
function view_output_button_Callback(hObject, eventdata, handles)
% Displays simulated time domain data and activates "view output" panel
if get(findobj('tag','help_button'),'value') == 1
    printhelp('view')
    return
end
showOutput(hObject,handles);

function showOutput(hObject,handles)
setState('output');
% plots data processed from data.fids according to current parameters
param = handles.param;
%load bestfit_T1Z.mat; 5 insert foreign data
fid = param.fid;% UNMODIFIED simulated time domain signals, one column for each tau value
%fid = sig; % kludge to read in foreign data!!
ftflag = get(findobj('tag','fft_button'),'value');
if isempty(fid)
    errordlg('nothing to plot')
    return
end
% retrieve processing parameters from edit windows (NOT param structure) since these are
% automatically restored when a saved session is recalled via the RESTORE button
ph0 = str2double(get(findobj('tag','edit_ph0'),'string'));
ph1 = str2double(get(findobj('tag','edit_ph1'),'string'));
ls = str2double(get(findobj('tag','edit_left_shift'),'string'));
lb = 1000*str2double(get(findobj('tag','em_val'),'string'));
gb = 1000*str2double(get(findobj('tag','gm_val'),'string'));
% gbe = 1000*str2double(get(findobj('tag','gme'),'string'));
sw = 1000*str2double(get(findobj('tag','edit_width'),'string'));
tau = 1.e-6*eval(get(findobj('tag','edit_pulse'),'string'));
[n , ntau] = size(param.fid); % n is the original number of points; ntau is the number of tau values
npts = param.zfpts;  % number of points with current zero fill, NOT necessarily the original number of points
[n , ntau] = size(param.fid); % n is the original number of points; ntau is the number of tau values
n2 = npts-1;
tt = [0:(n-1)]'/sw;                         % units of tt are seconds; sw is Hertz
R = pi*lb;                                  % EM
G = sign(gb)*((pi*gb/(2))^2)/log(2);        % GM
apod = exp(-R*tt).*exp(-G*tt.*tt)*ones(1,ntau);
param.buf = apod.*param.fid;
% get last point(s) for DC offset correction of one-sided DFT
last = param.buf(n,:);
% zero order phase correct in time domain,then left/right shift
param.buf = phcor(param.buf,ph0);
param.buf = shift2d(param.buf,ls);
% zero fill
param.buf = [param.buf ;zeros(npts-n,ntau)];
% FT and linear phase correction if needed
if ftflag
    param.buf(1,:) = (param.buf(1,:)+last)/2 ; % removes DC offset from one-sided DFT
    param.buf =  phcor(qft(param),0,ph1);
end
% find which spectrum has greatest intensity, and plot that one
[a b] = max(abs(param.buf));
[c d] = max(abs(a));
param.spec1 = param.buf(:,d);
if isequal(param.experiment,'2H T1Z') % plot the LAST tau-value for T1Z experiments
 param.spec1 = param.buf(:,ntau); 
end
tax = [0:(npts-1)]'/sw;
fax = ([0:(npts-1)]'-npts/2)*sw/npts;
if ftflag
    axes(handles.real_output_axes);
    plot(fax,real(param.spec1),'b');
    set(gca,'Xdir','reverse');
    axes(handles.imag_output_axes);
    plot(fax,imag(param.spec1),'r');
    set(gca,'Xdir','reverse');
else
    axes(handles.real_output_axes);
    plot(real(param.buf(:,ntau)),'b'); % use POINTS for abcissa so that user can more easily find left shifts
    axes(handles.imag_output_axes);
    plot(tax,imag(param.buf(:,ntau)),'r');
end
handles.param = param;
guidata(hObject,handles)

% --- Executes on button press in site_button.
function site_button_Callback(hObject, eventdata, handles)
% Turns on SITE DEFINITION panel with updated parameters
if get(findobj('tag','help_button'),'value') == 1
    printhelp('site')
    return
end
setState('site');
showdata(handles,'site');
updatecurrentframe(handles);

% --- Executes on button press in other_button.
function other_button_Callback(hObject, eventdata, handles)
% Turns on "OTHER" panel and updates parameters
% note clumsy code needed to update delay units as managed
% by MATLAB's uibutton group
if get(findobj('tag','help_button'),'value') == 1
    printhelp('other')
    return
end
param = handles.param;
h = findobj('tag','units');
hh = get(h,'SelectedObject');
switch get(hh,'tag')
    case 'radiobutton2'
        param.deunits = 'usec';
    case 'radiobutton3'
        param.deunits = 'msec';
    case 'radiobutton4'
        param.deunits = 'sec';
end
handles.param = param;
guidata(hObject,handles);
setState('other');
showdata(handles,'other');

function setState(str)
% this function determines which groups (panels) of controls
% are made visible or invisible in accord with user choices
set(findobj('tag','site_panel'),'visible','off');
set(findobj('tag','delays_panel'),'visible','off');
set(findobj('tag','other_panel'),'visible','off');
set(findobj('tag','run_panel'),'visible','off');
set(findobj('tag','output_panel'),'visible','off');
set(findobj('tag','site_button'),'Value',0);
set(findobj('tag','delays_button'),'Value',0);
set(findobj('tag','other_button'),'Value',0);
set(findobj('tag','run_button'),'Value',0);
set(findobj('tag','view_output_button'),'Value',0);
set(findobj('tag','analyze_panel'),'visible','off');
set(findobj('tag','one_frame_panel'),'visible','off');
expt_num = get(findobj('tag','expt_select'),'Value');
efgtags = [findobj('tag','edit_CQ') findobj('tag','uipanel7') findobj('tag','text168') ...
              findobj('tag','edit_eta_Q') findobj('tag','text169')];
          csatags = [findobj('tag','edit_delta_iso') findobj('tag','uipanel9') findobj('tag','text170') ...
              findobj('tag','text_span') findobj('tag','text_skew') findobj('tag','edit_span') ...
              findobj('tag','edit_skew')]; 
switch str
    case 'site'
        set(findobj('tag','site_panel'),'visible','on');
        set(findobj('tag','site_button'),'Value',1);
        set(findobj('tag','one_frame_panel'),'visible','off');
        if expt_num == 10
            set([findobj('tag','uipanel7') efgtags],'visible','off');% panel for EFG site angles
        else
            set([findobj('tag','uipanel7') efgtags],'visible','on');
        end
        if (expt_num < 6) || isequal(expt_num,8)
            set([csatags findobj('tag','uipanel9')],'visible','off'); % panel for CSA site angles
            if expt_num == 1
                set([csatags findobj('tag','uipanel9')],'visible','on'); % panel for CSA site angles
            end
        else
            set([csatags findobj('tag','uipanel9')],'visible','on'); % panel for CSA site angles
        end                
    case 'delays'
        set(findobj('tag','delays_panel'),'visible','on');
        set(findobj('tag','delays_button'),'Value',1);
    case 'other'
        set(findobj('tag','other_panel'),'visible','on');
        set(findobj('tag','other_button'),'Value',1);
    case 'run'
        set(findobj('tag','run_panel'),'visible','on');
        set(findobj('tag','run_button'),'Value',1);
    case 'output'
        nn = get(findobj('tag','exp_popup'),'val');
        set(findobj('tag','output_panel'),'visible','on');
        set(findobj('tag','view_output_button'),'Value',1);
    case 'analyze'
       set(findobj('tag','analyze_panel'),'visible','on');
       set(findobj('tag','delays_button'),'Value',1)
end

function showcurrentdata(handles)
% which data does user want to show in the main text window?
if get(findobj('tag','site_button'),'value') == 1
    showdata(handles,'site');
    return;
elseif get(findobj('tag','delays_button'),'value') == 1
    showdata(handles,'delays');
    return;
elseif get(findobj('tag','other_button'),'value') == 1
    showdata(handles,'other');
    return;
elseif get(findobj('tag','run_button'),'value') == 1
    showdata(handles,'run');
    return;
end

function showdata(handles,opt)
% show user-selected data in the main text window. This routine
% works in conjunction with showcurrentdata(handles) (see above)
% if the display isn't quite right on your monitor, try (careful!!)
% changes of the formatting in this routine
set(findobj('tag','main_text'),'Value',1);
frames = handles.frames;
param = handles.param;
if strcmp(opt,'site')
    if get(findobj('tag','calculate_button'),'value') == 1
        p = '1-Frame Equivalent Parameters';
            p = strvcat(p,' ');
        p = strvcat(p,sprintf('%s','    1-Frame Angles FOR EFG PAS:'));
        p = strvcat(p,'     alpha      beta     gamma      CQ        eta      d-iso      csa     eta_csa   rel. wt.');
        for n=1:param.sites_total
            alpha=180*param.oneangle(n,1)/pi;
            beta=180*param.oneangle(n,2)/pi;
            gamma=180*param.oneangle(n,3)/pi;
            cq = param.cq(n);
            eta = param.eta(n);
            diso = param.diso(n);
            dsigcsa = param.dsigcsa(n);
            etacsa = param.etacsa(n);
            wt = param.wts(n);
            p = strvcat(p,sprintf('%s%10.3f%10.3f%10.3f%10.3f%10.3f%10.3f%10.3f%10.3f%10.3f','',alpha,beta,gamma,...
                cq,eta,diso,dsigcsa,etacsa,wt));
        end
            p = strvcat(p,' ');
        p = strvcat(p,sprintf('%s','Rate Matrices:'));
        for n=1:length(param.kmatrxcell)
            p = strvcat(p,sprintf('%s%d%s','Frame ',n,':'));
            for k=1:length(param.kmatrxcell{n})
                p = strvcat(p,sprintf('%s','      ',mat2str(param.kmatrxcell{n}(k,:))));
            end
        end
    else
        p = strvcat(sprintf('%s %d','Number of Jump Frames: ',param.nframes));
        for nfr = 1:param.nframes
            p = strvcat(p,' ');
            p = strvcat(p,sprintf('%s%d','                            Frame ',nfr));
            p = strvcat(p,' ');
            p = strvcat(p,sprintf('%s',' Site Angles for EFG PAS: '));
            p = strvcat(p,sprintf('%s','    SITE        ','ALPHA      ','BETA     ',...
                'GAMMA'));
            for nsi = 1:frames{nfr}.sites
                alpha = frames{nfr}.cq_phi(nsi);
                beta = frames{nfr}.cq_theta(nsi); 
                gamma = frames{nfr}.cq_psi(nsi);
                p = strvcat(p,sprintf('%s%5d%s%10.3f%10.3f%10.3f','   ',fix(nsi),'   ',alpha,beta,gamma));
            end
            p = strvcat(p,' ');
            p = strvcat(p,sprintf('%s',' Site Angles for CSA PAS: '));
            
            p = strvcat(p,sprintf('%s','    SITE        ','ALPHA      ','BETA     ',...
                'GAMMA'));
            for nsi = 1:frames{nfr}.sites
                alpha = frames{nfr}.csa_phi(nsi);
                beta = frames{nfr}.csa_theta(nsi); 
                gamma = frames{nfr}.csa_psi(nsi);
                p = strvcat(p,sprintf('%s%5d%s%10.3f%10.3f%10.3f','   ',fix(nsi),'   ',alpha,beta,gamma));
            end
            p = strvcat(p,' ');
        end
    end
elseif strcmp(opt,'delays')
    p = 'Delay Lists (for later added experiment types)';
elseif strcmp(opt,'disabled')
    nfr = 1;
    p = sprintf('%s %d %s %10.3e %s %s','Frame ',nfr,'  Rate:',frames{nfr}.rate,'s^-1   Model:',frames{nfr}.model);
    p = strvcat(p,'Template: (i,j) = 1 implies jump from site j to site i');
    ns = frames{nfr}.sites; % now comes a string image of the rate template for frame 1
    for ii = 1:ns
        rtemp = sprintf('%s%d','  ',frames{nfr}.template(ii,1));
        for jj = 2:ns
            rtemp = [ rtemp sprintf('%d',frames{nfr}.template(ii,jj))];
        end
        p = strvcat(p,rtemp);
    end
    p = strvcat(p,' ');
    for nfr = 2:param.nframes% now comes string images of the rate templates for subsequent frames
        p = strvcat(p,sprintf('%s %d %s %10.3e %s %s','Frame ',nfr,'  Rate:',frames{nfr}.rate,'s^-1   Model:',frames{nfr}.model));
        p = strvcat(p,'Template: (i,j) = 1 implies jump from site j to site i');
        ns = frames{nfr}.sites;
        for ii = 1:ns
            rtemp = sprintf('%s%d','  ',frames{nfr}.template(ii,1));
            for jj = 2:ns
                rtemp = [ rtemp sprintf('%d',frames{nfr}.template(ii,jj))];
            end
            p = strvcat(p,rtemp);
        end
    end
elseif strcmp(opt,'other')
    p = sprintf('%s%s','              ',param.title);
    p = strvcat(p,sprintf('%s %.2f %s','               PW1-90:       ',param.t90,' microsec'));
    p = strvcat(p,sprintf('%s %.2f %s','        Pulse Spacing:       ',param.pulse,' microsec'));
    p = strvcat(p,sprintf('%s %d',     '        Points in FID:       ',param.nfid));
    p = strvcat(p,sprintf('%s %d',     '      Points per echo:       ',param.nppe));
    p = strvcat(p,sprintf('%s %d',     '     Number of echoes:       ',param.nechoes));
    p = strvcat(p,sprintf('%s %f %s',  '            Frequency:       ',param.w0,'MHz'));
    p = strvcat(p,sprintf('%s %.2f %s','                  SWH:       ',param.swh,' KHz'));
    p = strvcat(p,sprintf('%s %.2f %s','                Dwell:       ',param.time,' us'));
    p = strvcat(p,sprintf('%s %d',     'Powder increments (beta):    ',param.nact));
    if param.sr > 0
        p = strvcat(p,sprintf('%s %.2f %s','            Spin rate:       ',param.sr,' KHz'));
        beta_magic = 180*acos(sqrt(1/3))/pi;
        p = strvcat(p,sprintf('%s %.4f %s','                Angle:       ',param.beta_r+beta_magic,' Degrees'));
        p = strvcat(p,sprintf('%s %d', '       Sideband Pairs:       ',param.pairs));
    end
    p = strvcat(p,sprintf('%s %s %s %10.3f','Delay List (',param.deunits,' )',param.delays(1)));
    for jj = 2:length(param.delays)
        p = strvcat(p,sprintf('%s %10.3f','                    ',param.delays(jj)));
    end
elseif strcmp(opt,'run')
    p = strvcat(sprintf('%s','Experiment:   ',param.experiment));
    p = strvcat(p,sprintf('%s','CPU time:   ',num2str(param.runTime)));
    p = strvcat(p,sprintf('%s','powder increments:   ',num2str(param.nact)));
else
    return
end
 p = str2mat(p);
 set(findobj('tag','main_text'),'string',p);

function edit_frames_Callback(hObject, eventdata, handles)
% This routine runs when you change the number of frames
% This is a MAJOR change, you should check  the paraameters
% in all frames afterwards. In particular, parameters 
% that are set in the one-frame equivalent context will change!
if get(findobj('tag','help_button'),'value') == 1
    printhelp('frames1')
    return
end
param = handles.param ;
oldframes = param.nframes;
param.nframes = str2num(get(hObject,'String'));
handles.param = param;
frames = handles.frames;
for n=oldframes+1:param.nframes
 frames{n} = struct('sites',3,...                        % number of jump sites in this frame
                    'template',[0 1 1;1 0 1; 1 1 0],...  % off-diagonal element (i,j) implies jumps from (col) j to (row) i
                    'cq_phi',[0 120 240],...             % alpha euler angles for cq
                    'cq_theta',[74 74 74],...            % beta euler angles for cq
                    'cq_psi',[0 0 0],...                 % gamma euler angles for cq
                    'csa_phi',[0 180 0],...              % alpha euler angles for csa
                    'csa_theta',[0 60 179],...           % beta euler angles for csa
                    'csa_psi',[0 0 75],...               % gamma euler angles for csa
                    'rate',5.e3,...                      % jump rate, s^(-1)
                    'model','all');                      % model = 'all' | 'cha' | 'lin' | 'cir'
end
ns = frames{1}.sites;
for n = 2:param.nframes
    ns = ns*frames{n}.sites;
end
param.sites_total = ns;
param.cq = param.cq(1)*ones(1,ns);
param.diso= param.diso(1)*ones(1,ns);
param.dsigcsa= param.dsigcsa(1)*ones(1,ns);
param.eta= param.eta(1)*ones(1,ns);
param.etacsa= param.etacsa(1)*ones(1,ns);
param.wts = ones(1,ns)/ns;
handles.frames = frames;
handles.param = param;
guidata(hObject, handles);
showcurrentdata(handles);
updatecurrentframe(handles);

function updatecurrentframe(handles)
% this  routine updates the param data structure to 
% include changes made to individual frames. After it
% runs, you must re-edit one-frame PAS values
param = handles.param;
current = param.currentframe;
nframes = param.nframes;
if current > nframes
    current = nframes;
    param.currentframe = current;
    handles.param = param;
    guidata(handles.output, handles);
end
line1 = 'Specify sites for PAS (frame 1)';
line2 = 'in crystal-fixed frame';
if nframes == 1
    set(findobj('tag','frame_text'),'String',strvcat(line1,line2));
else
    switch current
        case nframes
            linea = sprintf('Specify sites for frame %d',current);
            set(findobj('tag','frame_text'),'String',strvcat(linea,line2));
        case 1
            lineb = sprintf('in frame %d',2);
            set(findobj('tag','frame_text'),'String',strvcat(line1,lineb));
        otherwise
            linea = sprintf('Specify sites for frame %d',current);
            lineb = sprintf('in frame %d',current+1);
            set(findobj('tag','frame_text'),'String',strvcat(linea,lineb));
    end
end
if current == 1
    set(findobj('tag','prev_frame_button'),'Enable','off');
else
    set(findobj('tag','prev_frame_button'),'Enable','on');
end
if current == nframes
    set(findobj('tag','next_frame_button'),'Enable','off');
else
    set(findobj('tag','next_frame_button'),'Enable','on');
end
frames = handles.frames;
set(findobj('tag','edit_alpha_cq'),'String',sprintf('%7.2f',frames{current}.cq_phi'));
set(findobj('tag','edit_beta_cq'),'String',sprintf('%7.2f',frames{current}.cq_theta'));
set(findobj('tag','edit_gamma_cq'),'String',sprintf('%7.2f',frames{current}.cq_psi'));
set(findobj('tag','edit_alpha_csa'),'String',sprintf('%7.2f',frames{current}.csa_phi'));
set(findobj('tag','edit_beta_csa'),'String',sprintf('%7.2f',frames{current}.csa_theta'));
set(findobj('tag','edit_gamma_csa'),'String',sprintf('%7.2f',frames{current}.csa_psi'));
set(findobj('tag','edit_jump_rate'),'String',sprintf('%10.3e',frames{current}.rate'));
switch frames{current}.model
    case 'all'
        set(findobj('tag','popup_model'),'Value',1);
    case 'cha'
        set(findobj('tag','popup_model'),'Value',2);
    case 'cir'
        set(findobj('tag','popup_model'),'Value',3);
    case 'oth'
        set(findobj('tag','popup_model'),'Value',4);
end
ns = frames{1}.sites;
for n = 2:param.nframes
    ns = ns*frames{n}.sites;
end
param.sites_total = ns;
if length(param.cq) ~= ns
    param.cq= param.cq(1)*ones(1,ns);
end
if length(param.diso) ~= ns
    param.diso= param.diso(1)*ones(1,ns);
end
if length(param.dsigcsa) ~= ns
    param.dsigcsa= param.dsigcsa(1)*ones(1,ns);
end
if length(param.eta) ~= ns
    param.eta = param.eta(1)*ones(1,ns);
end
if length(param.etacsa) ~= ns
    param.etacsa = param.etacsa(1)*ones(1,ns);
end
if length(param.wts) ~= ns
    param.wts = ones(1,ns)/ns;
end
handles.frames = frames;
handles.param = param;
guidata(handles.output, handles);

function result = checkcurrentsites(handles,size)
% This routine, and the two that follow, automatically
% keep the number of entries consistent from row to row
% in the windows where user selects site-dependent Euler angles
current = handles.param.currentframe;
frames = handles.frames;
param = handles.param;
if frames{current}.sites == size
    result = handles;
    return
end
frames{current}.sites = size;
frames{current}.cq_phi = sizedarray(frames{current}.cq_phi,size);
frames{current}.cq_theta = sizedarray(frames{current}.cq_theta,size);
frames{current}.cq_psi = sizedarray(frames{current}.cq_psi,size);
frames{current}.csa_phi = sizedarray(frames{current}.csa_phi,size);
frames{current}.csa_theta = sizedarray(frames{current}.csa_theta,size);
frames{current}.csa_psi = sizedarray(frames{current}.csa_psi,size);
param.frames = frames;
handles.frames = frames;
handles.param = param;
guidata(handles.output, handles);
updatecurrentframe(handles);
result = handles;
turnoffoneframe(handles);
function result = sizedarray(array,size)
result = array;
if length(array) < size
    result(1,size) = 0;
elseif length(array) > size
    result = result(1:size);
end
function turnoffoneframe(handles)
param = handles.param;
if get(findobj('tag','calculate_button'),'value') == 1
    set(findobj('tag','calculate_button'),'value',0);
    showdata(handles,'site');
end

% --- Executes during object creation, after setting all properties.
function edit_frames_CreateFcn(hObject, eventdata, handles)
%edit controls usually have a white background on Windows.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end

function edit_pw90_Callback(hObject, eventdata, handles)
% The 90 degree pulse width is not presently used by EXPRESS
if get(findobj('tag','help_button'),'value') == 1
    printhelp('pw90')
    return
end
param = handles.param ;
param.t90 = eval(get(hObject,'String'));
handles.param = param;
guidata(hObject, handles);
showcurrentdata(handles);

% --- Executes during object creation, after setting all properties.
function edit_pw90_CreateFcn(hObject, eventdata, handles)
% Hint: edit controls usually have a white background on Windows.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end

function edit_pulse_Callback(hObject, eventdata, handles)
% sets P1-tau-P2 pulse spacing for echo experiments
if get(findobj('tag','help_button'),'value') == 1
    printhelp('spacing')
    return
end
param = handles.param ;
param.pulse = eval(get(hObject,'String'));
nn = get(findobj('tag','exp_popup'),'val');
if nn == 2 || nn==8 || nn == 11 % for all spikelet echo simulations
    dw = eval(get(findobj('tag','edit_time'),'string'));
    npts = eval(get(findobj('tag','edit_points_fid'),'string')); 
    param.nppe = 2*param.pulse/dw;
    param.nechoes = floor(dw*npts/param.pulse/2);
end
handles.param = param;
guidata(hObject, handles);
showcurrentdata(handles);

% --- Executes during object creation, after setting all properties.
function edit_pulse_CreateFcn(hObject, eventdata, handles)
% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end

function edit_points_fid_Callback(hObject, eventdata, handles)
% sets number of (complex) time domain points simulated
if get(findobj('tag','help_button'),'value') == 1
    printhelp('npts')
    return
end
param = handles.param ;
param.nfid = str2num(get(hObject,'String'));
nn = get(findobj('tag','exp_popup'),'val');
if nn == 2 || nn==8 || nn == 11 % for all spikelet echo simulations
    twotau = 2*eval(get(findobj('tag','edit_pulse'),'string'));
    dw = eval(get(findobj('tag','edit_time'),'string'));
    param.nechoes = floor(dw*param.nfid/twotau);
    param.nppe = twotau/dw;
    
    
end
param.zfpts = param.nfid; % reset to no zero fill
set(findobj('tag','ZF_menu'),'value',1); % and also set the ZF menu selection to zero
handles.param = param;
guidata(hObject, handles);
showcurrentdata(handles);

% --- Executes during object creation, after setting all properties.
function edit_points_fid_CreateFcn(hObject, eventdata, handles)
% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end

function edit_larmor_Callback(hObject, eventdata, handles)
% sets Larmor frequency; units are MHz
if get(findobj('tag','help_button'),'value') == 1
    printhelp('larmor')
    return
end
param = handles.param ;
param.w0 = eval(get(hObject,'String'));
handles.param = param;
guidata(hObject, handles);
showcurrentdata(handles);

% --- Executes during object creation, after setting all properties.
function edit_larmor_CreateFcn(hObject, eventdata, handles)
% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end

function edit_width_Callback(hObject, eventdata, handles)
% sets spectral window,SW, in kHz. This is also the Nyquist frequency, 1/(dwell time)
% spectrum is displayed from (approximately) -SW/2 to + SW/2
if get(findobj('tag','help_button'),'value') == 1
    printhelp('swh')
    return
end
param = handles.param ;
param.swh = eval(get(hObject,'String'));
param.time = 1000 / param.swh;
set(findobj('tag','edit_time'),'String',num2str(param.time));
nn = get(findobj('tag','exp_popup'),'val');
if nn == 2 || nn==8 || nn == 11 % for all spikelet echo simulations
    tau = eval(get(findobj('tag','edit_pulse'),'string'));
    param.nppe = 2*tau/param.time;
    param.nechoes = floor(param.nfid/param.nppe);
    set(findobj('tag','edit_points_fid'),'string',num2str(param.nfid));
end
handles.param = param;
guidata(hObject, handles);
showcurrentdata(handles);

% --- Executes during object creation, after setting all properties.
function edit_width_CreateFcn(hObject, eventdata, handles)
% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end

function edit_n_Callback(hObject, eventdata, handles)
% specifies a number of tiles for powder average
% the actual number used for the beta angle is algorithm dependent,
% but not less than the vales specified here.
if get(findobj('tag','help_button'),'value') == 1
    printhelp('npowder')
    return
end
param = handles.param ;
param.n = str2num(get(hObject,'String'));
tmp = gettiles(param.n,param.tiletype); % get the ACTUAL number used by the current tiling algorithm
param.nact = size(tmp,1);
clear tmp;
handles.param = param;
guidata(hObject, handles);
showcurrentdata(handles);

% --- Executes during object creation, after setting all properties.
function edit_n_CreateFcn(hObject, eventdata, handles)
% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end

function edit_spin_rate_Callback(hObject, eventdata, handles)
% Sets spin rate in kHz. Setting this to zero turns off edit boxes for the 
% related parameters MAS offset and # of sidebands. 
if get(findobj('tag','help_button'),'value') == 1
    printhelp('spinrate')
    return
end
param = handles.param ;
param.sr = eval(get(hObject,'String'));
handles.param = param;
guidata(hObject, handles);
showcurrentdata(handles);
if param.sr > 0
    set(findobj('tag','spin_panel'),'visible','on');
else
    set(findobj('tag','spin_panel'),'visible','off');
end

% --- Executes during object creation, after setting all properties.
function edit_spin_rate_CreateFcn(hObject, eventdata, handles)
% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end

% --- Executes on selection change in main_text.
function main_text_Callback(hObject, eventdata, handles)
% Parameters displayed in the main text window are ignored,
% EXCEPT for those set via ANALYZE for T1Z, T1Q and T1Zspikelet simulations
if get(findobj('tag','help_button'),'value') == 1
    printhelp('maintext')
    return
end
param = handles.param;
X = param.exp_region;
ptslist = param.ptslist;
[npts,ntau] = size(param.buf);
if isequal(param.experiment,'2H T1Z')||isequal(param.experiment,'T1Zspikelet') % displays T1Z semi-log plot for user selected point
    if ~isempty(param.M_list)
        index = get(hObject,'Value');
        if index < 7
            set(hObject,'val',7)
            return
        end
        Z = param.M_list(index-6,:);
        Z = (Z(ntau)-Z)/(2.*Z(ntau));
        fax = ((0:(npts-1))'-npts/2)*param.swh/npts;
        expfax = fax(X(1):X(2));
        yspec = real(param.buf(X(1):X(2),ntau));
        axes(handles.window_a)
        plot(expfax,yspec,fax(ptslist(index-6)),real(param.buf(ptslist(index-6),ntau)),'ro','MarkerFaceColor','g');
        axes(handles.window_b)
        semilogy(param.taulist(:),Z(:),'ro','MarkerFaceColor','g');
        set(handles.window_b,'userdata',[param.taulist(:) Z(:)]);
    end
end
if isequal(param.experiment,'2H T1Q') % displays T1Q semi-log plot for user selected point
    if ~isempty(param.M_list)
        index = get(hObject,'Value');
        if index < 7
            set(hObject,'val',7)
            return
        end
        Z = param.M_list(index-6,:);
        Z = Z./Z(1);
        fax = ((0:(npts-1))'-npts/2)*param.swh/npts;
        expfax = fax(X(1):X(2));
        yspec = real(param.buf(X(1):X(2),ntau));
        axes(handles.window_a)
        plot(expfax,yspec,fax(ptslist(index-6)),real(param.buf(ptslist(index-6),ntau)),'ro','MarkerFaceColor','g');
        axes(handles.window_b)
        semilogy(param.taulist(:),Z(:),'ro','MarkerFaceColor','g');
        set(handles.window_b,'userdata',[param.taulist(:) Z(:)]);
    end
end

% --- Executes during object creation, after setting all properties.
function main_text_CreateFcn(hObject, eventdata, handles)
% Hint: listbox controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end

% --- Executes on button press in calculate_button.
function calculate_button_Callback(hObject, eventdata, handles)
% This poorly-named routine turns on a panal that allows 
% user to specify tensor PAS values and relative weights for individual
% one-frame equivalent sites. It also calculates and displays equivalent one-frame site angles
if get(findobj('tag','help_button'),'value') == 1
    printhelp('oneframe')
    return
end
param = handles.param;
efgtags = param.efgtags ; csatags = param.csatags;
expt_num = get(findobj('tag','expt_select'),'Value');
if expt_num < 6
    set(csatags,'visible','off');
    if expt_num == 1
        set([csatags findobj('tag','uipanel9')],'visible','on'); % panel for CSA site angles
    end
else
    set(csatags,'visible','on');
end
if expt_num == 10
    set(efgtags,'visible','off');
else
    set(efgtags,'visible','on');
end
frames = handles.frames;
nframes = param.nframes;
if get(findobj('tag','calculate_button'),'value') == 1
    set(findobj('tag','uipanel7'),'visible','off');
    set(findobj('tag','uipanel9'),'visible','off');
    set(findobj('tag','edit_CQ'),'string',sprintf('%10.3f',param.cq(1)));
    set(findobj('tag','edit_eta_Q'),'string',sprintf('%10.3f',param.eta(1)));
    set(findobj('tag','edit_delta_iso'),'string',sprintf('%10.3f',param.diso(1)));
    set(findobj('tag','edit_span'),'string',sprintf('%10.3f',param.dsigcsa(1)));
    set(findobj('tag','edit_skew'),'string',sprintf('%10.3f',param.etacsa(1)));
    set(findobj('tag','one_frame_panel'),'visible','on');
    for n=1:nframes
        Nsite(n) = frames{n}.sites;
        Nmodel(n,:) = frames{n}.model;
        Nrate(n) = frames{n}.rate;
        Ntemplate{n} = frames{n}.template;
    end
    sitenum=1;
    Fangle = zeros(sum(Nsite),3);
    for n=1:nframes
        param.frames{n}.cq_phi = frames{n}.cq_phi;
        param.frames{n}.cq_theta = frames{n}.cq_theta;
        param.frames{n}.cq_psi = frames{n}.cq_psi;
        param.frames{n}.csa_phi = frames{n}.csa_phi;
        param.frames{n}.csa_theta = frames{n}.csa_theta;
        param.frames{n}.csa_psi = frames{n}.csa_psi;
        for siteadd=1:Nsite(n)
            angle1 = [pi*frames{n}.csa_phi(siteadd)/180 pi*frames{n}.csa_theta(siteadd)/180 pi*frames{n}.csa_psi(siteadd)/180];
            angle2 = [pi*frames{n}.cq_phi(siteadd)/180 pi*frames{n}.cq_theta(siteadd)/180 pi*frames{n}.cq_psi(siteadd)/180];
            Fangle(siteadd+sitenum-1,:) = angle2;
            Fanglecsa(siteadd+sitenum-1,:) = angle1;
        end
        sitenum = sitenum + Nsite(n);
    end
    [onek, kmatrxcell]=oneframekmatrx(Nsite,Nmodel, Nrate, Ntemplate);
    [oneangle Tsite]=oneframeangle(Nsite,Fangle);
    [oneanglecsa Tsite]=oneframeangle(Nsite,Fanglecsa);
    % for unequal weights (site populations), assume that
    % kmatrix(j,k)=kmatrix(k,j) is the average, 0.5*[K(i<--j) + K(j<--i)]
    % Since kmatrix must satisfy detailed balance, apply a DIAGONAL
    % symmetrizing transformation. This implies a new, SYMMETRIC K-matrix:
    siten = size(onek,1);

    for jj = 1:siten
        tmp = 0;
        for kk = 1:siten
            if kk ~= jj
                tmp = tmp -2*param.wts(kk)*onek(jj,kk)/(param.wts(jj)+param.wts(kk));
            end
        end
        onek(jj,jj) = tmp;
    end
    for jj = 1:siten
        for kk = 1:siten
            onek(jj,kk) = 2*onek(jj,kk)*sqrt(param.wts(jj)*param.wts(kk))/(param.wts(jj)+param.wts(kk));
        end
    end
    param.onek = onek;
    param.oneangle = oneangle;
    param.oneanglecsa = oneanglecsa;
    param.kmatrxcell = kmatrxcell;
    param.zfpts = param.nfid;
    ll = size(param.oneangle,1);
    param.total_sites = ll;
    if length(param.cq) == 1
        param.cq = param.cq*ones(1,ll);
    end
    if length(param.eta) == 1
        param.eta = param.eta*ones(1,ll);
    end
    if length(param.diso) == 1
        param.diso = param.diso*ones(1,ll);
    end
    if length(param.dsigcsa) == 1
        param.dsigcsa = param.dsigcsa*ones(1,ll);
    end
    if length(param.etacsa) == 1
        param.etacsa = param.etacsa*ones(1,ll);
    end
    if length(param.wts) == 1
        param.wts = 1/param.wts*ones(1,ll)/ll;
    end
    handles.param = param;
    guidata(hObject, handles);
else % button is "out"
    expt_num = get(findobj('tag','expt_select'),'Value');
    if param.spin == 0.5 % turn off EFG panel for I = 1/2
        set(findobj('tag','uipanel7'),'visible','off');
    else
        set(findobj('tag','uipanel7'),'visible','on');
    end
    if expt_num < 6 % first five entries in experiment list have no CSA
        set(findobj('tag','uipanel9'),'visible','off');
        if expt_num == 1
                set([csatags findobj('tag','uipanel9')],'visible','on'); % allow CSA  site angles for QE 
            end
    else
        set(findobj('tag','uipanel9'),'visible','on');
    end
    set(findobj('tag','one_frame_panel'),'visible','off');
end
handles.param = param;
guidata(hObject,handles);
showdata(handles,'site');

% --- Executes on selection change in exp_popup.
function exp_popup_Callback(hObject, eventdata, handles)
% Menu options for which experiment to simulate
if get(findobj('tag','help_button'),'value') == 1
    printhelp('expt')
    return
end
param = handles.param;
mastags = param.mastags ; efgtags = param.efgtags ; csatags = param.csatags;
delaytags = param.delaytags;
val = get(hObject,'Value');
set(findobj('Tag','expt_select'),'Value',val);
set(findobj('Tag','tilestyle'),'Value',1)
param.nact = set_actual_n(handles);
switch val
    case 1
        param.experiment = '2H QE';
        set(findobj('Tag','spin'),'Value',2) % reset spin to I = 1
        set([mastags delaytags],'visible','off')
        set(efgtags,'visible','on');
    case 2
        param.experiment = '2H Spikelets';
        set(findobj('Tag','spin'),'Value',2)
        set([mastags delaytags csatags],'visible','off')
        set(efgtags,'visible','on');
    case 3
        param.experiment = '2H T1Z';
        set([mastags csatags],'visible','off')
        set(findobj('Tag','spin'),'Value',2)
        set([efgtags delaytags],'visible','on');
    case 4
        param.experiment = '2H T1Q';
        set([mastags csatags],'visible','off')
        set(findobj('Tag','spin'),'Value',2)
        set([efgtags delaytags],'visible','on');
    case 5
        param.experiment = '2H OMAS';
        set(findobj('Tag','spin'),'Value',2)
        set([mastags efgtags],'visible','on')
        set([delaytags csatags],'visible','off');
    case 6
        param.experiment = '2H OMAS w/CSA';
        set(findobj('Tag','spin'),'Value',2)
        set([mastags csatags efgtags],'visible','on')
        set(delaytags,'visible','off');
    case 7
        param.experiment = 'I > 1 Static C.T.';
        set([mastags delaytags] ,'visible','off')
        set([csatags efgtags],'visible','on');
    case 8
        param.experiment = 'I > 1 Spikelets';
        set([mastags delaytags],'visible','off')
        set([csatags efgtags] ,'visible','on');
    case 9
        param.experiment = 'I > 1 OMAS';
        set([mastags csatags efgtags],'visible','on')
        set(delaytags,'visible','off');
        set(findobj('Tag','tilestyle'),'Value',3);
        param.nact = set_actual_n(handles);
    case 10
        param.experiment = 'I = 1/2 1PULS';
        set(findobj('Tag','spin'),'Value',1) % reset spin to I = 1/2
        set([mastags csatags],'visible','on')
        set([delaytags efgtags],'visible','off');
    case 11
        param.experiment = 'T1Zspikelet'; %'user routine 1';
        %Note replacement of user routine 1 by T1Zspikelet
        % insert set-up code as necessary for user-defined experiments
        % this should match code in the expt_select callback routine
        set(findobj('Tag','spin'),'Value',2)
        set([mastags delaytags csatags],'visible','off')
        set(efgtags,'visible','on');
        set([efgtags delaytags],'visible','on');
    case 12
        param.experiment = 'user routine 2';
        % insert set-up code if necessary for user-defined experiments
        % this should match code in the expt_select callback routine
    case 13
        param.experiment = 'user routine 3';
        % insert set-up code if necessary for user-defined experiments
        % this should match code in the expt_select callback routine
end
handles.param = param;
guidata(hObject, handles);
showcurrentdata(handles);

% --- Executes during object creation, after setting all properties.
function exp_popup_CreateFcn(hObject, eventdata, handles)
% Hint: popupmenu controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end

% --- Executes on button press in next_frame_button.
function next_frame_button_Callback(hObject, eventdata, handles)
% points user to parameters for next frame
if get(findobj('tag','help_button'),'value') == 1
    printhelp('nextframe')
    return
end
param = handles.param ;
param.currentframe = param.currentframe + 1;
handles.param = param;
guidata(hObject, handles);
updatecurrentframe(handles);

function result = shouldloadfile(hObject, handles, string)
% works with  function checkcurrentsites to maintain a consistent 
% number of entries in each row of the site selection boxes for Euler angles
result = exist(string,'file');
if result == 2
    param = handles.param;
    frames = handles.frames;
    load(string, '-mat');
    if exist('cqVal','var') == 1
        frames{param.currentframe}.cq = cqVal;
    end
    if exist('weightsVal','var') == 1
        frames{param.currentframe}.wts = weightsVal;
    end
    if exist('kMatrixVal','var') == 1
        frames{param.currentframe}.template = kMatrixVal;
        frames{param.currentframe}.model = 'oth';
    end
    if exist('cqEtaVal','var') == 1
        frames{param.currentframe}.cq_eta = cqEtaVal;
    end
    if exist('cqAlphaVal','var') == 1
        frames{param.currentframe}.cq_phi = cqAlphaVal;
    end
    if exist('cqBetaVal','var') == 1
        frames{param.currentframe}.cq_theta = cqBetaVal;
    end
    if exist('cqGammaVal','var') == 1
        frames{param.currentframe}.cq_psi = cqGammaVal;
    end
    if exist('csaSigmaVal','var') == 1
        frames{param.currentframe}.csa_sigma = csaSigmaVal;
    end
    if exist('csaIsoVal','var') == 1
        frames{param.currentframe}.csa_iso = csaIsoVal;
    end
    if exist('csaEtaVal','var') == 1
        frames{param.currentframe}.csa_eta = csaEtaVal;
    end
    if exist('csaAlphaVal','var') == 1
        frames{param.currentframe}.csa_phi = csaAlphaVal;
    end
    if exist('csaBetaVal','var') == 1
        frames{param.currentframe}.csa_theta = csaBetaVal;
    end
    if exist('csaGammaVal','var') == 1
        frames{param.currentframe}.csa_psi = csaGammaVal;
    end
    if exist('jumpRateVal','var') == 1
        frames{param.currentframe}.rate = jumpRateVal;
    end
    handles.frames = frames;
    guidata(hObject, handles);
    if exist('nSitesVal','var') == 1
        handles = checkcurrentsites(handles,nSitesVal);
    end
    updatecurrentframe(handles);
    showcurrentdata(handles);
else
    result = 0;
end

function edit_alpha_cq_Callback(hObject, eventdata, handles)
% update EFG site angle alpha
if get(findobj('tag','help_button'),'value') == 1
    printhelp('cqangles')
    return
end
if shouldloadfile(hObject, handles, get(hObject,'String')) ~= 0
    return;
end
param = handles.param;
frames = handles.frames;
frames{param.currentframe}.cq_phi = str2num(get(hObject,'String'));
handles.frames = frames;
param.frames =frames;
handles.param = param;
guidata(hObject, handles);
handles = checkcurrentsites(handles,length(frames{param.currentframe}.cq_phi));
showcurrentdata(handles);
turnoffoneframe(handles);

% --- Executes during object creation, after setting all properties.
function edit_alpha_cq_CreateFcn(hObject, eventdata, handles)
% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end

function edit_beta_cq_Callback(hObject, eventdata, handles)
% update EFG site angle beta
if get(findobj('tag','help_button'),'value') == 1
    printhelp('cqangles')
    return
end
if shouldloadfile(hObject, handles, get(hObject,'String')) ~= 0
    return;
end
param = handles.param;
frames = handles.frames;
frames{param.currentframe}.cq_theta = str2num(get(hObject,'String'));
handles.frames = frames;
param.frames =frames;
handles.param = param;
guidata(hObject, handles);
handles = checkcurrentsites(handles,length(frames{param.currentframe}.cq_theta));
showcurrentdata(handles);
turnoffoneframe(handles);

% --- Executes during object creation, after setting all properties.
function edit_beta_cq_CreateFcn(hObject, eventdata, handles)
% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end

function edit_gamma_cq_Callback(hObject, eventdata, handles)
% update EFG site angle gamma
if get(findobj('tag','help_button'),'value') == 1
    printhelp('cqangles')
    return
end
if shouldloadfile(hObject, handles, get(hObject,'String')) ~= 0
    return;
end
param = handles.param;
frames = handles.frames;
frames{param.currentframe}.cq_psi = str2num(get(hObject,'String'));
handles.frames = frames;
param.frames =frames;
handles.param = param;
guidata(hObject, handles);
handles = checkcurrentsites(handles,length(frames{param.currentframe}.cq_psi));
showcurrentdata(handles);
turnoffoneframe(handles);

% --- Executes during object creation, after setting all properties.
function edit_gamma_cq_CreateFcn(hObject, eventdata, handles)
% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end

function edit_alpha_csa_Callback(hObject, eventdata, handles)
% update CSA site angle alpha
if get(findobj('tag','help_button'),'value') == 1
    printhelp('csangles')
    return
end
if shouldloadfile(hObject, handles, get(hObject,'String')) ~= 0
    return;
end
param = handles.param;
frames = handles.frames;
frames{param.currentframe}.csa_phi = str2num(get(hObject,'String'));
handles.frames = frames;
param.frames =frames;
handles.param = param;
guidata(hObject, handles);
handles = checkcurrentsites(handles,length(frames{param.currentframe}.csa_phi));
showcurrentdata(handles);
turnoffoneframe(handles);

% --- Executes during object creation, after setting all properties.
function edit_alpha_csa_CreateFcn(hObject, eventdata, handles)
% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end

function edit_beta_csa_Callback(hObject, eventdata, handles)
% update CSA site angle beta
if get(findobj('tag','help_button'),'value') == 1
    printhelp('csangles')
    return
end
if shouldloadfile(hObject, handles, get(hObject,'String')) ~= 0
    return;
end
param = handles.param;
frames = handles.frames;
frames{param.currentframe}.csa_theta = str2num(get(hObject,'String'));
handles.frames = frames;
guidata(hObject, handles);
handles = checkcurrentsites(handles,length(frames{param.currentframe}.csa_theta));
showcurrentdata(handles);
turnoffoneframe(handles);

% --- Executes during object creation, after setting all properties.
function edit_beta_csa_CreateFcn(hObject, eventdata, handles)
% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end

function edit_gamma_csa_Callback(hObject, eventdata, handles)
% update CSA site angle gamma
if get(findobj('tag','help_button'),'value') == 1
    printhelp('csangls')
    return
end
if shouldloadfile(hObject, handles, get(hObject,'String')) ~= 0
    return;
end
param = handles.param;
frames = handles.frames;
frames{param.currentframe}.csa_psi = str2num(get(hObject,'String'));
handles.frames = frames;
param.frames =frames;
handles.param = param;
guidata(hObject, handles);
handles = checkcurrentsites(handles,length(frames{param.currentframe}.csa_psi));
showcurrentdata(handles);
turnoffoneframe(handles);

% --- Executes during object creation, after setting all properties.
function edit_gamma_csa_CreateFcn(hObject, eventdata, handles)
% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end

% --- Executes on button press in prev_frame_button.
function prev_frame_button_Callback(hObject, eventdata, handles)
% point user to parameters for previous frame
if get(findobj('tag','help_button'),'value') == 1
    printhelp('prevframe')
    return
end
param = handles.param ;
param.currentframe = param.currentframe - 1;
handles.param = param;
guidata(hObject, handles);
updatecurrentframe(handles);

function edit_jump_rate_Callback(hObject, eventdata, handles)
% enter jump rate for current frame
if get(findobj('tag','help_button'),'value') == 1
    printhelp('rate1')
    return
end
if shouldloadfile(hObject, handles, get(hObject,'String')) ~= 0
    return;
end
param = handles.param;
frames = handles.frames;
frames{param.currentframe}.rate = eval(get(hObject,'String'));
handles.frames = frames;
param.frames =frames;
handles.param = param;
guidata(hObject, handles);
handles = updateCurrentTemplate(handles);
showcurrentdata(handles);
turnoffoneframe(handles);

function result = updateCurrentTemplate(handles)
% stores a user-selected, predefined template for rate matrix of  the current frame
param = handles.param;
frames = handles.frames;
if frames{param.currentframe}.model == 'oth'
    result = handles;
    return
end
frames{param.currentframe}.template = creatematrix(frames{param.currentframe}.sites,frames{param.currentframe}.model,frames{param.currentframe}.rate,handles);
handles.frames = frames;
param.frames =frames;
handles.param = param;
guidata(findobj('tag','edit_jump_rate'), handles);
result = handles;

% --- Executes during object creation, after setting all properties.
function edit_jump_rate_CreateFcn(hObject, eventdata, handles)
% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end

% --- Executes on selection change in popup_model.
function popup_model_Callback(hObject, eventdata, handles)
% hObject    handle to popup_model (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: contents = get(hObject,'String') returns popup_model contents as cell array
%        contents{get(hObject,'Value')} returns selected item from popup_model
if get(findobj('tag','help_button'),'value') == 1
    printhelp('model')
    return
end
param = handles.param;
frames = handles.frames;
val = get(hObject,'Value');
switch val
    case 1
        frames{param.currentframe}.model = 'all';
    case 2
        frames{param.currentframe}.model = 'cha';
    case 3
        frames{param.currentframe}.model = 'cir';
    case 4
        frames{param.currentframe}.model = 'oth';
end
frames{param.currentframe}.template = creatematrix(frames{param.currentframe}.sites,frames{param.currentframe}.model,frames{param.currentframe}.rate,handles);
handles.frames = frames;
param.frames =frames;
handles.param = param;
guidata(hObject, handles);
showcurrentdata(handles);
turnoffoneframe(handles);

% --- Executes during object creation, after setting all properties.
function popup_model_CreateFcn(hObject, eventdata, handles)
% Hint: popupmenu controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end

% --- Executes on button press in start_run_button.
function start_run_button_Callback(hObject, eventdata, handles)
% This is where EXPRESS selects a simulation routine, in accord with the
% menu item selected by the user. First, it makes sure to use the most current
% one-frame parameters
if get(findobj('tag','help_button'),'value') == 1
    printhelp('start')
    return
end
param = handles.param;
frames = handles.frames;
nframes = param.nframes;
% switch param.runflags(1) % check for user-modified one-frame K matrix
%     case 0 % there is no user-modified, custom one-frame K-matrix
%         onek = param.onek; 
%     case 1 % point param.onek to the user-modified K-matrix
%         onek = param.custom_onek; % preserve default one-frame K-matrix
%         param.onek = param.custom_onek; % pass this one to the simulators
%     otherwise
%         onek = param.onek;
% end
% oneangle = param.oneangle;
% oneanglecsa = param.oneanglecsa;
if isempty(param.onek)
    errordlg('must define one-frame parameters')
    return
end
tt = clock; % start the clock; "TIC"
switch param.experiment
    case '2H OMAS'
        if ~isequal(get(findobj('tag','spin'),'value'),2)
            errordlg('Must set spin I = 1')
            return
        end
        showdata(handles,'site');
        param.fid=DSIM(param);
        param.nfid = size(param.fid,1); % update number of points computed
        param.zfpts = param.nfid;
        reset_view;
    case '2H QE'
        if ~isequal(get(findobj('tag','spin'),'value'),2)
            errordlg('Must set spin I = 1')
            return
        end
        showdata(handles,'site');
        param.fid=staticsimCQCSA(param);
        param.nfid = size(param.fid,1); % update number of points computed
        param.zfpts = param.nfid;
        reset_view;
    case '2H Spikelets'
        if ~isequal(get(findobj('tag','spin'),'value'),2)
            errordlg('Must set spin I = 1')
            return
        end
        showdata(handles,'site');
        param.fid=spikelet(param);
        param.nfid = size(param.fid,1); % update number of points computed
        param.zfpts = param.nfid;
        reset_view;
    case '2H T1Z'
        if ~isequal(get(findobj('tag','spin'),'value'),2)
            errordlg('Must set spin I = 1')
            return
        end
        showdata(handles,'site');
        param.fid=T1Zsim(param);
        param.nfid = size(param.fid,1); % update number of points computed
        param.zfpts = param.nfid;
        reset_view;
        set(findobj('tag','delays_button'),'enable','on'); % enable ANALYZE button
    case '2H T1Q'
        if ~isequal(get(findobj('tag','spin'),'value'),2)
            errordlg('Must set spin I = 1')
            return
        end
        showdata(handles,'site');
        param.fid=T1Qsim(param);
        param.nfid = size(param.fid,1); % update number of points computed
        param.zfpts = param.nfid;
        reset_view;
        set(findobj('tag','delays_button'),'enable','on'); % enable ANALYZE button        
        
    case '2H OMAS w/CSA'
        if ~isequal(get(findobj('tag','spin'),'value'),2)
            errordlg('Must set spin I = 1')
            return
        end
        showdata(handles,'site');
        param.fid=DSIMCSA(param);
        param.nfid = size(param.fid,1); % update number of points computed
        param.zfpts = param.nfid;
        reset_view;
    case 'I > 1 Spikelets'
        showdata(handles,'site');
        param.fid = CQ2nd(param);
        param.nfid = size(param.fid,1); % update number of points computed
        param.zfpts = param.nfid;
        reset_view;
    case 'I > 1 OMAS'
        showdata(handles,'site');
        param.fid = CQ2CTvas(param);
        param.nfid = size(param.fid,1); % update number of points computed
        param.zfpts = param.nfid;
        reset_view;
       
    case 'I > 1 Static C.T.'
        showdata(handles,'site');
        cqmax = max(abs(param.cq))*2000*pi ; % max CQ in rad/sec
        kbar = norm(param.onek); %norm of rate matrix
        ratecheck = cqmax^2/(kbar*2*pi*param.w0*1.e6); % not used in this release
        param.fid = CQ2CTstatic(param);
        param.nfid = size(param.fid,1); % update number of points computed
        param.zfpts = param.nfid;
        reset_view;
    case 'I = 1/2 1PULS'
        if ~isequal(get(findobj('tag','spin'),'value'),1)
            errordlg('Must set spin I = 1/2')
            return
        end
        showdata(handles,'site');
        if param.sr ~= 0                    % NOTE!! set spin rate to zero for a static line shape
            param.fid = OMASCSA(param);     % otherwise, you get a MAS or OMAS line shape
            param.nfid = size(param.fid,1); % update number of points computed
            param.zfpts = param.nfid;
            reset_view;
        else
            param.fid = staticsimcsa(param);
            param.nfid = size(param.fid,1); % update number of points computed
            param.zfpts = param.nfid;
            reset_view;
        end
    case 'T1Zspikelet' 
            param.fid = T1Zspikelet(param);
            param.nfid = size(param.fid,1); % update number of points computed
            param.zfpts = param.nfid;
            reset_view;
            set(findobj('tag','delays_button'),'enable','on'); % enable ANALYZE button
    case 'user routine 2' % uncomment the following when user2 routine is to be available
            %param.fid = user2(param)
            %param.nfid = size(param.fid,1); % update number of points computed
            %param.zfpts = param.nfid;
            %reset_view;
    case 'user routine 3' % uncomment the following when user3 routine is to be available
            %param.fid = user3(param)
            %param.nfid = size(param.fid,1); % update number of points computed
            %param.zfpts = param.nfid;
            %reset_view;
      
    otherwise % could add user code to the switch options 
        errordlg('Experiment not found')
end
param.runTime = etime(clock,tt); % stop the cpu timer; "TOC"
%param.onek = onek; % restore default K-matrix
set(gcbo,'value',0);
handles.param = param;
guidata(hObject, handles);
% clean up
set(findobj('tag','runTime_text'),'string',sprintf('%f %s',param.runTime,'s'));
set(findobj('tag','runTime_label'),'visible','on');
set(findobj('tag','runTime_text'),'visible','on');
set(findobj('tag','view_output_button'),'enable','on');

function edit_time_Callback(hObject, eventdata, handles)
% set dwell time here; units are microseconds
if get(findobj('tag','help_button'),'value') == 1
    printhelp('dwell')
    return
end
param = handles.param ;
param.time = eval(get(hObject,'String'));
param.swh = 1000 / param.time;
set(findobj('tag','edit_width'),'String',num2str(param.swh));
nn = get(findobj('tag','exp_popup'),'val');
if nn == 2 || nn==8 % for a spikelet echo simulation
    tau = eval(get(findobj('tag','edit_pulse'),'string'));
    param.nppe = 2*tau/param.time;
    param.nechoes = floor(param.nfid/param.nppe);
    set(findobj('tag','edit_points_fid'),'string',num2str(param.nfid));
end
handles.param = param;
guidata(hObject, handles);
showcurrentdata(handles);

% --- Executes during object creation, after setting all properties.
function edit_time_CreateFcn(hObject, eventdata, handles)
% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end

% --- Executes on button press in export_button.
function export_button_Callback(hObject, eventdata, handles)
% saves ascii data files of time domain and (processed) frequency domain
% data for use by external programs
if get(findobj('tag','help_button'),'value') == 1
    printhelp('export')
    return
end
[filename,pathname] = uiputfile('','export to ');
if isequal(filename,0) || isequal(pathname,0)
    warndlg('data not saved');
else
    specname = strcat(filename,'_spec');
    fidname = strcat(filename,'_fid');
    specpath = fullfile(pathname, specname);
    fidpath = fullfile(pathname, fidname); 
    spec = handles.param.buf;
    fid = handles.param.fid;
    dlmwrite(specpath, spec, '\t',0,0);
    dlmwrite(fidpath,fid, '\t',0,0); 
end

% --- Executes on button press in fft_button.
function fft_button_Callback(hObject, eventdata, handles)
% toggles FFT flag true/false
if get(findobj('tag','help_button'),'value') == 1
    printhelp('fft')
    return
end
showOutput(hObject,handles);

function edit_left_shift_Callback(hObject, eventdata, handles)
% sets number of left shifts; fractional values allowed!
if get(findobj('tag','help_button'),'value') == 1
    printhelp('ls')
    return
end
param = handles.param;
param.left_shift = str2num(get(hObject,'String'));
handles.param = param;
guidata(hObject, handles);
showOutput(hObject,handles);

% --- Executes during object creation, after setting all properties.
function edit_left_shift_CreateFcn(hObject, eventdata, handles)
% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end

function edit_CQ_Callback(hObject, eventdata, handles)
% set EFG PAS CQ value(s) in equivalent one-frame  context
if get(findobj('tag','help_button'),'value') == 1
    printhelp('cqparms')
    return
end
param = handles.param;
param.cq = str2num(get(hObject,'string'));
if length(param.cq) ~= param.total_sites
    param.cq = param.cq(1)*ones(1,param.total_sites);
end
handles.param = param;
guidata(hObject,handles);
showcurrentdata(handles);

% --- Executes during object creation, after setting all properties.
function edit_CQ_CreateFcn(hObject, eventdata, handles)
% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end

function edit_eta_Q_Callback(hObject, eventdata, handles)
% set EFG PAS eta value(s) in equivalent one-frame  context
if get(findobj('tag','help_button'),'value') == 1
    printhelp('cqparms')
    return
end
param = handles.param;
param.eta = str2num(get(hObject,'string'));
if length(param.eta) ~= param.total_sites
    param.eta = param.eta(1)*ones(1,param.total_sites);
end
handles.param = param;
guidata(hObject,handles);
showcurrentdata(handles);

% --- Executes during object creation, after setting all properties.
function edit_eta_Q_CreateFcn(hObject, eventdata, handles)
% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end

function edit_delta_iso_Callback(hObject, eventdata, handles)
% set CSA PAS delta-iso value(s) in equivalent one-frame  context
if get(findobj('tag','help_button'),'value') == 1
    printhelp('csparms')
    return
end
if get(findobj('tag','help_button'),'value') == 1
    printhelp('csparms')
    return
end
param = handles.param;
param.diso = str2num(get(hObject,'string'));
if length(param.diso) ~= param.total_sites
    param.diso = param.diso(1)*ones(1,param.total_sites);
end
handles.param = param;
guidata(hObject,handles);
showcurrentdata(handles);

% --- Executes during object creation, after setting all properties.
function edit_delta_iso_CreateFcn(hObject, eventdata, handles)
% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end

function edit_span_Callback(hObject, eventdata, handles)
% set CSA PAS anisotropy value(s) in equivalent one-frame  context. "span" label is misleading!
% this is delta-sigma as defined by Haeberlen!
if get(findobj('tag','help_button'),'value') == 1
    printhelp('csparms')
    return
end
param = handles.param;
param.dsigcsa = str2num(get(hObject,'string'));
if length(param.dsigcsa) ~= param.total_sites
    param.dsigcsa = param.dsigcsa(1)*ones(1,param.total_sites);
end

handles.param = param;
guidata(hObject,handles);
showcurrentdata(handles);

% --- Executes during object creation, after setting all properties.
function edit_span_CreateFcn(hObject, eventdata, handles)
% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end


function edit_skew_Callback(hObject, eventdata, handles)
% set CSA PAS eta value(s) in equivalent one-frame  context. "skew" label is misleading!
% this is eta as defined by Haeberlen!
if get(findobj('tag','help_button'),'value') == 1
    printhelp('csparms')
    return
end
param = handles.param;
param.etacsa = str2num(get(hObject,'string'));
if length(param.etacsa) ~= param.total_sites
    param.etacsa = param.etacsa(1)*ones(1,param.total_sites);
end

handles.param = param;
guidata(hObject,handles);
showcurrentdata(handles);

% --- Executes during object creation, after setting all properties.
function edit_skew_CreateFcn(hObject, eventdata, handles)
% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end

function edit_rel_wt_Callback(hObject, eventdata, handles)
% set relative weight(s) in equivalent one-frame  context. 
if get(findobj('tag','help_button'),'value') == 1
    printhelp('wts')
    return
end
param = handles.param;
param.wts = str2num(get(hObject,'string'));
if length(param.wts) ~= param.total_sites
    param.wts = param.wts(1)*ones(1,param.total_sites);
else
    param.wts = param.wts/sum(param.wts); % make sure weights add to unity!
end
handles.param = param;
guidata(hObject,handles);
showcurrentdata(handles);

% --- Executes during object creation, after setting all properties.
function edit_rel_wt_CreateFcn(hObject, eventdata, handles)
% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end

% --- Executes on button press in radio_multiedit.
function radio_multiedit_Callback(hObject, eventdata, handles)
% turns window on/off for editing PAS paraameters for individual sites
if get(findobj('tag','help_button'),'value') == 1
    printhelp('multi_edit')
    return
end
param = handles.param;
short = 0.14 ; long = 0.74;
tt = ['edit_CQ' 'edit_eta_Q' 'edit_delta_iso' 'edit_span' 'edit_skew' 'edit_rel_wt'];
if get(hObject,'Value') ==1
    hh = findobj('tag','edit_CQ');
    p = get(hh,'Position');  p(3) = long; set(hh,'Position',p);
    set (hh,'string',sprintf('%9.1f',param.cq));
    hh = findobj('tag','edit_eta_Q');
    p = get(hh,'Position');  p(3) = long; set(hh,'Position',p);
    set (hh,'string',sprintf('%9.3f',param.eta));
    hh = findobj('tag','edit_delta_iso');
    p = get(hh,'Position');  p(3) = long; set(hh,'Position',p);
    set (hh,'string',sprintf('%10.2f',param.diso));
    hh = findobj('tag','edit_span');
    p = get(hh,'Position');  p(3) = long; set(hh,'Position',p);
     set (hh,'string',sprintf('%10.2f',param.dsigcsa));
    hh = findobj('tag','edit_skew');
    p = get(hh,'Position');  p(3) = long; set(hh,'Position',p);
     set (hh,'string',sprintf('%10.2f',param.etacsa));
    hh = findobj('tag','edit_rel_wt');
    p = get(hh,'Position');  p(3) = long; set(hh,'Position',p); 
     set (hh,'string',sprintf('%10.4f',param.wts));
else
    hh = findobj('tag','edit_CQ');
    p = get(hh,'Position');  p(3) = short; set(hh,'Position',p);
    set (hh,'string',sprintf('%9.1f',param.cq(1)));
    hh = findobj('tag','edit_eta_Q');
    p = get(hh,'Position');  p(3) = short; set(hh,'Position',p);
    set (hh,'string',sprintf('%9.1f',param.eta(1)));
    hh = findobj('tag','edit_delta_iso');
    p = get(hh,'Position');  p(3) = short; set(hh,'Position',p);
    set (hh,'string',sprintf('%9.1f',param.diso(1)));
    hh = findobj('tag','edit_span');
    p = get(hh,'Position');  p(3) = short; set(hh,'Position',p);
    set (hh,'string',sprintf('%9.1f',param.dsigcsa(1)));
    hh = findobj('tag','edit_skew');
    p = get(hh,'Position');  p(3) = short; set(hh,'Position',p);
    set (hh,'string',sprintf('%9.1f',param.etacsa(1)));
    hh = findobj('tag','edit_rel_wt');
    p = get(hh,'Position');  p(3) = short; set(hh,'Position',p); 
    set (hh,'string',sprintf('%9.1f',param.wts(1)));
end

function printhelp(string)
% express help function
% displays help messages for most user interface controls
% the file testhelp.txt must be in the MATLAB path
fid = fopen('testhelp.txt','rt');
mes = '';
test = strcat('$$',string);
while ~feof(fid)
    tline = fgets(fid);
    ll = min(length(tline),length(test));
    if strncmp(test,tline,ll)
       while ~feof(fid)
           tline = fgets(fid);
           if ~strcmp(tline(1:2),'%%')
               mes = strvcat(mes,tline);
           else
               mes = strvcat(mes,' ');
               mes = strvcat(mes,'Release the HELP button to re-activate the controls');
               set(findobj('tag','main_text'),'string',mes,'val',1);
               fclose(fid);
               return
           end
       end
    end
end
fclose(fid);

% --- Executes on button press in phase_button.
function phase_button_Callback(hObject, eventdata, handles)
% invokes a mini-GUI to perform phase correction of simulated data
% this is reaerly needed, especially if left shifts are set properly
if get(findobj('tag','help_button'),'value') == 1
    printhelp('phase')
    return
end
param = handles.param;
SW = param.swh;
if get(gcbo,'value') == 1
    if isempty(param.spec1)
        errordlg('need to FT before PC')
        return
    end
    nn = length(param.spec1);
    ax = [-nn/2 :(nn/2 - 1)]'*SW/nn ;
    fdata = zeros(length(ax),2);
    fdata(:,1) = ax;
    fdata(:,2) = param.spec1(:);
    ph0han = findobj('tag','edit_ph0');
    ph1han = findobj('tag','edit_ph1');
    
    phase0 = eval(get(ph0han,'string'));
    phase1 = eval(get(ph1han,'string'));
    pivot = nn/2;
    phasein(1) = 0 ; phasein(2) = 0 ; phasein(3) = pivot;
    [ph0,ph1,newpivot] = phaseit(fdata,phasein);   % activate GUI for phase adjust
    if isempty(ph0)
        ph0 = 0;
    end
    if isempty(ph1)
        ph1 = 0;
    end
    if isempty(newpivot)
        newpivot = pivot;
    end
    set(ph0han,'string',sprintf('%8.3f',phase0+ph0));
    set(ph1han,'string',sprintf('%8.3f',phase1+ph1));
    showOutput(hObject,handles);
    set(gcbo,'value',0)
end

% --- Executes on button press in ZF_menu.
function ZF_menu_Callback(hObject, eventdata, handles)
% set zero fill options
if get(findobj('tag','help_button'),'value') == 1
    printhelp('zf')
    return
end
param = handles.param;
current_npts = param.zfpts;
n2 = size(param.fid,1);
nzf = fix(get(gcbo,'value'))  ;
if  nzf == 1
    len = n2;
else
        len =  n2*2^(nzf-1);
end
param.zfpts = len;
handles.param = param;
guidata(hObject,handles)
showOutput(hObject,handles);


function em_val_Callback(hObject, eventdata, handles)
% set line width parameter for exponential apodization. Units are kHz!
if get(findobj('tag','help_button'),'value') == 1
    printhelp('em')
    return
end
param = handles.param;
param.lb = str2num(get(hObject,'String'));
handles.param = param;
guidata(hObject, handles);
showOutput(hObject,handles);

% --- Executes during object creation, after setting all properties.
function em_val_CreateFcn(hObject, eventdata, handles)
% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end

function OMAS_offset_Callback(hObject, eventdata, handles)
% set offset from exact MAS angle. Units are degrees.
if get(findobj('tag','help_button'),'value') == 1
    printhelp('omas_offset')
    return
endif get(findobj('tag','help_button'),'value') == 1
    printhelp('omas_offset')
    return
end
param = handles.param;
param.beta_r = eval(get(hObject,'String'));
handles.param = param;
guidata(hObject,handles);
showdata(handles,'other');

% --- Executes during object creation, after setting all properties.
function OMAS_offset_CreateFcn(hObject, eventdata, handles)
% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end

% --- Executes on selection change in delays.
function delays_Callback(hObject, eventdata, handles)
% edit the delay list. Units are set by a button group and can be sec, millisec, or microsec
if get(findobj('tag','help_button'),'value') == 1
    printhelp('delaylist')
    return
end
param = handles.param;
param.delays = str2num(get(hObject,'String'));
current_units = param.deunits;
h = findobj('tag','units');
hh = get(h,'SelectedObject');
switch get(hh,'tag')
    case 'radiobutton2'
        param.deunits = 'usec';
        factor = 1.e-6;
    case 'radiobutton3'
        param.deunits = 'msec';
        factor = 1.e-3;
    case 'radiobutton4'
        param.deunits = 'sec';
        factor = 1;
end
param.taulist = param.delays*factor;
handles.param = param;
guidata(hObject,handles);
showdata(handles,'other');

% --- Executes during object creation, after setting all properties.
function delays_CreateFcn(hObject, eventdata, handles)
% Hint: listbox controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end

function pairs_Callback(hObject, eventdata, handles)
% sets number of sideband pairs for Floquet simulation routines
% the number of Floquet states per site is 2*npairs + 1
if get(findobj('tag','help_button'),'value') == 1
    printhelp('npairs')
    return
end
param = handles.param ;
param.pairs = str2num(get(hObject,'String'));
handles.param = param;
guidata(hObject, handles);
showcurrentdata(handles);

% --- Executes during object creation, after setting all properties.
function pairs_CreateFcn(hObject, eventdata, handles)
% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end



% --- Executes on button press in annotate_button.
function annotate_button_Callback(hObject, eventdata, handles)
% generates an editable MATLAB figure with real part of simulated data
if get(findobj('tag','help_button'),'value') == 1
    printhelp('prettyplot')
    return
end
param = handles.param;
figure(1);
hold off % maybe there already is one...
n = length(param.spec1);
if isempty(n)
    errordlg('nothing to plot')
    return
end
sw = param.swh;
fax = ([0:(n-1)]-n/2)*sw/n;
tax = (0:(n-1))*param.time;
if get(findobj('tag','fft_button'),'value')
    plot(fax,real(param.spec1));
    set(gca,'xdir','reverse');
else
    plot(real(param.buf)) % plot time domain data vs points to aid user determination of left shifts
end

function edit_ph0_Callback(hObject, eventdata, handles)
% sets zero order phase correction (degrees)
if get(findobj('tag','help_button'),'value') == 1
    printhelp('ph0')
    return
end
param = handles.param;
param.ph0 = str2num(get(hObject,'String'));
handles.param = param;
guidata(hObject, handles);
showOutput(hObject,handles);

% --- Executes during object creation, after setting all properties.
function edit_ph0_CreateFcn(hObject, eventdata, handles)
% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end



function edit_ph1_Callback(hObject, eventdata, handles)
% sets first order phase correction (degrees)
if get(findobj('tag','help_button'),'value') == 1
    printhelp('ph1')
    return
end
param = handles.param;
param.ph1 = str2num(get(hObject,'String'));
handles.param = param;
guidata(hObject, handles);
showOutput(hObject,handles);

% --- Executes during object creation, after setting all properties.
function edit_ph1_CreateFcn(hObject, eventdata, handles)
% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end

function gm_val_Callback(hObject, eventdata, handles)
% sets line width parameter (in kHz) for gaussian apodization
if get(findobj('tag','help_button'),'value') == 1
    printhelp('gm')
    return
end
param = handles.param;
param.gb = str2num(get(hObject,'String'));
handles.param = param;
guidata(hObject, handles);
showOutput(hObject,handles);

% --- Executes during object creation, after setting all properties.
function gm_val_CreateFcn(hObject, eventdata, handles)
% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end

function data = swap(spec)
%	swaps FTT data so DC component is in the middle
%	the action takes place along columns
[m n]=size(spec);
data=zeros(m,n);m2 = fix(m/2);
data(1:m2,:)=spec(m2+1:m,:);
data(m2+1:m,:)=spec(1:m2,:);

% --- Executes on selection change in spin.
function spin_Callback(hObject, eventdata, handles)
% sets (numeric) value of spin I from menu
if get(findobj('tag','help_button'),'value') == 1
    printhelp('spin')
    return
end
spinval = [0.5 1 1.5 2.5 3.5 4.5];
param = handles.param;
index = fix(get(hObject,'value'));
param.spin = spinval(index);
handles.param = param;
guidata(hObject, handles);

% --- Executes during object creation, after setting all properties.
function spin_CreateFcn(hObject, eventdata, handles)
% Hint: popupmenu controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end

% --- Executes on selection change in taulist.
function taulist_Callback(hObject, eventdata, handles)
% select a value from the list of delay times IN THE T1Z/T1Q ANALYSIS WINDOW for plotting
if get(findobj('tag','help_button'),'value') == 1
    printhelp('dlomit')
    return
end
val = get(hObject,'value');
param = handles.param;
[npts,ntau] = size(param.buf);
fax = ([0:(npts-1)]'-npts/2)*param.swh/npts;
axes(handles.window_a);
plot(fax,real(param.buf(:,val)));
set(handles.window_a,'userdata',[fax real(param.buf(:,val))]);

% --- Executes during object creation, after setting all properties.
function taulist_CreateFcn(hObject, eventdata, handles)
% Hint: listbox controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end

% --- Executes on button press in use_tau.
function use_tau_Callback(hObject, eventdata, handles)
% identify which tau-values to use by fitting routines
if get(findobj('tag','help_button'),'value') == 1
    printhelp('dlomit')
    return
end
param = handles.param;
vh = findobj('tag','taulist');
n = get(vh,'value');
mod = get(vh,'string');
s = strtrim(mod(n));
sch = char(s) ;
if strcmp('***',sch(1:3)) % strip the *** identifier!!
    s = sch(4:length(sch)); 
else
    return % don't monkey with it
end
mod(n) = {s} ;
set(vh,'string',mod) ;
param.fit_taus(n) = 1;
handles.param = param ;
guidata(hObject, handles);

% --- Executes on button press in omit_tau.
function omit_tau_Callback(hObject, eventdata, handles)
% identify which tau-values to omit by fitting routines
if get(findobj('tag','help_button'),'value') == 1
    printhelp('dlomit')
    return
end
if get(findobj('tag','help_button'),'value') == 1
    printhelp('dlomit')
    return
end
param = handles.param;
vh = findobj('tag','taulist');
n = get(vh,'value');
mod = get(vh,'string');
s = strtrim(mod(n));
sch = char(s) ;
if strcmp('*',sch(1)) % don't add a second identifier!!
    return
end
s = strcat('***',s); mod(n) = s ;
set(vh,'string',mod) ;
param.fit_taus(n) = 0;
handles.param = param ;
guidata(hObject, handles);

% --- Executes on button press in use_all.
function use_all_Callback(hObject, eventdata, handles)
% tell fitting routines to use all tau-values 
if get(findobj('tag','help_button'),'value') == 1
    printhelp('dlomit')
    return
end
param = handles.param;
vh = findobj('tag','taulist');
mod = get(vh,'string');
for jj = 1:length(mod)
    s = strtrim(mod(jj));
    sch = char(s) ;
    if strcmp('***',sch(1:3)) % strip the *** identifier!!
        s = sch(4:length(sch));
        mod(jj) = {s};
    else
        mod(jj) = s ;
    end
    param.fit_taus(jj) = 1;
end
set(vh,'string',mod) ;
handles.param = param ;
guidata(hObject, handles);

% --- Executes on button press in baseline.
function baseline_Callback(hObject, eventdata, handles)
% set base line for fitting routines
if get(findobj('tag','help_button'),'value') == 1
    printhelp('baseline')
    return
end
param = handles.param;
[npts,ntau] = size(param.buf);
fax = ([0:(npts-1)]'-npts/2)*param.swh/npts;
axes(handles.window_a);
switch param.experiment
    case '2H T1Z'
        plot(real(param.buf(:,ntau)));
        showthis = ntau;
    case '2H T1Q'
        plot(real(param.buf(:,1)));
        showthis = 1;
        case 'T1Zspikelet'
        plot(real(param.buf(:,ntau)));
        showthis = ntau;
end
[x,y]= ginput(2);
x = floor(x);
y = real(param.buf(x,showthis));
if x(2) == x(1)
    errordlg('Please choose a wider region');
    return
end
if x(2) < x(1) % silly user defined window backwards
    x = flipud(x);
    y = flipud(y);
end
slope = (y(2)-y(1))/(fax(x(2))-fax(x(1)));
cept = y(1)-slope*fax(x(1));
expfax = [fax(x(1):x(2))];
baseline = cept + slope*expfax;
yexp = real(param.buf((x(1):x(2)),showthis));
plot(expfax,yexp-baseline+0.1*max(yexp),expfax,baseline+0.1*max(yexp),'r:');
set(handles.window_a,'userdata',[expfax yexp]);
param.exp_region = x;
param.fits_table = []; % clear out any previous fits
param.image_table = [];
handles.param = param ;
guidata(hObject, handles);
set(findobj('tag','getpoints'),'enable','on')

% --- Executes on selection change in fitfun.
function fitfun_Callback(hObject, eventdata, handles)
% select a fitting function
if get(findobj('tag','help_button'),'value') == 1
    printhelp('fitfun')
    return
end
param = handles.param;
fit_option{1} = 'Z(t) = Z(inf) + [Z(0)-Z(inf)]*exp(-t/T1Z)';
fit_option{2} = 'Q(t) = Q(0)exp(-t/T1Q)';
fit_option{3} = 'Mxy(t) = Mxy(0)exp(-t/T2)';
fit_option{4} = 'Z(t) = Z(inf) + [Z(0)-Z(inf)]*exp(-t/T1Z)';
switch get(hObject,'Value')
    case 1
        printhelp('fitfun')
    otherwise
        ms = cell(4,1);
        ms{1}= param.experiment;
        ms{2} = '  ';
        ms{3} = 'Fitting Function';
        ms{4} = fit_option{get(hObject,'Value')-1};
        set(findobj('tag','main_text'),'string',ms,'val',1);
        param.fit_label = ms;
end
handles.param = param;
guidata(hObject,handles);

% --- Executes during object creation, after setting all properties.
function fitfun_CreateFcn(hObject, eventdata, handles)
% Hint: popupmenu controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end

% --- Executes on button press in fit_button.
function fit_button_Callback(hObject, eventdata, handles)
% fit one user selected curve, T1Z or T1Q only!
switch get(findobj('tag','help_button'),'value')
    case 1
        printhelp('fitone')
        return
end
param = handles.param;
points = param.M_list;
[nmax,ntau] = size(points);
% get table to annotate with fit results
table = get(findobj('tag','main_text'),'string'); 
% get one recovery curve
index = get(findobj('tag','main_text'),'value')-6; % first point enty is on 7th line
if index < 1 || index > nmax
    errordlg('Please choose a point from the main text window')
    return
end
% generate list of points to fit, and a corresponding list of
% un-starred tau- values (as column vectors)
tval = find(param.fit_taus);
R = points(index,tval)';
tval = param.taulist(tval)';% now, tval IS the list of tau-values, not pointers!
nt = length(tval);
if isequal(param.experiment,'2H T1Z') || isequal(param.experiment,'T1Zspikelet') % 3-parameter fit
    table(6) = {'Frequency    M(inf)       T1Z        R1Z       M(0)'};
    set(findobj('tag','main_text'),'string',table);
    last = R(nt); % assume perfect inversion
    Z = 0.5*(1-R/last) ; % the Z-values will be fit...
    Z0 = -1;
    rz0 = log(Z(2)/Z(1))/(tval(1)-tval(2)); % initial guess for 1/T1Z is initial rate from 1st 2 points
    pz = [rz0 -last last]; % initial guess for [1/T1Z, Z(0), Z(inf)]
    pfit = fminsearch(@(pfit) t1ir(pfit,tval,R),pz); % simplex fit routine
    param.fits_table(index,1) = 1/pfit(1) ; % best fit T1Z = 1/R1Z
    param.fits_table(index,2) = pfit(2) ; % best fit Z(0)
    param.fits_table(index,3) = pfit(3) ; % best fit Z(inf) 
    Rfit = pfit(3) + (pfit(2)-pfit(3))*exp(-pfit(1)*tval); % best fit recovery curve
    Zfit = 0.5*(1-Rfit/Rfit(nt));
    % update display, erasing any old fit values
    table(index+6) = strcat(param.image_table(index+6),sprintf('%10.3f %10.3f %10.3f',1/pfit(1),pfit(1),pfit(2)));
    set(findobj('tag','main_text'),'string',table);
    axes(handles.window_b);
    semilogy(tval(1:nt-1),Z(1:nt-1),'ro',tval(1:nt-1),Zfit(1:nt-1),'b-','MarkerFaceColor','g');
    set(handles.window_b,'userdata','onefit')
elseif isequal(param.experiment,'2H T1Q') % two parameter fit
    table(6) = {'Frequency    M(0)            T1Q         R1Q'};
    set(findobj('tag','main_text'),'string',table);
    Z = abs(R/R(1)) ; % the Z-values will be fit...
    rz0 = log(Z(2)/Z(1))/(tval(1)-tval(2)); % initial guess for 1/T1Z is initial rate from 1st 2 points
    pz = [rz0 Z(1)]; % initial guess for [1/T1Z, Z(0)]
    pfit = fminsearch(@(pfit) t1q(pfit,tval,R),pz); % simplex fit routine
    param.fits_table(index,1) = 1/pfit(1) ; % best fit T1Z = 1/R1Z
    param.fits_table(index,2) = pfit(2) ; % best fit Z(0) 
    zfit = exp(-pfit(1)*tval); % best fit recovery curve
    % update display, erasing any old fit values
    table(index+6) = strcat(param.image_table(index+6),sprintf('%10.3f %10.3f ',1/pfit(1),pfit(1)));
    set(findobj('tag','main_text'),'string',table);
    axes(handles.window_b);
    semilogy(tval,Z,'ro',tval,zfit,'b-','MarkerFaceColor','g');
    set(handles.window_b,'userdata','onefit')
end
set(findobj('tag','profile_button'),'enable','off');

% --- Executes on button press in stack_plot_button.
function stack_plot_button_Callback(hObject, eventdata, handles)
% draw a stack plot
switch get(findobj('tag','help_button'),'value')
    case 1
        printhelp('stack')
        return
end
param = handles.param;
if isempty(param.buf)
    errordlg('Nothing to plot: Please process the data first...');
    return
end
hinc = eval(get(findobj('tag','h_offset'),'string'));
vinc = eval(get(findobj('tag','v_offset'),'string'));
axes(handles.window_b) ; 
[npts,ntau] =  size(param.buf) ; 
fax = ((0:(npts-1))'-npts/2)*param.swh/npts;
plot(fax,real(param.buf(:,1)),'b');
hold on
for jj = 2:ntau
    plot(fax-(jj-1)*hinc*fax(npts),real(param.buf(:,jj))+vinc*(jj-1),'b');
end
hold off
set(gca,'Xdir','reverse') ; 
set(handles.window_b,'userdata','stackplot')

% --- Executes on button press in profile_button.
function profile_button_Callback(hObject, eventdata, handles)
% draw anisotropy profile for T1Z or T1Q data
if get(findobj('tag','help_button'),'value') == 1
    printhelp('profile')
    return
end
param = handles.param;
X = param.exp_region;
ptslist = param.ptslist';
axes(handles.window_a);
[npts,ntau] = size(param.buf);
fax = ((0:(npts-1))'-npts/2)*param.swh/npts;
expfax = fax(X(1):X(2));
yspec = real(param.buf(X(1):X(2),ntau));
plot(expfax,yspec,fax(ptslist),real(param.buf(ptslist,ntau)),'ro');
set(handles.window_a,'userdata','getpoints')
Xlim = get(gca,'Xlim');
tval = find(param.fit_taus);% pointers to unstarred tau-values
R1 = param.fits_table(:,1);% first column assumed to contain relaxation RATES
axes(handles.window_b)
plot(fax(ptslist),R1,'ro');
set(gca,'Xlim',Xlim);
set(handles.window_b,'userdata','profile')

% --- Executes on button press in profile_button.
function delays_button_Callback(hObject, eventdata, handles)
% Despite its name (a survival from early coding...) this
% callback responds to the ANALYZE button.
% It allows user to construct anisotropy profiles 
% from 2H T1Z or T1Q simulations
if get(findobj('tag','help_button'),'value') == 1
    printhelp('analyze')
    return
end
setState('analyze')
param = handles.param;
ftflag = get(findobj('tag','fft_button'),'value');
if isequal(param.experiment,'2H T1Z') || isequal(param.experiment,'2H T1Q') || isequal(param.experiment,'T1Zspikelet')
        if ~ftflag
            errordlg('first, you must VIEW output and perform FT')
            return
        end
        % make a sorted list of tau-values and corresponding spectra
        tlist = param.taulist(:);
        [npts,ntau] = size(param.buf);
        dlist = cell(ntau,1);
        for jj = 1:ntau
            dlist{jj}= sprintf('%7.3f',tlist(jj));
        end
        [tt,in] = sort(tlist);
        tmp = dlist ; tspec = param.buf;
        for jj = 1:ntau
            dlist{jj} = tmp{in(jj)};
            param.buf(:,jj) = tspec(:,in(jj));
        end
        param.fit_taus = ones(ntau,1);
        set(findobj('tag','taulist'),'string',dlist,'value',1);
        ha = findobj('tag','window_a');
        hb = findobj('tag','window_b');
        axes(handles.window_a);
        fax = ([0:(npts-1)]'-npts/2)*param.swh/npts;
        plot(fax,real(param.buf(:,1)));
        set(handles.window_a,'userdata',[fax real(param.buf(:,1))]);
else
        errordlg('nothing to analyze')
end
handles.param = param;
guidata(hObject, handles);

% --- Executes on button press in drawa.
function drawa_Callback(hObject, eventdata, handles)
% makes an editable MATLAB figure from the upper window of the 
% EXPRESS output display axes
if get(findobj('tag','help_button'),'value') == 1
    printhelp('prettyplot')
    return
end
data = get(handles.window_a,'userdata');
figure(1) ; hold off ; % maybe there already is a figure 1...
if strcmp(data,'getpoints')
    param = handles.param;
    X = param.exp_region;
    ptslist = param.ptslist'; 
    [npts,ntau] = size(param.buf);
    fax = ((0:(npts-1))'-npts/2)*param.swh/npts;
    expfax = fax(X(1):X(2));
    switch param.experiment
        case '2H T1Z'% 2H T1Z: we need the "sum" magnetization for Zeeman Order
            showthis = ntau;
        case '2H T1Q'% 2H T1Z: we need the "difference" magnetization for Zeeman Order
            showthis = 1;
        case 'T1Zspikelet'% 2H T1Z: we need the "sum" magnetization for Zeeman Order
            showthis = ntau;
    end
    yspec = real(param.buf(X(1):X(2),showthis));
    plot(expfax,yspec,fax(ptslist),real(param.buf(ptslist,showthis)),'ro');

else
    plot(data(:,1) , data(:,2));
end

% --- Executes on button press in drawb.
function drawb_Callback(hObject, eventdata, handles)
% makes an editable MATLAB figure from the lower window of the 
% EXPRESS output display axes
if get(findobj('tag','help_button'),'value') == 1
    printhelp('prettyplot')
    return
end
data = get(handles.window_b,'userdata');
figure(2) ; hold off ; % maybe there already is a figure 2...
if strcmp(data,'stackplot')
    param = handles.param;
    [n,ntau] =  size(param.buf) ;
    fax = ((0:(n-1))'-n/2)*param.swh/n;
    plot(fax,real(param.buf(:,1)),'b');
    hinc = eval(get(findobj('tag','h_offset'),'string'));
    vinc = eval(get(findobj('tag','v_offset'),'string'));
    hold on
    for jj = 2:ntau
        plot(fax-(jj-1)*hinc*fax(n),real(param.buf(:,jj))+vinc*(jj-1),'b');
    end
    hold off
    set(gca,'Xdir','reverse');
elseif strcmp(data,'onefit')
    param = handles.param;
    tval = find(param.fit_taus);
    points = param.M_list;
    index = get(findobj('tag','main_text'),'value')-6; % first point enty is on 7th line    
    R = points(index,tval)';
    tval = param.taulist(tval)';
    nt = length(tval);
    last = R(nt);
    Z = 0.5*(1-R/last) ; % these Z-values were fit...
    pfit(1) = param.fits_table(index,1) ; % best fit R1Z = 1/T1Z
    pfit(2) = param.fits_table(index,2) ; % best fit Z(0)
    pfit(3) = param.fits_table(index,3) ; % best fit Z(inf) 
    Zfit = pfit(3) + (pfit(2)-pfit(3))*exp(-pfit(1)*tval); % best fit recovery curve
    semilogy(tval(1:nt-1),Z(1:nt-1),'ro',tval(1:nt-1),Zfit(1:nt-1),'b-','MarkerFaceColor','g'); 
elseif strcmp(data,'profile')
    param = handles.param;
    X = param.exp_region;
    ptslist = param.ptslist';
    [npts,ntau] = size(param.buf);
    fax = ((0:(npts-1))'-npts/2)*param.swh/npts;
    R1 = param.fits_table(:,1);
    plot(fax(ptslist),R1,'ro');
    set(handles.window_b,'userdata','profile')
else
    plot(data(:,1) , data(:,2));
end

% --- Executes on button press in fit_all_button.
function fit_all_button_Callback(hObject, eventdata, handles)
% fits all the user-selected points to T1Z or T1Q fitting functions
switch get(findobj('tag','help_button'),'value')
    case 1
        printhelp('fitall')
        return
end
param = handles.param;
points = param.M_list;
param.fits_table = []; % clear old fits_table
tval = find(param.fit_taus);% pointers to unstarred tau-values
tau = param.taulist(tval)';% list of unstarred tau-values
nt = length(tval);
[nmax,ntau] = size(points);
% get table to annotate with fit results
table = get(findobj('tag','main_text'),'string'); 
% loop over recovery curves
for index = 1:nmax
    R = points(index,tval)';
    if isequal(param.experiment,'2H T1Z') ||isequal(param.experiment,'T1Zspikelet')
        table(6) = {'Frequency    M(inf)       T1Z        R1Z       M(0)'};
        set(findobj('tag','main_text'),'string',table);
        last = R(nt); % assume perfect inversion
        Z = 0.5*(1-R/last) ; % the Z-values will be fit...
        if index ==1
            rz0 = log(Z(2)/Z(1))/(tau(1)-tau(2)); % initial guess for 1/T1Z is initial rate from 1st 2 points
        else
            rz0 = previous_rz;
        end
        pz = [rz0 -last last]; % intial guess for [1/T1Z, Z(0), Z(inf)]
        pfit = fminsearch(@(pfit) t1ir(pfit,tau,R),pz); % simplex fit routine
        previous_rz = pfit(1);
        param.fits_table(index,1) = 1/pfit(1) ; % best fit T1Z = 1/T1Z
        param.fits_table(index,2) = pfit(2) ; % best fit Z(0)
        param.fits_table(index,3) = pfit(3) ; % best fit Z(inf)
        Rfit = pfit(3) + (pfit(2)-pfit(3))*exp(-pfit(1)*tau); % best fit recovery curve
        Zfit = 0.5*(1-Rfit/Rfit(nt)) ;
        % update display, erasing any old fit values
        table(index+6) = strcat(param.image_table(index+6),sprintf('%10.3f %10.3f %10.3f',1/pfit(1),pfit(1),pfit(2)));
        set(findobj('tag','main_text'),'string',table);
        axes(handles.window_b);
        semilogy(tau(1:nt-1),Z(1:nt-1),'ro',tau(1:nt-1),Zfit(1:nt-1),'b-','MarkerFaceColor','g');
    elseif isequal(param.experiment,'2H T1Q') % two parameter fit
        table(6) = {'Frequency    M(0)           T1Q       R1Q'};
        set(findobj('tag','main_text'),'string',table);
        Z = abs(R/R(1)) ; % the Z-values will be fit...
        rz0 = log(Z(2)/Z(1))/(tau(1)-tau(2)); % initial guess for 1/T1Q is initial rate from 1st 2 points
        pz = [rz0 Z(1)]; % initial guess for [1/T1Z, Z(0)]
        pfit = fminsearch(@(pfit) t1q(pfit,tau,Z),pz); % simplex fit routine
        param.fits_table(index,1) = 1/pfit(1) ; % best fit T1Q = 1/R1Q
        param.fits_table(index,2) = pfit(2) ; % best fit Z(0)
        zfit = exp(-pfit(1)*tau); % best fit recovery curve
        % update display, erasing any old fit values
        table(index+6) = strcat(param.image_table(index+6),sprintf('%10.3f %10.3f ',1/pfit(1),pfit(1)));
        set(findobj('tag','main_text'),'string',table);
        axes(handles.window_b);
        semilogy(tau,Z,'ro',tau,zfit,'b-','MarkerFaceColor','g');
        set(handles.window_b,'userdata','onefit')
    end
end
set(findobj('tag','profile_button'),'enable','on');
handles.param = param;
guidata(hObject,handles);

function getpoints_Callback(hObject, eventdata, handles)
% This callback builds a table of line shape points
% to be fit for T1Z (QE or QCPMG Spikelet) or T1Q anisotropy computation & display
if get(findobj('tag','help_button'),'value') == 1
    printhelp('getpoints')
    return
end
if get(findobj('tag','fitfun'),'val') == 1
    errordlg('First select a fitting function')
    return
end
param = handles.param;
param.M_list = [];
param.ptslist = []; % clear old points lists
if isequal(param.experiment,'T1Zspikelet') % Gets all spikelet echoes in a user specified range
    [npts,ntau] = size(param.buf);
    X = param.exp_region;
    yspec = real(param.buf(X(1):X(2),ntau));
    pax = [X(1):X(2)];
    p = fpeaks(pax,real(param.buf(X(1):X(2),ntau)),0.002,0.7,1,3);
    ptslist = round(p(:,2));
    threshold = max(real(param.buf(X(1):X(2),ntau)));
    ind = find(real(param.buf(ptslist,ntau)) > 0.1*threshold);
    ptslist = ptslist(ind);
    set(hObject,'String',sprintf('%d',length(ptslist)-1))
else
X = param.exp_region;
span = abs(X(2)-X(1));
K = abs(fix(str2double(get(hObject,'String'))));
if K > span
    errordlg('Too many points for displayed region')
    return
end
if K == 0 || K==1
    set(hObject,'String','1')
    ptslist = (X(1)+X(2))/2;
else
    inc = span/K;
    ptslist = floor((X(1)+0.1*span):inc:(X(2)-0.1*span)');
end
end
axes(handles.window_a);
[npts,ntau] = size(param.buf);
fax = ((0:(npts-1))'-npts/2)*param.swh/npts;
expfax = fax(X(1):X(2));
switch param.experiment
    case '2H T1Z'% 2H T1Z: we need the "sum" magnetization for Zeeman Order
        showthis = ntau;
    case '2H T1Q'% 2H T1Z: we need the "difference" magnetization for Zeeman Order
        showthis = 1;
    case 'T1Zspikelet'% 2H T1Z: we need the "sum" magnetization for Zeeman Order
        showthis = ntau;
end
yspec = real(param.buf(X(1):X(2),showthis));
plot(expfax,yspec,fax(ptslist),real(param.buf(ptslist,showthis)),'ro');
set(handles.window_a,'userdata','getpoints');
m_tau_list = real(param.buf(ptslist,:));

switch param.experiment
    case '2H T1Z'% 2H T1Z: we need the "sum" magnetization for Zeeman Order
        for jj = 1: length(ptslist)
            kk = ptslist(jj);
            m_tau_list(jj,:) = 0.5*(real(param.buf(kk,:))+real(param.buf(npts+1-kk,:)));
        end
    case '2H T1Q'% 2H T1Z: we need the "difference" magnetization for Zeeman Order
        for jj = 1: length(ptslist)
            kk = ptslist(jj);
            m_tau_list(jj,:) = 0.5*(real(param.buf(kk,:))-real(param.buf(npts+1-kk,:)));
        end
    case 'T1Zspikelet' % just get the peak height
        for jj = 1: length(ptslist)
            kk = ptslist(jj);
            m_tau_list(jj,:) = real(param.buf(kk,:));
        end
end
param.M_list = m_tau_list;
param.ptslist = ptslist;
% Make table of points for display in main text window
Mlist = cell(length(ptslist)+ 1,1);
Mlist{1} = 'Frequency    M(inf)';
for jj = 2:length(ptslist)+1
    Mlist{jj} = sprintf('%7.3f %s %7.3d',fax(ptslist(jj-1)),'  ', m_tau_list(jj-1,ntau));
end
header = param.fit_label;
header = header(1:4);
hs = size(header,1); ms = size(Mlist,1);
st = cell(hs+ms+1,1);
st(1:hs) = header;
st(hs+1) = {' '};
for jj = 1:ms
    st(hs+1+jj) = Mlist(jj);
end
set(findobj('tag','main_text'),'string',st);
param.image_table = st;
handles.param = param;
guidata(hObject,handles);

% --- Executes during object creation, after setting all properties.
function getpoints_CreateFcn(hObject, eventdata, handles)
% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end

function h_offset_Callback(hObject, eventdata, handles)
% sets horizontal offset for STACKPLOT 
if get(findobj('tag','help_button'),'value') == 1
    printhelp('stack')
    return
end

% --- Executes during object creation, after setting all properties.
function h_offset_CreateFcn(hObject, eventdata, handles)
% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end

function v_offset_Callback(hObject, eventdata, handles)
% sets vertical offset for STACKPLOT 
if get(findobj('tag','help_button'),'value') == 1
    printhelp('stack')
    return
end

% --- Executes during object creation, after setting all properties.
function v_offset_CreateFcn(hObject, eventdata, handles)
% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end

function sumsq = t1ir(x,tau,data)
%
% x(3) = m(inf) ; x(2) = m(0) ; x(1) = 1/T1Z 
% This one is for 3-parameter, single exponential fit to T1 or T2 data
sim = x(3)+(x(2)-x(3))*exp(-x(1)*tau) ;
dev = sim(:)' - data(:)' ;
sumsq = sum(dev.*dev);

function sumsq = t1q(x,tau,data)
%
% x(2) = m(0) ;  x(1) = 1/T1Q 
% This one is for 2-parameter, single exponential fit to T1Q or T2 data
sim = x(2)*exp(-x(1)*tau) ;
dev = sim(:)' - data(:)' ;
sumsq = sum(dev.*dev);

function reset_view
% resets view options to no left shift, no zero fill, no apodization, and
% no FT
set(findobj('tag','ZF_menu'),'value',1); % reset to no zero-fill
set(findobj('tag','edit_left_shift'),'string','0'); % reset to no left shift
set(findobj('tag','fft_button'),'value',0); % reset to no FT
set(findobj('tag','em_val'),'string','0'); % reset to no EM
set(findobj('tag','gm_val'),'string','0'); % reset to no GM
set(findobj('tag','gme'),'string','0'); % also for echoes
set(findobj('tag','delays_button'),'enable','off'); % disable ANALYZE button


% --- Executes on selection change in expt_select.
function expt_select_Callback(hObject, eventdata, handles)
% changes user-specified simulation algorithm
% maintains consistency with a similar menu in OTHER panel
% this one is in the RUN panel
if get(findobj('tag','help_button'),'value') == 1
    printhelp('expt')
    return
end
param = handles.param;
mastags = param.mastags ; efgtags = param.efgtags ; csatags = param.csatags;
delaytags = param.delaytags; 
val = get(hObject,'Value');
set(findobj('Tag','exp_popup'),'Value',val);
set(findobj('Tag','tilestyle'),'Value',1)
param.nact = set_actual_n(handles);
switch val
    case 1
        param.experiment = '2H QE';
        set(findobj('Tag','spin'),'Value',2)
        set([mastags delaytags],'visible','off')
        set(efgtags,'visible','on');
    case 2
        param.experiment = '2H Spikelets';
        set(findobj('Tag','spin'),'Value',2)
        set([mastags delaytags csatags],'visible','off')
        set(efgtags,'visible','on');
    case 3
        param.experiment = '2H T1Z';
        set([mastags csatags],'visible','off')
        set(findobj('Tag','spin'),'Value',2)
        set([efgtags delaytags],'visible','on');
    case 4
        param.experiment = '2H T1Q';
        set([mastags csatags],'visible','off')
        set(findobj('Tag','spin'),'Value',2)
        set([efgtags delaytags],'visible','on');
    case 5
        param.experiment = '2H OMAS';
        set(findobj('Tag','spin'),'Value',2)
        set([mastags efgtags],'visible','on')
        set([delaytags csatags],'visible','off');
    case 6
        param.experiment = '2H OMAS w/CSA';
        set(findobj('Tag','spin'),'Value',2)
        set([mastags csatags efgtags],'visible','on')
        set(delaytags,'visible','off');
    case 7
        param.experiment = 'I > 1 Static C.T.';
        set([mastags delaytags] ,'visible','off')
        set([csatags efgtags],'visible','on');
    case 8
        param.experiment = 'I > 1 Spikelets';
        set([mastags delaytags],'visible','off')
        set([csatags efgtags] ,'visible','on');
    case 9 
        param.experiment = 'I > 1 OMAS';
        set([mastags csatags efgtags],'visible','on')
        set(findobj('Tag','tilestyle'),'Value',3)
        set(delaytags,'visible','off');
        param.nact = set_actual_n(handles);
    case 10
        param.experiment = 'I = 1/2 1PULS';
        set(findobj('Tag','spin'),'Value',1)
        set([mastags csatags],'visible','on')
        set([delaytags efgtags],'visible','off');
    case 11
        param.experiment = 'T1Zspikelet';
        set(findobj('Tag','spin'),'Value',2)
        set([mastags delaytags csatags],'visible','off')
        set(efgtags,'visible','on');
        set([efgtags delaytags],'visible','on');
    case 12
        param.experiment = 'user routine 2';
        % insert set-up code if necessary for user-defined experiments
        % this should match code in the expt_select callback routine
    case 13
        param.experiment = 'user routine 3';
        % insert set-up code if necessary for user-defined experiments
        % this should match code in the expt_select callback routine
end
handles.param = param;
guidata(hObject, handles);
showcurrentdata(handles);

% --- Executes during object creation, after setting all properties.
function expt_select_CreateFcn(hObject, eventdata, handles)
% Hint: popupmenu controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end

% --- Executes on button press in editrates.
function editrates_Callback(hObject, eventdata, handles)
% opens a mini-gui that displays an editable version of the one-frame rate matrix
% command-line complaints form this routine about the number of rows and columns
% can be safely ignored
if get(findobj('tag','help_button'),'value') == 1
    printhelp('customrates')
    return
end
param = handles.param;
if isempty(param.onek)
    errordlg('must define one-frame parameters')
    return
end
scrsz = get(0,'ScreenSize');
scr = figure('Position',[1 scrsz(4)/2 0.75*scrsz(3) scrsz(4)/2],...
    'Name','One-frame Rate Matrix','NumberTitle','off');
hax =axes('units','pixels','position',[ 10 10  800 450]);
if param.runflags(1) ==1    
    data = param.custom_onek;
else
    data = param.onek;
end
wts = param.wts;
n = size(data,1);
clab = cell(n,1);
for jj = 1:n
    clab{jj} = num2str(jj);
end
      P = struct('modal',1, 'nVisibleRows',10 );
      P.nVisibleCols = 10;
      P.highlightCell = 0;
      P.colLabels = clab;
      P.precision = '%.7g';
      P.fontsize = 11;
      k_new = tedit(hax,data,P);
      if isempty(k_new) % user pressed cancel button
          param.runflags(1) = 0;
          handles.param = param;
          guidata(hObject, handles);
          set(hObject,'value',0)
          return
      else
          param.runflags(1) = 1; % user created a custom one-frame K matrix
      end      
nn = size(k_new,1);
for jj = 2:nn           % first, make sure new k-matrix is still symmetric          
    for kk = 1:jj    
        if kk < jj      
           k_new(jj,kk) = k_new(kk,jj);
        end
    end
end
for jj = 1:nn           % detailed balance for SYMMETRIZED matrix
    tmp = 0;
    for kk = 1:nn
        if kk ~= jj      
           tmp = tmp - k_new(jj,kk)*sqrt(wts(kk)/wts(jj));
        end
        k_new(jj,jj) = tmp;
    end
end
param.custom_onek = k_new;
handles.param = param;
guidata(hObject, handles);
set(hObject,'value',0)
    
% --- Executes on button press in view_sites.
function view_sites_Callback(hObject, eventdata, handles)
% turns on a mini-gui that shows site orientations in current frame, with animation
if get(findobj('tag','help_button'),'value') == 1
    printhelp('showmodel')
    return
end
param = handles.param;
cf = param.currentframe;
frames = handles.frames;
model = frames{cf}.model;
val = get(findobj('tag','popup_model'),'Value');
        switch val
            case 1
                frames{param.currentframe}.model = 'all';
            case 2
                frames{param.currentframe}.model = 'cha';
            case 3
                frames{param.currentframe}.model = 'cir';
            case 4
                frames{param.currentframe}.model = 'oth';
        end
frames{param.currentframe}.template = creatematrix(frames{param.currentframe}.sites,frames{param.currentframe}.model,frames{param.currentframe}.rate,handles);
handles.frames = frames;

if isequal(get(findobj('tag','expt_select'),'value'),10)
    alpha = frames{cf}.csa_phi;
    beta = frames{cf}.csa_theta;
    gamma = frames{cf}.csa_psi;
else
    alpha = frames{cf}.cq_phi;
    beta = frames{cf}.cq_theta;
    gamma = frames{cf}.cq_psi;
end
siteangles = [alpha(:) beta(:) gamma(:)];
template = frames{param.currentframe}.template;
showmodel(siteangles, model,template)
handles.param = param;
    guidata(hObject, handles);

% --- Executes on selection change in tilestyle.
function tilestyle_Callback(hObject, eventdata, handles)
% menu of tiling options
% caution: use z3 or kar options only for I > 1 MAS, since this is the only
% expt. that needs 3-angle tiling
% also, msh tiles the upper hemisphere only and should not be used for T1Z/T1Q simulations
if get(findobj('tag','help_button'),'value') == 1
    printhelp('tilestyle')
    return
end
param = handles.param;
param.nact = set_actual_n(handles);
s = get(hObject,'string');
v = get(hObject,'val');
param.tiletype = s{v};
handles.param = param;
    guidata(hObject, handles);
showcurrentdata(handles);
% --- Executes during object creation, after setting all properties.
function tilestyle_CreateFcn(hObject, eventdata, handles)
% hObject    handle to tilestyle (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: popupmenu controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end

function selcbk(source,eventdata)
% called in response to pushing any of the three buttons that select units for the delay list
% makes use of MATLAB's SelectionChangeFcn property of uibuttongroup objects
if get(findobj('tag','help_button'),'value') == 1
    printhelp('delayunits')
    return
end

function radiobutton2_callback(hObject,eventdata,handles)
%dummy

function nact = set_actual_n(handles)
% determines actual number of tiles set by user selected tile type and # of beta-angles
param = handles.param;
htile =findobj('tag','tilestyle');
contents = get(htile,'string');
ttype = contents{get(htile,'Value')};
n = str2num(get(findobj('tag','edit_n'),'string'));
tmp = gettiles(n,ttype);
nact = size(tmp,1);




function title_Callback(hObject, eventdata, handles)
% hObject    handle to title (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: get(hObject,'String') returns contents of title as text
%        str2double(get(hObject,'String')) returns contents of title as a double
param = handles.param;
param.title = get(hObject,'String');
handles.param = param;
guidata(hObject, handles);
showdata(handles,'other');

% --- Executes during object creation, after setting all properties.
function title_CreateFcn(hObject, eventdata, handles)
% hObject    handle to title (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end


