function varargout = phaseit(varargin)
%
%      Displays real part of spectrum and allows interactive phase correction
%
% 
%   Calling sequence:   [ph0,ph1,piv] = phaser(spec,phase) 
%
%
%                      spec: complex frequency domain spectrum to be phased
%      phase: initial (input) phase:  phase(1): ph0 = zero  order (degrees)
%                                     phase(2): ph1 = first order (degrees)
%                                     phase(3): piv = pivot point used with ph0 and ph1
%
%      All inputs are passed to phaser_OpeningFcn via varargin.
%
%      real part of spec is displayed and user can change ph0, ph1 and
%      pivot point as needed to get best pure absorption phase.
%      the phase corrected spectrum (displayed interactively) is 
%
%      spec.*exp(i*phcorr)
%
%      where phcorr = ph0+ph1*([0:npts-1]-piv)'/npts;
%
%      All outputs returned via varargout; phased spectrum is NOT returned
%
%             ph0 = zero  order (degrees)
%             ph1 = first order (degrees)
%             piv = pivot point used with ph0 and ph1

% Begin initialization code - DO NOT EDIT
gui_Singleton = 1;
gui_State = struct('gui_Name',       mfilename, ...
                   'gui_Singleton',  gui_Singleton, ...
                   'gui_OpeningFcn', @phaseit_OpeningFcn, ...
                   'gui_OutputFcn',  @phaseit_OutputFcn, ...
                   'gui_LayoutFcn',  [] , ...
                   'gui_Callback',   []);
if nargin && ischar(varargin{1})
    gui_State.gui_Callback = str2func(varargin{1});
end

if nargout
    [varargout{1:nargout}] = gui_mainfcn(gui_State, varargin{:});
else
    gui_mainfcn(gui_State, varargin{:});
end
% End initialization code - DO NOT EDIT


% --- Executes just before phaseit is made visible.
function phaseit_OpeningFcn(hObject, eventdata, handles, varargin)
% This function has no output args, see OutputFcn.
% hObject    handle to figure
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
% varargin   command line arguments to phaseit (see VARARGIN)

% Choose default command line output for phaseit
handles.output = hObject;
data = struct('ph0',[],'ph1',[]','fax',[],'spec',[],'piv',[],'pts',[]); % parameters used by callbacks
handles.output = hObject;
data.fax = varargin{1}(:,1);
data.amp = varargin{1}(:,2);
data.ph0 = varargin{2}(1);
data.ph1 = varargin{2}(2);
data.piv = fix(varargin{2}(3));
data.pts = pi*((0:length(data.amp)-1)'-data.piv)/(180*length(data.amp));
data.amp = data.amp/max(abs(data.amp)); % scale input data to max abs value 1
handles.data = data; 
xr = [min(data.fax) max(data.fax)] ; % set fixed x-axis range
yr = [-0.2 1.0];                     % set fixed yaxis range
h = axes('XLim',xr,'YLim',yr,... 
        'Position',[.072 0.353 0.859 0.584],...  % define fixed axes for spectrum display   
        'Drawmode','fast');                      % configure for fast update using low-level draw routines
ylabel('y','Rotation',0);xlabel('frequency');
hline = line('XData',data.fax,'YData',real(data.amp),'Color','blue'); % draw input spectrum
handles.hline = hline;
ManAxMode(h)                                     % set most axes properties to "manual" for fast update

% Update handles structure
guidata(hObject, handles);

% UIWAIT makes phaseit wait for user response (see UIRESUME)
 uiwait(handles.figure1);


% --- Outputs from this function are returned to the command line.
function varargout = phaseit_OutputFcn(hObject, eventdata, handles) 
% varargout  cell array for returning output args (see VARARGOUT);
% hObject    handle to figure
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Get default command line output from handles structure
data = handles.data;
varargout{1} = data.ph0; varargout{2} = data.ph1;
varargout{3} = data.piv ;
delete(handles.figure1);

% --- Executes on slider movement.
function ph0_slider_Callback(hObject, eventdata, handles)
% hObject    handle to ph0_slider (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: get(hObject,'Value') returns position of slider
%        get(hObject,'Min') and get(hObject,'Max') to determine range of slider
ph = get(gcbo,'value');
set(findobj('tag','ph0_val') ,'string',sprintf('%5.2f',ph));
data = handles.data;
data.ph0 = ph;
phase = pi*ph/180 + data.ph1*data.pts;
handles.data = data ;
guidata(hObject, handles);
corr = real(data.amp.*exp(i*phase));
%xr = [min(data.fax) max(data.fax)] ;
%yr = [min(real(data.amp)) max(real(data.amp))] ;
%set(gca,'XLim',xr,'Ylim',yr);
set(handles.hline,'Ydata',corr);
drawnow

% --- Executes during object creation, after setting all properties.
function ph0_slider_CreateFcn(hObject, eventdata, handles)
% hObject    handle to ph0_slider (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: slider controls usually have a light gray background.
if isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor',[.9 .9 .9]);
end


% --- Executes on button press in ph1_slider.
function ph1_slider_Callback(hObject, eventdata, handles)
% hObject    handle to ph1_slider (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
ph = get(gcbo,'value');
set(findobj('tag','ph1_val'),'string',sprintf('%5.2f',ph));
data = handles.data;
data.ph1 = ph;
phase = pi*data.ph0/180 + ph*data.pts;
handles.data = data ;
guidata(hObject, handles);
corr = real(data.amp.*exp(i*phase));
set(handles.hline,'Ydata',corr); 
drawnow
handles.data = data ;
guidata(hObject, handles);


function ph0_min_Callback(hObject, eventdata, handles)
% hObject    handle to ph0_min (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: get(hObject,'String') returns contents of ph0_min as text
%        str2double(get(hObject,'String')) returns contents of ph0_min as a double
newmin = str2double(get(gcbo,'string'));
oldmax = get(findobj('tag','ph0_slider'),'max');
oldmin = get(findobj('tag','ph0_slider'),'min');
ph = get(findobj('tag','ph0_slider'),'value');
if newmin > oldmax
    errordlg('Slider maximum must exceed minimum');
    set(gcbo,'string',sprintf('%5.2f',oldmin));
    return
end
if ph < newmin ; newmin = ph; end ;
set(gcbo,'string',sprintf('%5.2f',newmin));
set(findobj('tag','ph0_slider'),'min',newmin);

% --- Executes during object creation, after setting all properties.
function ph0_min_CreateFcn(hObject, eventdata, handles)
% hObject    handle to ph0_min (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end



function ph0_max_Callback(hObject, eventdata, handles)
% hObject    handle to ph0_max (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: get(hObject,'String') returns contents of ph0_max as text
%        str2double(get(hObject,'String')) returns contents of ph0_max as a double

newmax = str2double(get(gcbo,'string'));
oldmax = get(findobj('tag','ph0_slider'),'max');
oldmin = get(findobj('tag','ph0_slider'),'min');
ph = get(findobj('tag','ph0_slider'),'value');
if newmax < oldmin
    errordlg('Slider maximum must exceed minimum');
    set(gcbo,'string',sprintf('%5.2f',oldmax));
    return
end
if ph > newmax ; newmax = ph; end ;
set(gcbo,'string',sprintf('%5.2f',newmax));
set(findobj('tag','ph0_slider'),'max',newmax);

% --- Executes during object creation, after setting all properties.
function ph0_max_CreateFcn(hObject, eventdata, handles)
% hObject    handle to ph0_max (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end



function ph1_min_Callback(hObject, eventdata, handles)
% hObject    handle to ph1_min (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: get(hObject,'String') returns contents of ph1_min as text
%        str2double(get(hObject,'String')) returns contents of ph1_min as a double
newmin = str2double(get(gcbo,'string'));
oldmax = get(findobj('tag','ph1_slider'),'max');
oldmin = get(findobj('tag','ph1_slider'),'min');
ph = get(findobj('tag','ph1_slider'),'value');
if newmin > oldmax
    errordlg('Slider maximum must exceed minimum');
    set(gcbo,'string',sprintf('%5.2f',oldmin));
    return
end
if ph < newmin ; newmin = ph; end ;
set(gcbo,'string',sprintf('%5.2f',newmin));
set(findobj('tag','ph1_slider'),'min',newmin);

% --- Executes during object creation, after setting all properties.
function ph1_min_CreateFcn(hObject, eventdata, handles)
% hObject    handle to ph1_min (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end



function ph1_max_Callback(hObject, eventdata, handles)
% hObject    handle to ph1_max (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: get(hObject,'String') returns contents of ph1_max as text
%        str2double(get(hObject,'String')) returns contents of ph1_max as a double
newmax = str2double(get(gcbo,'string'));
oldmax = get(findobj('tag','ph1_slider'),'max');
oldmin = get(findobj('tag','ph1_slider'),'min');
ph = get(findobj('tag','ph1_slider'),'value');
if newmax < oldmin
    errordlg('Slider maximum must exceed minimum');
    set(gcbo,'string',sprintf('%5.2f',oldmax));
    return
end
if ph > newmax ; newmax = ph; end ;
set(gcbo,'string',sprintf('%5.2f',newmax));
set(findobj('tag','ph1_slider'),'max',newmax);

% --- Executes during object creation, after setting all properties.
function ph1_max_CreateFcn(hObject, eventdata, handles)
% hObject    handle to ph1_max (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end


% --- Executes on button press in Abort.
function Abort_Callback(hObject, eventdata, handles)
% hObject    handle to Abort (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
data = handles.data;
data.ph0 = [];
data.ph1 = [];
data.piv = [];
handles.data = data;
guidata(hObject, handles);
uiresume(handles.figure1);
% --- Executes on button press in exit.
function exit_Callback(hObject, eventdata, handles)
% hObject    handle to exit (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
uiresume(handles.figure1);


function ph0_val_Callback(hObject, eventdata, handles)
% hObject    handle to ph0_val (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: get(hObject,'String') returns contents of ph0_val as text
%        str2double(get(hObject,'String')) returns contents of ph0_val as a double
phs = get(gcbo,'string');
ph = str2double(phs);
max = get(findobj('tag','ph0_slider'),'max');
min = get(findobj('tag','ph0_slider'),'min');
if ph > max 
    set(findobj('tag','ph0_slider'),'max',ph);
    set(findobj('tag','ph0_max_box') ,'string',phs);
end
if ph < min 
    set(findobj('tag','ph0_slider'),'min',ph);
    set(findobj('tag','ph0_min_box'),'string',phs);
end 
set(findobj('tag','ph0_slider'),'value',ph);
data = handles.data;
data.ph0 = ph;
data.piv
phase = pi*ph/180 + data.ph1*data.pts;
handles.data = data ;
guidata(hObject, handles);
corr = real(data.amp.*exp(i*phase));
set(handles.hline,'Ydata',corr);
drawnow


% --- Executes during object creation, after setting all properties.
function ph0_val_CreateFcn(hObject, eventdata, handles)
% hObject    handle to ph0_val (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end



function ph1_val_Callback(hObject, eventdata, handles)
% hObject    handle to ph1_val (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: get(hObject,'String') returns contents of ph1_val as text
%        str2double(get(hObject,'String')) returns contents of ph1_val as a double
phs = get(gcbo,'string');
ph = str2double(phs);
max = get(findobj('tag','ph1_slider'),'max');
min = get(findobj('tag','ph1_slider'),'min');
if ph > max 
    set(findobj('tag','ph1_slider'),'max',ph);
    set(findobj('tag','ph1_max_box') ,'string',phs);
end
if ph < min 
    set(findobj('tag','ph1_slider'),'min',ph);
    set(findobj('tag','ph1_min_box'),'string',phs);
end 
set(findobj('tag','ph1_slider'),'value',ph);
data = handles.data;
data.ph1 = ph;
phase = pi*data.ph0/180 + ph*data.pts;
handles.data = data ;
guidata(hObject, handles);
corr = real(data.amp.*exp(i*phase));
set(handles.hline,'Ydata',corr); 
drawnow

% --- Executes during object creation, after setting all properties.
function ph1_val_CreateFcn(hObject, eventdata, handles)
% hObject    handle to ph1_val (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end
function ManAxMode(h)
    % Do not set CameraViewAngleMode, DataAspectRatioMode,
    % and PlotBoxAspectRatioMode to aviod exposing a bug
    pn = {'ALimMode',...
        'CameraPositionMode',...
        'CLimMode',...
        'TickDirMode','XLimMode',...
        'YLimMode','ZLimMode',...
        'XTickMode','YTickMode',...
        'ZTickMode','XTickLabelMode',...
        'YTickLabelMode','ZTickLabelMode'};
    for k = 1:13
        pv(k) = {'manual'};
    end
    set(h,pn,pv)



% --- Executes on button press in pivot.
function pivot_Callback(hObject, eventdata, handles)
% hObject    handle to pivot (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hint: get(hObject,'Value') returns toggle state of pivot
data = handles.data;
[x,y]=ginput(1);
piv = find( abs(data.fax - x) < 0.1)
if isempty(piv) 
    piv = data.piv;
else
    data.piv = fix(piv(1));
end
data.pts = pi*((0:length(data.amp)-1)'-data.piv)/(180*length(data.amp));
handles.data = data;
guidata(hObject, handles);

