function FID = CQ2CTstatic(param)
%
% param.fid = CQ2CTstatic(param)
%
% Line shape with site frequencies defined by Charpentiers'
% expressions for second order quadrupole contribution to central
% transition
%
% includes delta-iso, and CSA
% stationary sample; no MAS
%
% functions called by CQ2CTstatic
%           gettiles.m         -- get tiling angles for powder average
%           mergeangle_y.m     -- used here to rotate CSA PAS into EFG PAS frame, and EFG from PAS through MOL to LAB frame
%           charpentier.m      -- expressions from Charpentier's thesis for CT site frequencies due to 2nd order CQ
%           wigner.m           -- used by  to compute Wigner 2nd rank rotation matrices for CSA terms

% other necessary functions: (none)
%       
oneangle = param.oneangle;
oneanglecsa = param.oneanglecsa;
switch param.runflags(1) % check for user-modified one-frame K matrix
    case 0 % there is no user-modified, custom one-frame K-matrix
        kmatrix = param.onek; 
    case 1 % use the user-modified one-frame K-matrix
        kmatrix = param.custom_onek; % 
    otherwise
        kmatrix = param.onek;
end

cq = 2000*pi*param.cq; % CQ in rad/sec
eta = param.eta;
NFID = param.nfid;
wl = 2*pi*1.e6*param.w0;          % larmor frequency in rad/sec
dwell = param.time*1e-6;
siten = param.sites_total;
wts = param.wts(:);             % column vector of relative weights
magzero = sqrt(wts); 
R2 = 2*pi/(NFID*dwell); % helps reduce propagation errors
deltasig = 2*pi*param.w0*2*param.dsigcsa/3;
etasig = deltasig.*param.etacsa/sqrt(6);
diso = 2*pi*param.w0*param.diso;
%get angle pairs for powder average, ZCW tiling
Pinfo = gettiles(param.n,param.tiletype);
NAVG = size(Pinfo,1);
% get one-frame angles for CSA tensor in CQ PAS frame
for kk = 1:siten
    rcsa = oneanglecsa(kk,:);  % rotation CSA PAS ==> CQ PAS
    rcq = oneangle(kk,:);      % rotation CQ PAS ==> XTAL
    oneanglecsa(kk,:) = mergeangle_y(rcsa,rcq);
end

%kmatrix = kmatrix  ;%              - R2*eye(size(kmatrix));
% powder loop starts here
FID = zeros(NFID,1);
for powIndex=1:NAVG
    freqMatrix = zeros(siten);
    for k=1:siten
        PAS = [etasig(k) 0 deltasig(k) 0 etasig(k)];
        LAB = PAS * wigner(oneangle(k,:)) * wigner(Pinfo(powIndex,1:3));
        rot3 = mergeangle_y(oneangle(k,:),Pinfo(powIndex,1:3));
        freqMatrix(k,k) = diso(k)+LAB(1,3);% Chemical shift terms
        freqMatrix(k,k) = freqMatrix(k,k)+charpentier(cq(k),eta(k),wl,param.spin,rot3)+i*R2;
    end
    propA = i*freqMatrix + kmatrix ;
    %ratio = norm(kmatrix)/norm(freqMatrix);
    Adwell= expm(propA*dwell);
    magz  = magzero;
    FID(1) = FID(1) + magzero'*magz*Pinfo(powIndex,4); % time origin is just after the pulse
    for timeIndex=2:NFID-1
        magz = Adwell * magz;
        FID(timeIndex) = FID(timeIndex) + magzero'*magz * Pinfo(powIndex,4);
    end
end
%ratio = ratio/NAVG