function FID=CQ2CTvas(param)
%FID=CQCTvas(param)
%
% simulates central transition line shape for 1-Pulse MAS/OMAS including 
% 2nd order quadrupole + anisotropic chemical shift
% see the file express.m for parameter definitions
%
% functions called by CQ2CTvas
%           gettiles.m          -- get tiling angles for powder average (note use of 3-angle tiling!
%           CQCT_wcon.m         -- used in computing site frequencies in banded Floquet matrix
%           makeAmatrix2        -- combines rate & site frequencies to make a 5-band (Floquet) propagator matrix A
%           mergeangle_y.m      -- rotates CSA PAS into EFG PAS axes
%
% other necessary functions
%       DSIM_wigner.m           -- used by CQCT_wcon to compute Wigner 2nd rank rotation matrices
%       D4.m                    -- used by CQCT_wcon to compute Wigner 4th rank rotation matrices
oneangle = param.oneangle;
oneanglecsa = param.oneanglecsa;
switch param.runflags(1) % check for user-modified one-frame K matrix
    case 0 % there is no user-modified, custom one-frame K-matrix
        kmatrx = param.onek; 
    case 1 % use the user-modified one-frame K-matrix
        kmatrx = param.custom_onek; % 
    otherwise
        kmatrx = param.onek;
end
cq = param.cq';
eta = param.eta';
actualBeta = 180*acos(sqrt(1/3))/pi;
betaR = (param.beta_r+actualBeta)*pi/180; % back to radians...
NFID = param.nfid;
SR = param.sr;
NSIDE = param.pairs;
TIME = param.time*1e-6; % dwell time in seconds
sitenumb = param.sites_total;
oneanglecsa = param.oneanglecsa;
cq = 2000*pi*cq;                % cq to Rad/sec
I = param.spin;
wq = (cq/(2*I*(2*I-1))).^2*(I*(I+1)-3/4)/(2*pi*1.e6*param.w0); % units are rad/sec

Aln = [-0.3*(1+eta.^2/3) -6*(1-eta.^2/3)/7 12*eta/(7*sqrt(6)) 81*(1+eta.^2/18)/70 27*eta/(14*sqrt(10)) 9*eta.^2/(4*sqrt(70))];% note eta = (vxx-vyy)/vzz
%R2 = eye(size(kmatrx))*2*pi*i/(NFID*TIME); % helps reduce propagation errors, broadens spikelets!!
% wh = wq(:)/(2*pi*1000)*Aln(:,1) % uncomment this to print delta(2Q,iso) 
deltasig = 2*pi*param.w0*2*param.dsigcsa/3;
etasig = deltasig.*param.etacsa/sqrt(6);
diso = 2*pi*param.w0*param.diso;
SR=SR*2000*pi;                   % rotation speed in Rad/sec

Pinfo = gettiles(param.n,param.tiletype); NAVG = size(Pinfo,1); % note 3-angle ZCW tiling!
% get one-frame angles for CSA tensor in CQ PAS frame
for kk = 1:sitenumb
    rcsa = oneanglecsa(kk,:);  % rotation CSA PAS ==> CQ PAS
    rcq = oneangle(kk,:);      % rotation CQ PAS ==> XTAL
    oneanglecsa(kk,:) = mergeangle_y(rcsa,rcq);
end
wts = param.wts(:);             % column vector of relaive weights
magzero = sqrt(wts); % use square roots since A has been symmetrized, even when weights are unequal   
%generate the angle pairs for powder average, ZCW tiling

jj = 1:sitenumb;
indx = NSIDE*(2*jj-1) + jj;                 % picks out the C(0) coefficient for each site
c1=zeros(2*sitenumb*NSIDE+sitenumb,1);      % the initial C coefficient vector

for ii=1:sitenumb                           % create the initial C coeffecients
c1(indx(ii)) = magzero(ii);                       
end      
          
FID=zeros(NFID,1);

for Pindex=1:NAVG
    pangles = Pinfo(Pindex,1:3); weight = Pinfo(Pindex,4);     
    wcon=CQCT_wcon(pangles,betaR,oneangle,oneanglecsa,wq,Aln,deltasig,etasig,diso);
    amatrix = makeAmatrix2(wcon,kmatrx,NSIDE,SR);
    dw=expm(i*amatrix*TIME*SR);
    ct=c1;
    powdersignal=magzero'*ct(indx);
    FID(1)=FID(1)+powdersignal*weight;   
    for timeindex=2:NFID                    %begin timeloop
        ct=dw*ct;
        powdersignal=magzero'*ct(indx)*weight;
        FID(timeindex)=FID(timeindex)+powdersignal;
    end
end
