function FID=DSIM(param)
% FID=DSIM(param)
% Deuteron MAS/OMAS simulation routine
% (no chemical shifts)
% see the file express.m for parameter definitions
%
% functions called by DSIM
%           gettiles.m         -- get tiling angles for powder average
%           DSIM_wconstant.m   -- used in computing site frequencies in banded Floquet matrix
%           makeAmatrix        -- combines rate & site frequencies to make (Floquet) propagator matrix A
%
% other necessary functions
%       DSIM_wigner.m          -- used by DSIM_wconstant to compute Wigner 2nd rank rotation matrices
oneangle = param.oneangle;  
switch param.runflags(1) % check for user-modified one-frame K matrix
    case 0 % there is no user-modified, custom one-frame K-matrix
        kmatrx = param.onek; 
    case 1 % use the user-modified one-frame K-matrix
        kmatrx = param.custom_onek; % 
    otherwise
        kmatrx = param.onek;
end
cq = param.cq;
eta = param.eta;
actualBeta = 180*acos(sqrt(1/3))/pi;
beta = (param.beta_r+actualBeta)*pi/180; % back to radians...
NFID = param.nfid;
SR = param.sr;
NSIDE = param.pairs;
TIME = param.time*1e-6;         % dwell time in seconds
sitenumb = param.sites_total;
cq = 2000*pi*cq;                % cq to Rad/sec
SR=SR*2000*pi;                  % rotation speed to Rad/sec
wts = param.wts(:);             % column vector of relative weights
magzero = sqrt(wts);            % use square roots since A has been symmetrized to acount for possibly unequal site populations  
jj = 1:sitenumb;
indx = NSIDE*(2*jj-1) + jj;                 % picks out the C(0) Floquet coefficient for each site
c1=zeros(2*sitenumb*NSIDE+sitenumb,1);      % allocate space for the initial C coefficient vector
for ii=1:sitenumb                           % create the initial C coeffecients
c1(indx(ii)) = magzero(ii);                       
end      
Pinfo = gettiles(param.n,param.tiletype);               % get tile angles for powder average
NAVG = size(Pinfo,1);
FID=zeros(NFID,1);
for Pindex=1:NAVG-1
    phi=Pinfo(Pindex,1);
    theta=Pinfo(Pindex,2);                                                            
    weight=Pinfo(Pindex,4); 
    wconstant=DSIM_wconstant(phi,theta,beta,oneangle,cq,eta);
    amatrix=makeAmatrix(wconstant,kmatrx,NSIDE,SR);
    dw=expm(i*amatrix*TIME*SR);
    ct=c1;
    powdersignal=magzero'*ct(indx);
    FID(1)=FID(1)+powdersignal*weight; 
    for timeindex=2:NFID                    %begin timeloop
        ct=dw*ct;
        powdersignal=magzero'*ct(indx)*weight;
        FID(timeindex)=FID(timeindex)+powdersignal;
    end
end
FID = real(FID(:));
