function FID=PTENSORLi(param)
% FID=PTENSORLi(param)
% Lithium MAS/OMAS simulation routine
% (includes paramagnetic shift tensor as well as CQ)
%
% No dynamics are computed here, 
% see the file express.m for parameter definitions
%
% functions called by PTENSORLi
%           gettiles.m          -- get tiling angles for powder average
%           PTENSORLi_wcon.m    -- used in computing site frequencies for banded Floquet matrix
%           makeAmatrix3        -- combines rate & site frequencies to make (Floquet) propagator matrix A
%
% other necessary functions
%       DSIM_wigner.m          -- used by  PTENSORLi_wcon to compute Wigner 2nd rank rotation matrices


diso = 2*pi*param.w0*param.diso;            % isotropic shift in rad/sec
deltasig = 2*pi*param.w0*2*param.dsigcsa/3; % P-tensor shift anisotropy
etasig = deltasig.*param.etacsa/sqrt(6);    % P-tensor asyymetry parameter
%kmatrx = param.onek;
%oneanglecsa = param.oneanglecsa;
NAVG = param.n;
Pinfo = gettiles(NAVG,param.tiletype);
NAVG = size(Pinfo,1);

actualBeta = 180*acos(sqrt(1/3))/pi;
beta = (param.beta_r+actualBeta)*pi/180; % back to radians...
NFID = param.nfid;

SR = param.sr;
NSIDE = param.pairs;
TIME = param.time*1e-6;
sitenumb = 3;
SR=SR*2000*pi;                      % rotor speed to Rad/sec
wts = param.wts(:);                % column vector of relaive weights
magzero = sqrt(wts); % use square roots since A has been symmetrized, even when weights are unequal   
%generate the angle pairs for powder average, ZCW tiling


jj = 1:sitenumb;
indx = NSIDE*(2*jj-1) + jj;                 % picks out the C(0) coefficient for each site
c1=zeros(2*sitenumb*NSIDE+sitenumb,1);      % the initial C coefficient vector
 
for ii=1:sitenumb                           % create the initial C coeffecients
c1(indx(ii)) = magzero(ii);                       
end      

          
FID=zeros(NFID,1);

for Pindex=1:NAVG-1
    phi=Pinfo(Pindex,1);
    theta=Pinfo(Pindex,2);                                                            
    weight=Pinfo(Pindex,4);     
    wconstant=CSA_wconstant(phi,theta,beta,oneanglecsa,diso,deltasig,etasig);% just like DSIM for 2H, but different tensor parameters
    amatrix=makeAmatrix(wconstant,kmatrx,NSIDE,SR); % just like DSIM_hmatrix
    dw=expm(i*amatrix*TIME*SR);
    ct=c1;
    powdersignal=magzero'*ct(indx);
    FID(1)=FID(1)+powdersignal*weight;
    
    for timeindex=2:NFID                    %begin timeloop
        ct=dw*ct;
        powdersignal=magzero'*ct(indx)*weight;
        FID(timeindex)=FID(timeindex)+powdersignal;
    end
end
FID = FID(:); % make sure to return a (complex) column vector; do NOT zero the imaginary part!!