function FID=T1Qsim(param)
% FID=T1Qsim(param)
% Deuteron T1Q simulation routine: idealized BBJB sequence w/QE detection
% (no chemical shifts)
% (ideal, delta-function pulses)
% see the file express.m for parameter definitions
%
% functions called by T1Qsim
%           gettiles.m         -- get tiling angles for powder average
%           wigner.m           -- computes Wigner 2nd rank rotation matrices
%
% other necessary functions: (none)
%
% this routine is derivd from T1Zsim.m by dropping all reference to J2
% and  then computing the time evolution of populations from R1Q equations.
% Only ONE transition is calculated here, the (idealized) BBJB spectrum
% is constructed AFTER the Fourier transform by inverting the sign of
% negative frequency components, and 
cq = param.cq;
eta = param.eta;
NFID = param.nfid;
dwell = param.time*1e-6;
tau1 = param.pulse*1e-6;
wts = param.wts(:); % column vector of relative weights
siten = param.sites_total;
switch param.runflags(1) % check for user-modified one-frame K matrix
    case 0 % there is no user-modified, custom one-frame K-matrix
        kmatrix = param.onek; 
    case 1 % use the user-modified one-frame K-matrix
        kmatrix = param.custom_onek; % 
    otherwise
        kmatrix = param.onek;
end
siteAngle = param.oneangle;
tau = param.taulist; % relaxation delay list, in seconds
ntau = fix(length(tau));
cq = 2*pi*1000*cq;
wl = 2*pi*param.w0*1.e6 ; % Larmor frequency in rad/sec
eta = eta / sqrt(6); % Divide by sqrt(6) here instead of in the powder loop
%
% add artificial damping by 2x(digital resolution) to avoid roundoff
% accumulation in recursive FID calculation
%
R2 = 2*pi/(NFID*dwell);
Pinfo = gettiles(param.n,param.tiletype);
NAVG = size(Pinfo,1);


% compute factors in spectral densities that are INDEPENDNT of powder angles
% These are JKK'(Mw0) in the xtal fixed frame
[vec,kval] = eig(kmatrix);
lam = -diag(kval);
rat1 = lam./(lam.^2+wl^2);

PR2K = zeros(siten,5);
for jj = 1:siten
    PAS = [eta(jj) 0 1 0 eta(jj)]*cq(jj)*sqrt(3/8);
    D2 = wigner(siteAngle(jj,:));
    for k = 1:5
        PR2K(jj,k) = PAS*D2(:,k)*sqrt(wts(jj));
    end
end
J1KKP = zeros(5);

for b = 1:siten
    if abs(lam(b)) > 0.01 % omit small eigenvalues of K
        for k = 1:5
            for kp = 1:5
                tmp = 0;
                for ii = 1:siten
                    for jj = 1:siten
                        tmp = tmp + PR2K(ii,k)*vec(ii,b)*vec(jj,b)*conj(PR2K(jj,kp));
                    end
                end
                J1KKP(k,kp)=J1KKP(k,kp)+rat1(b)*tmp;
                
            end
        end
    end
end

magzero = sqrt(wts)*ones(1,ntau); 

FID = zeros(NFID, ntau);
% FID2 = FID;
for powIndex=1:NAVG
    D2 = wigner(Pinfo(powIndex,1:3));
    J1 = 0;
    for k = 1:5
        for kp = 1:5
            J1 = J1 + J1KKP(k,kp)*D2(k,4)*conj(D2(kp,4));
        end
    end
    freqMatrix = zeros(siten);
    for k=1:siten
        siteCq  =  cq(k) * 3 / 4;
        siteEta = eta(k);
        PAS = [siteEta 0 1 0 siteEta] * siteCq;
        LAB = PAS * wigner(siteAngle(k,:)) * D2;
        freqMatrix(k,k) = LAB(1,3);
    end
    propA = i*freqMatrix + kmatrix - R2*eye(size(kmatrix));
    Adwell= expm(propA*dwell);
    Atau1 = expm(conj(propA)*tau1);
    for jj = 1:ntau
        magz  = Atau1*magzero(:,jj)*exp(-tau(jj)*real(3*J1));
        FID(1,jj) = FID(1,jj) + magzero(:,jj)'*magz*Pinfo(powIndex,4); % time origin is just after second pulse
        for timeIndex=2:NFID-1
            magz = Adwell*magz;
            FID(timeIndex,jj) = FID(timeIndex,jj) + magzero(:,jj)'*magz*Pinfo(powIndex,4);
        end
    end
end
