function FID=spikelet(param)
% FID=spikelet(param)
% Deuteron QCPMG (spikelet echo) simulation routine
% (no chemical shifts)
% see the file express.m for parameter definitions
%
% functions called by spikelet
%           gettiles.m         -- get tiling angles for powder average
%           wigner.m           -- computes 2nd rank wigner rotation matrices
%
% other necessary functions: (none)

cq = param.cq;
eta = param.eta;
switch param.runflags(1) % check for user-modified one-frame K matrix
    case 0 % there is no user-modified, custom one-frame K-matrix
        kmatrix = param.onek; 
    case 1 % use the user-modified one-frame K-matrix
        kmatrix = param.custom_onek; % 
    otherwise
        kmatrix = param.onek;
end
siteAngle = param.oneangle; % these are for EFG tensors, not CSA
nppe = param.nppe;
nppe = floor(nppe);
nechoes = param.nechoes-1;
dwell = param.time*1e-6;
tau = param.pulse*1.e-6; % pulse spacing (seconds)
dink = 2*tau-dwell*floor(nppe);
wts = param.wts(:); % column vector of relaive weights
siten = param.sites_total;
edink = eye(siten);
cq = 2*pi*1000*cq;
eta = eta / sqrt(6); 
%
% Divide by sqrt(6) here instead of in the powder loop
% add artificial damping by 2x(digital resolution) to avoid roundoff
% accumulation in recursive FID calculation
% R2 = 2*pi/(nppe*nechoes*dwell);
R2 = 0;
Pinfo = gettiles(param.n,param.tiletype);
NAVG = size(Pinfo,1);

magzero = sqrt(wts);                 % sqrt used here because K-matrix has been symmetrized
FID = zeros(1, param.nfid);
FID = zeros(1,2^nextpow2(FID));
%FID = [FID 0]; % need an extra point since t = 0 corresponds to FID(1)
for powIndex=1:NAVG
    xtal_wt = Pinfo(powIndex,4);
    freqMatrix = zeros(siten);
    for k=1:siten
        siteCq  =  cq(k) * 3 / 4;
        siteEta = eta(k);
        PAS = [siteEta 0 1 0 siteEta] * siteCq;
        LAB = PAS * wigner(siteAngle(k,:))*wigner(Pinfo(powIndex,1:3));
        freqMatrix(k,k) = LAB(1,3);
    end
    propA = i*freqMatrix + kmatrix; - R2*eye(size(kmatrix));
    Adwell= expm(propA*dwell);
    Atau1 = expm(conj(propA*tau)); % first point is defined as t = 0; 
    magz  = Atau1*magzero;
    if dink > 0.001*dwell
        edink = expm(propA*dink);
    end
    FID(1) =  magzero'*magz*xtal_wt; % time origin is just after second pulse
    kk = 0;
    % first echo period
    for timeIndex=1:nppe
        magz = Adwell * magz; 
        kk = kk+1;
        FID(timeIndex) = FID(timeIndex) + magzero'*magz*xtal_wt;   
    end
    magz = edink*magz ; % propagate to end of 2*tau interval if nppe is fractional
    % subsequent echo periods
    for echonum = 2:nechoes
        magz = conj(magz);
        for timeIndex = 1:nppe
            magz = Adwell * magz;
            FID(kk+timeIndex)= FID(kk+timeIndex)+ magzero'*magz *xtal_wt;
        end
        kk = kk+nppe;
        magz = edink*magz;
    end
    % really only nechoes-1 echoes, propagate FID after last echo to allow decay
    for timeIndex = 1:nppe
            magz = Adwell * magz;
            FID(kk+timeIndex)= FID(kk+timeIndex)+ magzero'*magz *xtal_wt;
        end
    
end
FID = FID(:) ;% crucial!! This ensures a column vector, required for proper post-processing
FID = real(FID) ;% this symmetrization works only when chemical shifts are all zero


