function FID=staticsim(param)
% FID=staticsim(param)
% Deuteron quadrupole echo simulation routine
% (no chemical shifts)
% see the file express.m for parameter definitions
%
% functions called by staticsim
%           gettiles.m         -- get tiling angles for powder average
%           wigner.m           -- computes Wigner 2nd rank rotation matrices
%
% other necessary functions: (none)

cq = param.cq;
eta = param.eta;
switch param.runflags(1) % check for user-modified one-frame K matrix
    case 0 % there is no user-modified, custom one-frame K-matrix
        kmatrix = param.onek; 
    case 1 % use the user-modified one-frame K-matrix
        kmatrix = param.custom_onek; % 
    otherwise
        kmatrix = param.onek;
end
siteAngle = param.oneangle; % these are for EFG tensors, not CSA
NFID = param.nfid;
dwell = param.time*1e-6;
tau1 = param.pulse*1e-6;
wts = param.wts(:); % column vector of relative weights
siten = param.sites_total;
cq = 2*pi*1000*cq;
eta = eta / sqrt(6); % Divide by sqrt(6) here instead of in the powder loop
%
% add artificial damping by 2x(digital resolution) to avoid roundoff
% accumulation in recursive FID calculation -- delete the 0* to activate
%
R2 = 0*2*pi/(NFID*dwell);

Pinfo = gettiles(param.n,param.tiletype);
NAVG = size(Pinfo,1);
     
magzero = sqrt(wts);                 % sqrt used here because K-matrix has been symmetrized
FID = zeros(1, NFID);
for powIndex=1:NAVG
    freqMatrix = zeros(siten);
    for k=1:siten
        siteCq  =  cq(k) * 3 / 4;
        siteEta = eta(k);
        PAS = [siteEta 0 1 0 siteEta] * siteCq;
        LAB = PAS * wigner(siteAngle(k,:)) * wigner(Pinfo(powIndex,1:3));
        freqMatrix(k,k) = LAB(1,3);
    end
    propA = i*freqMatrix + kmatrix - R2*eye(size(kmatrix));
    Adwell= expm(propA*dwell);
    Atau1 = expm(conj(propA*tau1));
    magz  = Atau1*magzero;
    FID(1) = FID(1) + magzero'*magz*Pinfo(powIndex,4); % time origin is just after second pulse
    for timeIndex=2:NFID-1
        magz = Adwell * magz;
        FID(timeIndex) = FID(timeIndex) + magzero'*magz * Pinfo(powIndex,4);
    end
end
FID = FID(:) ;% return a column vector, for compatibility with multi-fid simulation routines
FID = real(FID); % this symmetrization of the FT spectrum works ONLY for zero chemical shift!!