function FID=staticsimcsa(param)
%        FID=staticsimcsa(param)
%
% simulates 1-Pulse-FID powder pattern for N-site jumps
% see express.m for parameter definitions
%
% no CQ in this routine, only CSA
%
% functions called by staticsimcsa
%           gettiles.m         -- get tiling angles for powder average
%           wigner.m           -- computes Wigner 2nd rank rotation matrices
%
% other necessary functions: (none)

siteAngle = param.oneanglecsa;
switch param.runflags(1) % check for user-modified one-frame K matrix
    case 0 % there is no user-modified, custom one-frame K-matrix
        kmatrix = param.onek; 
    case 1 % use the user-modified one-frame K-matrix
        kmatrix = param.custom_onek; % 
    otherwise
        kmatrix = param.onek;
end
diso = 2*pi*param.w0*param.diso; % isotropic shift in rad/sec
% units are rad/sec
deltasig = 2*pi*param.w0*2*param.dsigcsa/3;
etasig = deltasig.*param.etacsa/sqrt(6);
NFID = param.nfid;
dwell = param.time*1e-6;
%tau1 = param.pulse*1e-6;
wts = param.wts(:); % column vector of relaive weights
siten = param.sites_total;
%
% add artificial damping by 2x(digital resolution) to avoid roundoff
% accumulation in recursive FID calculation
%
R2 = 2*pi/(NFID*dwell);
Pinfo = gettiles(param.n,param.tiletype);
NAVG = size(Pinfo,1);
magzero = sqrt(wts);                 % sqrt used here because K-matrix has been symmetrized
FID = zeros(1, NFID);
for powIndex=1:NAVG
    freqMatrix = zeros(siten);
    for k=1:siten
        PAS = [etasig(k) 0 deltasig(k) 0 etasig(k)];
        LAB = PAS * wigner(siteAngle(k,:)) * wigner(Pinfo(powIndex,1:3));
        freqMatrix(k,k) = diso(k)+LAB(1,3);
    end
    propA = i*freqMatrix + kmatrix - R2*eye(size(kmatrix));
    Adwell= expm(propA*dwell);
    %Atau1 = expm(conj(propA*tau1));
    %magz  = Atau1*magzero;
    magz  = magzero;
    FID(1) = FID(1) + magzero'*magz*Pinfo(powIndex,4); % time origin is just after second pulse
    for timeIndex=2:NFID-1
        magz = Adwell * magz;
        FID(timeIndex) = FID(timeIndex) + magzero'*magz * Pinfo(powIndex,4);
    end
end
FID = FID(:) ;% return a column vector, for compatibility with multi-fid simulation routines
