function wcon = CQCT_wcon(pangles,betaR,oneangle,oneanglecsa,wq,Aln,deltasig,etasig,diso)
%        wcon = CQCT_wcon(pangles,betaR,oneangle,oneanglecsa,wq,Aln,deltasig,etasig,diso);
% This one is for 2nd order CQ (arbitrary I)
% also includes CSA
%
% pangles = [ax bx gx] = all three powder angles
% betaR = rotor axis angle
% oneangle = equivalent one-frame site angles
% wq = [Cq/(2I(2I-1)]^2/w0 * (I(I+1)-3/4) in rad /sec for all sites
% Aln = 4th rank functions of eta, see CQ2CTvas.m for implementation and
% JMR 95,509 (1991) for definitions
% Aln = [A00 A20 A22=A2m2 A40 A42 = A4m2 A44 = A4m4]
%
% wcon(1 2 3 4 5 6 7 8 9) = w(4 3 2 1 0 -1 -2 -3 -4)
%
nsites=size(oneangle,1);
wcon=zeros(nsites,9);
D2 = DSIM_wigner(pangles(1),pangles(2),pangles(3));
D4th = D4(pangles(1),pangles(2),pangles(3));
cbr = cos(betaR) ; sbr = sin(betaR);
d200 = 0.5*(3*cbr^2-1); d202 = sqrt(6)/4*sbr*sbr; d210 = -sqrt(3/2)*sbr*cbr;
D4b = D4(0,betaR,0);

for ii = 1:nsites
    wcon(ii,5) = diso(ii) +wq(ii)*Aln(ii,1) + 0*i; % l = 0 term
    a=oneangle(ii,1); 
    b=oneangle(ii,2); 
    g=oneangle(ii,3); 
    D2c = DSIM_wigner(oneanglecsa(ii,1),oneanglecsa(ii,2),oneanglecsa(ii,3));
    D2q = DSIM_wigner(a,b,g);
    t0 = deltasig(ii); t2 = etasig(ii);
    V = [t2 0 t0 0 t2]*D2c*D2q; % CSA tensor components in rotor-fixed frame
    t3=d202*V*conj(D2(:,1)); % CSA contributions
    t4=-d210*V*conj(D2(:,2)); % CSA contributions -- CHECK THIS sign !!
    t5=  d200*V*conj(D2(:,3));% CSA contributions
    D2ii = D2q*D2;
    
    t5 = t5 + wq(ii)*d200*(Aln(ii,2)*D2ii(3,3)+Aln(ii,3)*(D2ii(5,3)+D2ii(1,3))); % l = 2 CQ term, m = 0
    t4 = t4 + wq(ii)*d210*(Aln(ii,2)*D2ii(3,2)+Aln(ii,3)*(D2ii(5,2)+D2ii(1,2)));  % l = 2 CQ term, m = 1
    t3 = t3 + wq(ii)*d202*(Aln(ii,2)*D2ii(3,1)+Aln(ii,3)*(D2ii(5,1)+D2ii(1,1)));  % l = 2 CQ term, m = 2
    D4i = D4(a,b,g)*D4th;
    
    t1 = wq(ii)*D4b(1,5)*(Aln(ii,4)*D4i(5,1)+Aln(ii,5)*(D4i(3,1)+D4i(7,1))+Aln(ii,6)*(D4i(9,1)+D4i(1,1))); %l = 4, m=4 CQ term
    t2 = wq(ii)*D4b(2,5)*(Aln(ii,4)*D4i(5,2)+Aln(ii,5)*(D4i(8,2)+D4i(2,2))+Aln(ii,6)*(D4i(1,2)+D4i(9,2))); %l = 4, m=3 CQ term
    t3 = t3 + wq(ii)*D4b(3,5)*(Aln(ii,4)*D4i(5,3)+Aln(ii,5)*(D4i(3,3)+D4i(7,3))+Aln(ii,6)*(D4i(1,3)+D4i(9,3))); % add the l = 4, m=2 CQ term
    t4 = t4+ wq(ii)*D4b(1,5)*(Aln(ii,4)*D4i(5,1)+Aln(ii,5)*(D4i(3,4)+D4i(7,4))+Aln(ii,6)*(D4i(1,4)+D4i(9,4))); %add the l = 4, m=1 CQ term
    t5 = t5+ wq(ii)*D4b(5,5)*(Aln(ii,4)*D4i(5,5)+Aln(ii,5)*(D4i(3,5)+D4i(7,5))+Aln(ii,6)*(D4i(1,5)+D4i(9,5))); %l = 4, m=0 CQ term
    wcon(ii,1) = t1 ; wcon(ii,2) = t2 ; wcon(ii,3) = t3 ; wcon(ii,4) = t4 ; wcon(ii,5) = wcon(ii,5) + t5;
    wcon(ii,7) = conj(t3) ; wcon(ii,8) = conj(t2) ; wcon(ii,9) = conj(t1); wcon(ii,6) = conj(t4);
end


