function wconst=CSA_wconstant(phi,theta,beta,siteangles,diso,T0,T2)
%wconst=CSA_wconstant(phi,theta,beta,siteangles,diso,span,skew))
% 
% phi,theta are powder average angles (relative to rotor axis!) in radians
% beta (Radians) is rotor axis agle wrt. B0
% siteangles define PAS of site-dependent csa tensors
% T0 = 2/3(sig33-0.5*(sig11+sig22)) ; site dependent csa tesor principal values (Radians/sec)
% T2 = (sig11-sig22)/sqrt(6) ; radians/sec
% RLV 



sitenumb=size(siteangles,1);
wconst=zeros(sitenumb,5);
Dpt=DSIM_wigner(phi,theta,0);
Db=DSIM_wigner(0,beta,0);

%  D  2  1  0 -1 -2
%  2  a  b  c  d  e
%  1 -b  f  g  h  d
%  0  c -g  k  g  c
% -1 -d  h -g  f  b
% -2  e -d  c -b  a

for ii=1:sitenumb
                                                          
a=siteangles(ii,1);
b=siteangles(ii,2);
g=siteangles(ii,3);   
sb=sin(b);
cb=cos(b);
t0 = T0(ii); t2 = T2(ii); 
%A-->E are CSA tensor components in rotor-fixed frame
A=(3/2*cb*cb-1/2)*t0+(exp(-i*2*a)+exp(i*2*a))*sqrt(3/8)*sb*sb*t2;
B=sqrt(3/2)*sb*cb*exp(-i*g)*t0+exp(-i*2*a)*(-(1+cb)/2*sb)*exp(-i*g)*t2+exp(i*2*a)*((1-cb)/2*sb)*exp(-i*g)*t2;
%C=-sqrt(3/2)*sb*cb*exp(i*g)*t0+exp(-i*2*a)*(-(1-cb)/2*sb)*exp(i*g)*t2+exp(i*2*a)*((1+cb)/2*sb)*exp(i*g)*t2;
C = -conj(B);
D=sqrt(3/8)*sb*sb*exp(-2*i*g)*t0+exp(-i*2*a)*((1+cb)/2)^2*exp(-2*i*g)*t2+exp(i*2*a)*((1-cb)/2)^2*exp(-2*i*g)*t2;
%E=sqrt(3/8)*sb*sb*exp(2*i*g)*t0+exp(-i*2*a)*((1-cb)/2)^2*exp(2*i*g)*t2+exp(i*2*a)*((1+cb)/2)^2*exp(2*i*g)*t2;
E = conj(D);

% w(1 2 3 4 5)= w(2 1 0 -1 -2)
V = [D B A C E];
wconst(ii,1)=Db(1,3)*V*conj(Dpt(:,1));
wconst(ii,2)=Db(2,3)*V*conj(Dpt(:,2));
wconst(ii,3)=diso(ii) + Db(3,3)*V*conj(Dpt(:,3));
%wconst(ii,4)=Db(4,3)*V*Dpt(:,4);
%wconst(ii,5)=Db(5,3)*V*Dpt(:,5);
wconst(ii,4)= conj(wconst(ii,2)) ;
wconst(ii,5) = conj(wconst(ii,1));

end


