function freq = charpentier(cq,eta,wl,I, rot)


% freq = charpentier(cq,eta,deltasig,etasig,rsig,wl,I, rot)
%
% returns central transition frequency for 2nd order quadrupole coupling, where:
%
% cq = quadrupole coupling constant in rad/sec
% eta = asymmetry parameter
% wl = Larmor frequency in Rad/sec
% I = nuclear spin  (must be half-integral, greater than 1)
% rot = [alpha,beta,0] (units are radians) is the Euler rotation from CQ PAS to LAB fixed axes
%
% Expressions taken from Charpentier's Thesis, p24
%
etasq = eta*eta;
b00 = -0.3*(1+etasq/3);
b20 = (etasq-3)/14;
b40 = (etasq+18)/140;
b22 = sqrt(3/2)*eta/7;
b42 = 3*eta/(14*sqrt(10));
b44 = 0.25*etasq/sqrt(70);
cb = cos(rot(2)) ; sb = sin(rot(2));
c2b = cos(2*rot(2)); c4b = cos(4*rot(2));
c2a = cos(2*rot(1)); c4a = cos(4*rot(1));
ip1 = I*(I+1);
wq = cq/(2*I*(2*I-1));
prefac = 0.5*wq*wq/wl;
d440 = sqrt(70)*sb^4/16;
d420 = sqrt(10)*(3+4*c2b-7*c4b)/64;
d400 = (9+20*c2b+35*c4b)/64;
d220 = sqrt(3/8)*sb*sb;
d200 = 0.5*(3*cb*cb-1);
g2 = b22*d220*2*c2a + b20*d200;
g4 = b44*d440*2*c4a + b42*d420*2*c2a + b40*d400;
freq = prefac*(2*b00*(ip1-0.75) + (8*ip1-6)*g2 + (18*ip1-27/2)*g4);
%wh = prefac*(2*b00*(ip1-0.75))/(2000*pi) % uncomment to print delta(CQ2 iso)