function k=creatematrix(n,model,rate,handles)
% k=creatematrix(n,model,rate);
% 
% returns a rate matrix with connectivity set by user's choice among preset
% patterns, or by a user-editable custom template
% 
% connectivity pattern is just ones in appropriate places
% this routine multiplies the ones by the jump rate for this frame, and 
% makes sure the diagonal elements are the negative sum of off diagonal
% elements in the same column. It returns a symmetric template; in case of
% unequal site populations the specified rate is the arithmetic mean of the
% forward and backward rate constants.
%
% creatematrix is also used to build a custom template.
switch lower(model)
    case{'all'}
        k=ones(n);
        k=k-diag(sum(k));
        k=k*rate;
    case{'lin' 'cha'}
        k=zeros(n);
        k=k+diag(ones(n-1,1),-1)+diag(ones(n-1,1),1);
        k=k-diag(sum(k));
        k=k*rate;
    case{'cir'}
        k=zeros(n);
        k(1,n)=1;
        k(n,1)=1;
        k=k+diag(ones(n-1,1),-1)+diag(ones(n-1,1),1);
        k=k-diag(sum(k));
        k=k*rate;
    otherwise % roll your own rate template
        param = handles.param;
        scrsz = get(0,'ScreenSize');
        scr = figure('Position',[1 scrsz(4)/2 0.75*scrsz(3) scrsz(4)/2],...
            'Name','One-frame Rate Matrix','NumberTitle','off');
        hax =axes('units','pixels','position',[ 10 10  800 450]);
        data = zeros(n);
        wts = param.wts;
        clab = cell(n,1);
        for jj = 1:n
            clab{jj} = num2str(jj);
        end
        P = struct('modal',1, 'nVisibleRows',10 );
        P.nVisibleCols = 10;
        P.highlightCell = 0;
        P.colLabels = clab;
        P.precision = '%.7g';
        P.fontsize = 11;
        k_new = tedit(hax,data,P);
        if isempty(k_new) % user pressed cancel button; so use 'all' template
            k=ones(n);
            k=k-diag(sum(k));
            k=k*rate;
            return
        else % we SUPPOSE the user knows what he/she is doing....but check detailed balance anyway
            k = rate*k_new;
            for row  = 2:n
                for col = 1:row-1
                    k_new(row,col) = k_new(col,row);
                end
            end
            for jj = 1:n
                k_new(jj,jj) = 0;
            end
            k_new=(k_new-diag(sum(k_new)));
            k = rate*k_new;
        end
end

