function Pinfo = gettiles(n,type)
% 
%   Pinfo = gettiles(n,type')
%
%   returns powder angles & weights,    Pinfo(:,1) = alpha
%                                       Pinfo(:,2) = beta
%                                       Pinfo(:,3) = gamma
%                                       Pinfo(:,4) = weight
% type = 'zcw' <== simple ZCW on a sphere
%      = 'shrewd" <== spherical, 2-angle ZCW with gaussian weighting
%      = 'msh' <== optimized ZCW for hemisphere
%      = 'z3' <== ZCW 3-angle sets 
%      = 'karney' <== 3-angle sets derived from quaternions
% 
% n = requested number of beta-angles. The actual number
%     is the closet legal value greater than or equal to
%     the requested value.
%
%     NOTE: when n is 13 or smaller, 'type' is ignored and
%           the angles are computed directly for ZCW on a sphere


if n <=4
    Pinfo = [0 0 0 1]; % just do one increment
    return
else
    switch lower(type)
        case 'zcw'
            F=[1 1];
            while sum(F)<n
                F=[F(2) sum(F)];
            end
            n=sum(F);                    % this is the actual number of tiles
            Pinfo = zeros(n,4);
            for Pindex=0:n-1
                Ptheta=(Pindex+1/2)*pi/n;
                Pphi=mod(Pindex*F(1),n)*2*pi/n;
                Pweight=sin(Ptheta);
                Pinfo(Pindex+1,:)=[Pphi Ptheta 0 Pweight]; % this varible stores the powder angles and signal weight information.
            end
            return
        case 'msh'
            mask = [66 120 190 276 378 496 630 780 946 1128 1326 3321 5151];
            m = mask(find(mask>n,1));
            if isempty(m)
                m = 66;
            end
            msh = zeros(m,3);
            fname = strcat('msh',num2str(m));
            load (fname)
            Pinfo = [msh(:,1) msh(:,2) zeros(size(msh,1),1) msh(:,3)]; 
            clear msh;
            return
        case 'z3'
            mask = [50 100 144 200 300 538 1154 3722 6044];
            m = mask(find(mask>n,1));
            if isempty(m)
                m = 50;
            end
            fname = strcat('z3_',num2str(m));
            load (fname);
            Pinfo = z3 ; clear z3
            return
        case 'kar'
                mask = [24 60 216 360 648 1992 3768 7416];
            m = mask(find(mask>n,1));
            if isempty(m)
                m = 24;
            end
            fname = strcat('kar_',num2str(m));
            load (fname);
            Pinfo = kar; clear kar ;
            return
        otherwise
            Pinfo = [0 0 0 1];
            return
    end
end