function amat = makeAmatrix(wcon,onek,nside,ws)
% 
% amat = makeAmatrix(wcon,onek,nside,ws)
%
% This routine builds a (truncated) Floquet representation of the
% Liouville operator for n-site jump dynamics
%
% ROWS of wcon(n,m) contain expressions for the "bands" of the site
% frequency matrix, COLUMNS depend on the interaction. See JCP 124,1 (2006)
% for details. units of wcon are rad/sec
%
% onek is the equivalent one-frame rate matrix for (possibly composite)
% n-site jumps. units of onek are s^(-1)
%
% nside is the number of sideband pairs
% ws is the spin rate, in rad/sec (NOT kHz -- be sure to convert before
% calling makeAmatrix)
% 
m = 2*nside+1 ; mm1 = m-1; mm2 = m-2;

    a1 = wcon(1,3)*eye(m)/ws+ diag(-nside:nside)+ ...
        (wcon(1,4)*diag(ones(1,mm1),1)+ wcon(1,2)*diag(ones(1,mm1),-1)+ wcon(1,5)*diag(ones(1,mm2),2)+...
        wcon(1,1)*diag(ones(1,mm2),-2))/ws;
    
nsites = size(onek,1);
for jj = 2:nsites
   a2 = wcon(jj,3)*eye(m)/ws+ diag(-nside:nside)+ ...
        (wcon(jj,4)*diag(ones(1,mm1),1)+ wcon(jj,2)*diag(ones(1,mm1),-1)+ wcon(jj,5)*diag(ones(1,mm2),2)+...
        wcon(jj,1)*diag(ones(1,mm2),-2))/ws; 
    a1 = blkdiag(a1,a2);
end
amat = -i*kron(onek/ws , eye(m)) + a1;


