function new_a=mergeangle_y(a1,a2)
% new_a=mergeangle_y(a1,a2)
%
% computes Euler angles [alfa beta gamma]=new_a(a1,a2)
% that define a single rotation R = R2*R1, equivalent to R1[alfa1 beta1 gamma1]
% FOLLOWED by R2[alfa2 beta2 gamma2]
%
% angles in radians, NOT degrees
%
% a=[phi, theta, psi] aka [alfa,beta,gamma] as defined by ROSE
% the rotation is psi around z, then theta around ORIGINAL y, then phi around ORIGINAL z
% this is the same as alfa around z, then beta around INTERMEDIATE y, then
% gamma aound INTERMEDIATE z as discussed for example, by Brink & Satchler
% p20. Note that a positive rotation is defined by the "right hand rule": 
% point your right thumb along the (positive) axis of a rotation, and the
% sense of positive rotation is the direction your fingers curl.
%
al=a1(1);
ca=cos(al);sa=sin(al);
be=a1(2);
cb=cos(be);sb=sin(be);
ga=a1(3);
cg=cos(ga);sg=sin(ga);

D1=[ cg -sg 0;
    sg cg 0;
    0  0 1;];
% note location of minus sign in C1 & C2 !!
C1=[cb   0 sb;
    0    1   0;
    -sb  0 cb];
%C1 = [ 1 0 0;
% 0 ct st ;
%0 -st ct];
B1=[ ca -sa 0;
    sa ca 0;
    0   0 1];
R1=B1*C1*D1;       % R1 = exp(-i*alfa*Iz)*exp(-i*beta*Iy)*exp(-i*gamma*Iz)

al=a2(1);
ca=cos(al);sa=sin(al);
be=a2(2);
cb=cos(be);sb=sin(be);
ga=a2(3);
cg=cos(ga);sg=sin(ga);

D2=[ cg -sg 0;
    sg cg 0;
    0  0 1;];
% note location of minus sign in C1 & C2 !!
C2=[cb   0  sb;
    0    1   0;
    -sb  0 cb];
%C1 = [ 1 0 0;
% 0 ct st ;
%0 -st ct];
B2=[ ca -sa 0;
    sa ca 0;
    0   0 1];
R2=B2*C2*D2;

R=R1*R2;  % note the order: T(new) = Rinverse*T(old)*R = R2inverse*R1inverse*T(old)*R1*R2

beta=acos(R(3,3)); % 0 <= beta <= pi

if R(3,3)~=1 && R(3,3)~=-1 % the general case....
    % 0 <= alfa <= 2*pi
    ca=R(1,3)/sin(beta);
    sa=R(2,3)/sin(beta);
    if abs(ca) < 1.e-5
        ca = 0;
    end
    if abs(sa) < 1.e-5
        sa = 0;
    end
    if ca>=0 && sa>=0 % [0,pi/2)
        alfa=asin(sa);
    elseif ca<=0 && sa>=0 %[pi/2 pi)
        alfa=acos(ca);
    elseif ca<=0 && sa<=0 %[pi 3*pi/2)
        alfa=2*pi-acos(ca);
    elseif sa<=0 && ca >=0 %[3*pi/2 2pi)
        alfa=2*pi+asin(sa);
    end
    % 0 <= gamma <= 2*pi
    cg=-R(3,1)/sin(beta);
    sg=R(3,2)/sin(beta);
    if abs(cg) < 1.e-5
        cg = 0;
    end
    %if abs(sg) < 1.e-5
       % sg = 0.0;
%end
    if cg>=0 && sg>=0 % [0,pi/2)
        gam=asin(sg);
    elseif cg<=0 && sg>=0 %[pi/2 pi)
        gam=acos(cg);
    elseif cg<=0 && sg<=0 %[pi 3*pi/2)
        gam=2*pi-acos(cg);
    elseif cg>=0 && sg <=0 %[3*pi/2 2pi)
        gam=2*pi+asin(sg);
    end
elseif R(3,3)==1 % beta=0 then choose gamma = 0
    gam=0;
    sa=-R(1,2);
    ca=R(1,1);
    if ca>=0 && sa>=0 % [0,pi/2)
        alfa=asin(sa);
    elseif sa>=0 && ca<=0 %[pi/2 pi)
        alfa=acos(ca);
    elseif ca<=0 && sa<=0 %[pi 3*pi/2)
        alfa=2*pi-acos(ca);
    elseif sa<=0 && ca >=0 %[3*pi/2 2pi)
        alfa=2*pi+asin(sa);
    end

else          %theta=pi then alfa = 0
    alfa=0;
    sg=R(2,1);
    cg=R(2,2);
    if cg>=0 && sg>=0 % [0,pi/2]
        gam=asin(sg);
    elseif sg>=0 && cg<=0 %[pi/2 pi)
        gam=acos(cg);
    elseif cg<=0 && sg<=0 %[pi 3*pi/2)
        gam=2*pi-acos(cg);
    elseif sg<=0 && cg >=0 %[3*pi/2 2pi)
        gam=2*pi+asin(sg);
    end
end
% replace angles very close to 2*pi with zero, for cosmetic reasons
if abs(alfa-2*pi) <= 0.00001
    alfa = 0;
end
if abs(gam-2*pi) <= 0.00001
    gam = 0;
end
new_a= real([alfa beta gam]);


    