function [oneangle, Tsite]=oneframeangle(Nsite,Fangle)
% [oneangle Tsite]=oneframeangle(Nsite,Fangle)
% Nsite is a row vector whose elements are number of sites in each frame: [n1 n2 ... ]
% where n1 refers to frame 1, (the PAS frame); n2 to frame 2 (1st intermediate jump axes)
% and the last entry in Nsite is the number of orientations the last intermediate jump
% frame has with respect to molecule fixed reference axes
% Fangle is a sum(Nsite)x3 matrix, each row is a set of Euler angles [alpha, beta, gamma]
% 
% by YYH

%check 
if sum(Nsite)~=size(Fangle,1)
    ERROR('angle info does not match site info');
end
maxframe=length(Nsite);
Tsite=Nsite(1);
oneangle=Fangle(1:Tsite,:);
Bsite=Tsite;

for frame_index=2:maxframe
    oldTsite=Tsite;
    oldoneangle=oneangle;
    Tsite=0;
    
    for jj=1:Nsite(frame_index) % loop at the F2 at first, inter-frame
        for ii=1:oldTsite% then loop at the F1 local-frame, related to K matrix
            Tsite = Tsite+1;
            oneangle(Tsite,:)=mergeangle_y(oldoneangle(ii,:),Fangle(Bsite+jj,:));
        end
    end
    Bsite=Bsite+Nsite(frame_index);
end
oneangle=real(oneangle);
