function [onek,kmatrxcell]=oneframekmatrx(Nsite,Nmodel,Nrate,Ntemplate)
%function [onek, kmatrxcell]=oneframekmatrx(Nsite,Nmodel, Nrate)
%
% Nsite is a row vector [N1 N2 N3 ...] where Nj is the number of jump sites in frame j 
% Nrate is a row vector [k1 k2 ...] where kj is the jump rate for sites WITHIN frame j
% Nmodel character array that specifies site connectivity in each frame,
% eg, ['all'; 'cha';'other';'cir'] for four frames
% all=all; cha=chain or lin; cir=circle; only the first three letters of the model name are used; 
% frame order 1 2 3 ... is the same for Nsite, Nrate and Nmodel
% Ntemplate is a user-specified jump connectrivity matrix , used for any
% frame whose model is 'other'
%
% NOTE: the order in which frames is specified MATTERS: the first one must
% transform tensors in the PAS frame to the first jump frame axes; this is the XTAL fixed frame for a one-frame, static problem 
% by YYH

maxFindex=size(Nrate,2);% # of jump frames 
%check 
if maxFindex~=size(Nmodel,1)
    ERROR('jump model info does not match jump rate info');
end

%generate the Kmatirx;

kmatrxcell=cell(1,maxFindex);
for Findex=1:maxFindex
    Fsize=Nsite(1,Findex);
    Fmodel=Nmodel(Findex,:);
    Frate=Nrate(1,Findex);
    if strcmp(Fmodel(1:3),'oth')
        kmatrxcell{Findex}=Ntemplate{Findex} ;
    else
        kmatrxcell{Findex}=creatematrix(Fsize,Fmodel,Frate);
    end
end
onek=kmatrxcell{maxFindex};
for Findex=maxFindex-1:-1:1
    oldk=onek;
    onek=mergematrix(oldk,kmatrxcell{Findex});
end

